/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "bindings/core/v8/V8ErrorEvent.h"

#include "bindings/core/v8/V8BindingForCore.h"
#include "platform/bindings/V8PrivateProperty.h"

namespace blink {

void V8ErrorEvent::errorAttributeGetterCustom(
    const v8::FunctionCallbackInfo<v8::Value>& info) {
  v8::Isolate* isolate = info.GetIsolate();

  auto private_error = V8PrivateProperty::GetErrorEventError(isolate);
  v8::Local<v8::Value> cached_error =
      private_error.GetOrUndefined(info.Holder());
  if (!cached_error->IsUndefined()) {
    V8SetReturnValue(info, cached_error);
    return;
  }

  ErrorEvent* event = V8ErrorEvent::ToImpl(info.Holder());
  ScriptState* script_state = ScriptState::From(isolate->GetCurrentContext());
  ScriptValue error = event->error(script_state);
  v8::Local<v8::Value> error_value =
      error.IsEmpty() ? v8::Local<v8::Value>(v8::Null(isolate))
                      : error.V8Value();
  private_error.Set(info.Holder(), error_value);
  V8SetReturnValue(info, error_value);
}

}  // namespace blink
