/*
 * Copyright (C) 2007, 2008, 2011 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CSSFontFaceSource_h
#define CSSFontFaceSource_h

#include "base/macros.h"
#include "core/CoreExport.h"
#include "platform/fonts/FontCacheKey.h"
#include "platform/fonts/FontSelectionTypes.h"
#include "platform/heap/Handle.h"
#include "platform/wtf/Allocator.h"
#include "platform/wtf/HashMap.h"
#include "platform/wtf/LinkedHashSet.h"

namespace blink {

class FontDescription;
class SimpleFontData;

enum FontDisplay {
  kFontDisplayAuto,
  kFontDisplayBlock,
  kFontDisplaySwap,
  kFontDisplayFallback,
  kFontDisplayOptional,
  kFontDisplayEnumMax
};

class CORE_EXPORT CSSFontFaceSource
    : public GarbageCollectedFinalized<CSSFontFaceSource> {
 public:
  virtual ~CSSFontFaceSource();

  virtual bool IsLocal() const { return false; }
  virtual bool IsLoading() const { return false; }
  virtual bool IsLoaded() const { return true; }
  virtual bool IsValid() const { return true; }

  scoped_refptr<SimpleFontData> GetFontData(const FontDescription&,
                                            const FontSelectionCapabilities&);

  virtual bool IsLocalFontAvailable(const FontDescription&) { return false; }
  virtual void BeginLoadIfNeeded() {}
  virtual void SetDisplay(FontDisplay) {}

  virtual bool IsInBlockPeriod() const { return false; }
  virtual bool IsInFailurePeriod() const { return false; }

  // For UMA reporting
  virtual bool HadBlankText() { return false; }

  virtual void Trace(blink::Visitor* visitor) {}

 protected:
  CSSFontFaceSource() = default;
  virtual scoped_refptr<SimpleFontData> CreateFontData(
      const FontDescription&,
      const FontSelectionCapabilities&) = 0;
  void PruneTable();

 private:
  void PruneOldestIfNeeded();
  using FontDataTable = HashMap<FontCacheKey,
                                scoped_refptr<SimpleFontData>,
                                FontCacheKeyHash,
                                FontCacheKeyTraits>;
  using FontCacheKeyAgeList =
      LinkedHashSet<FontCacheKey, FontCacheKeyHash, FontCacheKeyTraits>;

  FontDataTable font_data_table_;
  FontCacheKeyAgeList font_cache_key_age;
  DISALLOW_COPY_AND_ASSIGN(CSSFontFaceSource);
};

}  // namespace blink

#endif
