/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SimplifyMarkupCommand_h
#define SimplifyMarkupCommand_h

#include "core/editing/commands/CompositeEditCommand.h"

namespace blink {

class SimplifyMarkupCommand final : public CompositeEditCommand {
 public:
  static SimplifyMarkupCommand* Create(Document& document,
                                       Node* first_node,
                                       Node* node_after_last) {
    return new SimplifyMarkupCommand(document, first_node, node_after_last);
  }

  virtual void Trace(blink::Visitor*);

 private:
  SimplifyMarkupCommand(Document&, Node* first_node, Node* node_after_last);

  void DoApply(EditingState*) override;
  int PruneSubsequentAncestorsToRemove(
      HeapVector<Member<ContainerNode>>& nodes_to_remove,
      size_t start_node_index,
      EditingState*);

  Member<Node> first_node_;
  Member<Node> node_after_last_;
};

}  // namespace blink

#endif  // SimplifyMarkupCommand_h
