/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef V0CustomElementLifecycleCallbacks_h
#define V0CustomElementLifecycleCallbacks_h

#include "platform/heap/Handle.h"
#include "platform/wtf/text/AtomicString.h"

namespace blink {

class Element;

class V0CustomElementLifecycleCallbacks
    : public GarbageCollectedFinalized<V0CustomElementLifecycleCallbacks> {
 public:
  virtual ~V0CustomElementLifecycleCallbacks() = default;

  enum CallbackType {
    kNone = 0,
    kCreatedCallback = 1 << 0,
    kAttachedCallback = 1 << 1,
    kDetachedCallback = 1 << 2,
    kAttributeChangedCallback = 1 << 3
  };

  bool HasCallback(CallbackType type) const { return callback_type_ & type; }

  virtual void Created(Element*) = 0;
  virtual void Attached(Element*) = 0;
  virtual void Detached(Element*) = 0;
  virtual void AttributeChanged(Element*,
                                const AtomicString& name,
                                const AtomicString& old_value,
                                const AtomicString& new_value) = 0;

  virtual void Trace(blink::Visitor* visitor) {}

 protected:
  explicit V0CustomElementLifecycleCallbacks(CallbackType type)
      : callback_type_(type) {}

 private:
  CallbackType callback_type_;
};

}  // namespace blink

#endif  // V0CustomElementLifecycleCallbacks_h
