/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PickerIndicatorElement_h
#define PickerIndicatorElement_h

#include "core/html/HTMLDivElement.h"
#include "core/html/forms/DateTimeChooser.h"
#include "core/html/forms/DateTimeChooserClient.h"

namespace blink {

class HTMLInputElement;

class PickerIndicatorElement final : public HTMLDivElement,
                                     public DateTimeChooserClient {
  USING_GARBAGE_COLLECTED_MIXIN(PickerIndicatorElement);

 public:
  // PickerIndicatorOwner implementer must call removePickerIndicatorOwner when
  // it doesn't handle event, e.g. at destruction.
  class PickerIndicatorOwner : public GarbageCollectedMixin {
   public:
    virtual ~PickerIndicatorOwner() = default;
    virtual bool IsPickerIndicatorOwnerDisabledOrReadOnly() const = 0;
    // FIXME: Remove. Deprecated in favor of double version.
    virtual void PickerIndicatorChooseValue(const String&) = 0;
    virtual void PickerIndicatorChooseValue(double) = 0;
    virtual Element& PickerOwnerElement() const = 0;
    virtual bool SetupDateTimeChooserParameters(DateTimeChooserParameters&) = 0;
  };

  static PickerIndicatorElement* Create(Document&, PickerIndicatorOwner&);
  ~PickerIndicatorElement() override;
  void Trace(blink::Visitor*) override;

  void OpenPopup();
  void ClosePopup();
  bool WillRespondToMouseClickEvents() override;
  void RemovePickerIndicatorOwner() { picker_indicator_owner_ = nullptr; }
  AXObject* PopupRootAXObject() const;

  // DateTimeChooserClient implementation.
  Element& OwnerElement() const override;
  void DidChooseValue(const String&) override;
  void DidChooseValue(double) override;
  void DidEndChooser() override;

 private:
  PickerIndicatorElement(Document&, PickerIndicatorOwner&);
  LayoutObject* CreateLayoutObject(const ComputedStyle&) override;
  void DefaultEventHandler(Event*) override;
  void DetachLayoutTree(const AttachContext& = AttachContext()) override;
  bool IsPickerIndicatorElement() const override;
  InsertionNotificationRequest InsertedInto(ContainerNode*) override;
  void DidNotifySubtreeInsertionsToDocument() override;

  HTMLInputElement* HostInput();

  Member<PickerIndicatorOwner> picker_indicator_owner_;
  Member<DateTimeChooser> chooser_;
};

DEFINE_TYPE_CASTS(PickerIndicatorElement,
                  Element,
                  element,
                  element->IsPickerIndicatorElement(),
                  element.IsPickerIndicatorElement());

}  // namespace blink
#endif
