/*
 * This file is part of the select element layoutObject in WebCore.
 *
 * Copyright (C) 2006, 2007, 2009 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef LayoutListBox_h
#define LayoutListBox_h

#include "core/layout/LayoutBlockFlow.h"

namespace blink {

class HTMLSelectElement;

class LayoutListBox final : public LayoutBlockFlow {
 public:
  explicit LayoutListBox(Element*);
  ~LayoutListBox() override;

  unsigned size() const;

  // Unlike scrollRectToVisible this will not scroll parent boxes.
  void ScrollToRect(const LayoutRect&);

  const char* GetName() const override { return "LayoutListBox"; }

 private:
  HTMLSelectElement* SelectElement() const;

  bool IsOfType(LayoutObjectType type) const override {
    return type == kLayoutObjectListBox || LayoutBlockFlow::IsOfType(type);
  }

  void ComputeLogicalHeight(LayoutUnit logical_height,
                            LayoutUnit logical_top,
                            LogicalExtentComputedValues&) const override;
  void ComputeIntrinsicLogicalWidths(
      LayoutUnit& min_logical_width,
      LayoutUnit& max_logical_width) const override;

  void StopAutoscroll() override;

  LayoutUnit DefaultItemHeight() const;
  LayoutUnit ItemHeight() const;
};

DEFINE_LAYOUT_OBJECT_TYPE_CASTS(LayoutListBox, IsListBox());

}  // namespace blink

#endif  // LayoutListBox_h
