/*
 * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
 *           (C) 2000 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef CursorData_h
#define CursorData_h

#include "core/style/StyleImage.h"
#include "platform/geometry/IntPoint.h"

namespace blink {

class CursorData {
  DISALLOW_NEW_EXCEPT_PLACEMENT_NEW();

 public:
  CursorData(StyleImage* image,
             bool hot_spot_specified,
             const IntPoint& hot_spot)
      : image_(image),
        hot_spot_specified_(hot_spot_specified),
        hot_spot_(hot_spot) {}

  bool operator==(const CursorData& o) const {
    return hot_spot_ == o.hot_spot_ && image_ == o.image_;
  }

  bool operator!=(const CursorData& o) const { return !(*this == o); }

  StyleImage* GetImage() const { return image_.Get(); }
  void SetImage(StyleImage* image) { image_ = image; }

  bool HotSpotSpecified() const { return hot_spot_specified_; }

  // Hot spot in the image in logical pixels.
  const IntPoint& HotSpot() const { return hot_spot_; }

  void Trace(blink::Visitor* visitor) { visitor->Trace(image_); }

 private:
  Member<StyleImage> image_;
  bool hot_spot_specified_;
  IntPoint hot_spot_;  // for CSS3 support
};

}  // namespace blink

WTF_ALLOW_CLEAR_UNUSED_SLOTS_WITH_MEM_FUNCTIONS(blink::CursorData);

#endif  // CursorData_h
