/*
 * Copyright (C) 2014 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GOOGLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GOOGLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MediaDeviceInfo_h
#define MediaDeviceInfo_h

#include "modules/ModulesExport.h"
#include "platform/bindings/ScriptWrappable.h"
#include "platform/heap/HeapAllocator.h"
#include "platform/wtf/text/WTFString.h"
#include "public/platform/modules/mediastream/media_devices.mojom-blink.h"

using blink::mojom::blink::MediaDeviceType;

namespace blink {

class ScriptState;
class ScriptValue;

class MODULES_EXPORT MediaDeviceInfo final : public ScriptWrappable {
  DEFINE_WRAPPERTYPEINFO();

 public:
  static MediaDeviceInfo* Create(const String& device_id,
                                 const String& label,
                                 const String& group_id,
                                 MediaDeviceType);

  String deviceId() const;
  String kind() const;
  String label() const;
  String groupId() const;

  // Used for testing only.
  MediaDeviceType DeviceType() const;

  ScriptValue toJSONForBinding(ScriptState*);

 private:
  MediaDeviceInfo(const String& device_id,
                  const String& label,
                  const String& group_id,
                  MediaDeviceType);

  String device_id_;
  String label_;
  String group_id_;
  MediaDeviceType device_type_;
};

using MediaDeviceInfoVector = HeapVector<Member<MediaDeviceInfo>>;

}  // namespace blink

#endif  // MediaDeviceInfo_h
