/*
 *  Copyright (C) 2003, 2008, 2012 Apple Inc. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#ifndef WTF_dtoa_h
#define WTF_dtoa_h

#include "platform/wtf/ASCIICType.h"
#include "platform/wtf/WTFExport.h"
#include "platform/wtf/dtoa/double-conversion.h"
#include "platform/wtf/text/Unicode.h"

namespace WTF {

// Size = 80 for sizeof(DtoaBuffer) + some sign bits, decimal point, 'e',
// exponent digits.
const unsigned kNumberToStringBufferLength = 96;
typedef char NumberToStringBuffer[kNumberToStringBufferLength];

WTF_EXPORT const char* NumberToString(double, NumberToStringBuffer);
WTF_EXPORT const char* NumberToFixedPrecisionString(
    double,
    unsigned significant_figures,
    NumberToStringBuffer);
WTF_EXPORT const char* NumberToFixedWidthString(double,
                                                unsigned decimal_places,
                                                NumberToStringBuffer);

WTF_EXPORT double ParseDouble(const LChar* string,
                              size_t length,
                              size_t& parsed_length);
WTF_EXPORT double ParseDouble(const UChar* string,
                              size_t length,
                              size_t& parsed_length);

namespace Internal {
double ParseDoubleFromLongString(const UChar* string,
                                 size_t length,
                                 size_t& parsed_length);
}

inline double ParseDouble(const LChar* string,
                          size_t length,
                          size_t& parsed_length) {
  return double_conversion::StringToDoubleConverter::StringToDouble(
      reinterpret_cast<const char*>(string), length, &parsed_length);
}

inline double ParseDouble(const UChar* string,
                          size_t length,
                          size_t& parsed_length) {
  const size_t kConversionBufferSize = 64;
  if (length > kConversionBufferSize)
    return Internal::ParseDoubleFromLongString(string, length, parsed_length);
  LChar conversion_buffer[kConversionBufferSize];
  for (size_t i = 0; i < length; ++i)
    conversion_buffer[i] =
        IsASCII(string[i]) ? static_cast<LChar>(string[i]) : 0;
  return ParseDouble(conversion_buffer, length, parsed_length);
}

}  // namespace WTF

using WTF::NumberToStringBuffer;
using WTF::NumberToString;
using WTF::NumberToFixedPrecisionString;
using WTF::NumberToFixedWidthString;
using WTF::ParseDouble;

#endif  // WTF_dtoa_h
