/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WebSpeechSynthesizerClient_h
#define WebSpeechSynthesizerClient_h

#include "WebSpeechSynthesisUtterance.h"
#include "WebSpeechSynthesisVoice.h"
#include "WebVector.h"

namespace blink {

class WebSpeechSynthesizerClient {
 public:
  virtual ~WebSpeechSynthesizerClient() = default;

  virtual void SetVoiceList(const WebVector<WebSpeechSynthesisVoice>&) = 0;
  virtual void DidStartSpeaking(const WebSpeechSynthesisUtterance&) = 0;
  virtual void DidPauseSpeaking(const WebSpeechSynthesisUtterance&) = 0;
  virtual void DidResumeSpeaking(const WebSpeechSynthesisUtterance&) = 0;
  virtual void DidFinishSpeaking(const WebSpeechSynthesisUtterance&) = 0;
  virtual void SpeakingErrorOccurred(const WebSpeechSynthesisUtterance&) = 0;
  virtual void WordBoundaryEventOccurred(const WebSpeechSynthesisUtterance&,
                                         unsigned char_index) = 0;
  virtual void SentenceBoundaryEventOccurred(const WebSpeechSynthesisUtterance&,
                                             unsigned char_index) = 0;
};

}  // namespace blink

#endif  // WebSpeechSynthesizerClient_h
