// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   core/css/templates/CSSOMKeywords.cpp.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/ComputedStyleFieldAliases.json5
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/CSSProperties.json5


#include "core/css/cssom/CSSOMKeywords.h"

#include "core/css/CSSPropertyIDTemplates.h"
#include "core/css/cssom/CSSKeywordValue.h"
#include "platform/wtf/HashMap.h"

namespace blink {

bool CSSOMKeywords::ValidKeywordForProperty(CSSPropertyID id,
                                            const CSSKeywordValue& keyword) {
  CSSValueID valueID = keyword.KeywordValueID();
  if (valueID == CSSValueInvalid) {
    return false;
  }

  if (valueID == CSSValueInherit || valueID == CSSValueInitial ||
      valueID == CSSValueUnset) {
    // These are css-wide keywords that are valid for all properties.
    return true;
  }

  switch (id) {
  case CSSPropertyAnimationDirection: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueReverse:
    case CSSValueAlternate:
    case CSSValueAlternateReverse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyAnimationIterationCount: {
    switch (valueID) {
    case CSSValueInfinite:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyDirection: {
    switch (valueID) {
    case CSSValueLtr:
    case CSSValueRtl:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextOrientation: {
    switch (valueID) {
    case CSSValueSideways:
    case CSSValueMixed:
    case CSSValueUpright:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWritingMode: {
    switch (valueID) {
    case CSSValueHorizontalTb:
    case CSSValueVerticalRl:
    case CSSValueVerticalLr:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBackfaceVisibility: {
    switch (valueID) {
    case CSSValueVisible:
    case CSSValueHidden:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBackgroundImage: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderBottomStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderBottomWidth: {
    switch (valueID) {
    case CSSValueThin:
    case CSSValueMedium:
    case CSSValueThick:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderCollapse: {
    switch (valueID) {
    case CSSValueSeparate:
    case CSSValueCollapse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderImageSource: {
    switch (valueID) {
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderLeftStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderLeftWidth: {
    switch (valueID) {
    case CSSValueThin:
    case CSSValueMedium:
    case CSSValueThick:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderRightStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderRightWidth: {
    switch (valueID) {
    case CSSValueThin:
    case CSSValueMedium:
    case CSSValueThick:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderTopStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBorderTopWidth: {
    switch (valueID) {
    case CSSValueThin:
    case CSSValueMedium:
    case CSSValueThick:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBottom: {
    switch (valueID) {
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBoxSizing: {
    switch (valueID) {
    case CSSValueContentBox:
    case CSSValueBorderBox:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBreakAfter: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueAvoid:
    case CSSValueAvoidColumn:
    case CSSValueAvoidPage:
    case CSSValueColumn:
    case CSSValueLeft:
    case CSSValuePage:
    case CSSValueRecto:
    case CSSValueRight:
    case CSSValueVerso:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBreakBefore: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueAvoid:
    case CSSValueAvoidColumn:
    case CSSValueAvoidPage:
    case CSSValueColumn:
    case CSSValueLeft:
    case CSSValuePage:
    case CSSValueRecto:
    case CSSValueRight:
    case CSSValueVerso:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyBreakInside: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueAvoid:
    case CSSValueAvoidColumn:
    case CSSValueAvoidPage:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyCaptionSide: {
    switch (valueID) {
    case CSSValueTop:
    case CSSValueBottom:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyClear: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueLeft:
    case CSSValueRight:
    case CSSValueBoth:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyColumnFill: {
    switch (valueID) {
    case CSSValueBalance:
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyColumnRuleStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyColumnSpan: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueAll:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyCursor: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueDefault:
    case CSSValueNone:
    case CSSValueContextMenu:
    case CSSValueHelp:
    case CSSValuePointer:
    case CSSValueProgress:
    case CSSValueWait:
    case CSSValueCell:
    case CSSValueCrosshair:
    case CSSValueText:
    case CSSValueVerticalText:
    case CSSValueAlias:
    case CSSValueCopy:
    case CSSValueMove:
    case CSSValueNoDrop:
    case CSSValueNotAllowed:
    case CSSValueEResize:
    case CSSValueNResize:
    case CSSValueNeResize:
    case CSSValueNwResize:
    case CSSValueSResize:
    case CSSValueSeResize:
    case CSSValueSwResize:
    case CSSValueWResize:
    case CSSValueEwResize:
    case CSSValueNsResize:
    case CSSValueNeswResize:
    case CSSValueNwseResize:
    case CSSValueColResize:
    case CSSValueRowResize:
    case CSSValueAllScroll:
    case CSSValueZoomIn:
    case CSSValueZoomOut:
    case CSSValueWebkitGrab:
    case CSSValueWebkitGrabbing:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyDisplay: {
    switch (valueID) {
    case CSSValueInline:
    case CSSValueBlock:
    case CSSValueListItem:
    case CSSValueInlineBlock:
    case CSSValueTable:
    case CSSValueInlineTable:
    case CSSValueTableRowGroup:
    case CSSValueTableHeaderGroup:
    case CSSValueTableFooterGroup:
    case CSSValueTableRow:
    case CSSValueTableColumnGroup:
    case CSSValueTableColumn:
    case CSSValueTableCell:
    case CSSValueTableCaption:
    case CSSValueWebkitBox:
    case CSSValueWebkitInlineBox:
    case CSSValueFlex:
    case CSSValueInlineFlex:
    case CSSValueGrid:
    case CSSValueInlineGrid:
    case CSSValueContents:
    case CSSValueFlowRoot:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyEmptyCells: {
    switch (valueID) {
    case CSSValueShow:
    case CSSValueHide:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyFlexDirection: {
    switch (valueID) {
    case CSSValueRow:
    case CSSValueRowReverse:
    case CSSValueColumn:
    case CSSValueColumnReverse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyFlexWrap: {
    switch (valueID) {
    case CSSValueNowrap:
    case CSSValueWrap:
    case CSSValueWrapReverse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyFloat: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueLeft:
    case CSSValueRight:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyHeight: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueFitContent:
    case CSSValueMinContent:
    case CSSValueMaxContent:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyHyphens: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueManual:
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyImageRendering: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueOptimizeSpeed:
    case CSSValueOptimizeQuality:
    case CSSValueWebkitOptimizeContrast:
    case CSSValuePixelated:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyIsolation: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueIsolate:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyLeft: {
    switch (valueID) {
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyListStylePosition: {
    switch (valueID) {
    case CSSValueOutside:
    case CSSValueInside:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyListStyleType: {
    switch (valueID) {
    case CSSValueDisc:
    case CSSValueCircle:
    case CSSValueSquare:
    case CSSValueDecimal:
    case CSSValueDecimalLeadingZero:
    case CSSValueArabicIndic:
    case CSSValueBengali:
    case CSSValueCambodian:
    case CSSValueKhmer:
    case CSSValueDevanagari:
    case CSSValueGujarati:
    case CSSValueGurmukhi:
    case CSSValueKannada:
    case CSSValueLao:
    case CSSValueMalayalam:
    case CSSValueMongolian:
    case CSSValueMyanmar:
    case CSSValueOriya:
    case CSSValuePersian:
    case CSSValueUrdu:
    case CSSValueTelugu:
    case CSSValueTibetan:
    case CSSValueThai:
    case CSSValueLowerRoman:
    case CSSValueUpperRoman:
    case CSSValueLowerGreek:
    case CSSValueLowerAlpha:
    case CSSValueLowerLatin:
    case CSSValueUpperAlpha:
    case CSSValueUpperLatin:
    case CSSValueCjkEarthlyBranch:
    case CSSValueCjkHeavenlyStem:
    case CSSValueEthiopicHalehame:
    case CSSValueEthiopicHalehameAm:
    case CSSValueEthiopicHalehameTiEr:
    case CSSValueEthiopicHalehameTiEt:
    case CSSValueHangul:
    case CSSValueHangulConsonant:
    case CSSValueKoreanHangulFormal:
    case CSSValueKoreanHanjaFormal:
    case CSSValueKoreanHanjaInformal:
    case CSSValueHebrew:
    case CSSValueArmenian:
    case CSSValueLowerArmenian:
    case CSSValueUpperArmenian:
    case CSSValueGeorgian:
    case CSSValueCjkIdeographic:
    case CSSValueSimpChineseFormal:
    case CSSValueSimpChineseInformal:
    case CSSValueTradChineseFormal:
    case CSSValueTradChineseInformal:
    case CSSValueHiragana:
    case CSSValueKatakana:
    case CSSValueHiraganaIroha:
    case CSSValueKatakanaIroha:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyMixBlendMode: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueMultiply:
    case CSSValueScreen:
    case CSSValueOverlay:
    case CSSValueDarken:
    case CSSValueLighten:
    case CSSValueColorDodge:
    case CSSValueColorBurn:
    case CSSValueHardLight:
    case CSSValueSoftLight:
    case CSSValueDifference:
    case CSSValueExclusion:
    case CSSValueHue:
    case CSSValueSaturation:
    case CSSValueColor:
    case CSSValueLuminosity:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyObjectFit: {
    switch (valueID) {
    case CSSValueFill:
    case CSSValueContain:
    case CSSValueCover:
    case CSSValueNone:
    case CSSValueScaleDown:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOutlineStyle: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueHidden:
    case CSSValueInset:
    case CSSValueGroove:
    case CSSValueOutset:
    case CSSValueRidge:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueSolid:
    case CSSValueDouble:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverflowAnchor: {
    switch (valueID) {
    case CSSValueVisible:
    case CSSValueNone:
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverflowWrap: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueBreakWord:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverflowX: {
    switch (valueID) {
    case CSSValueVisible:
    case CSSValueHidden:
    case CSSValueScroll:
    case CSSValueAuto:
    case CSSValueOverlay:
    case CSSValueWebkitPagedX:
    case CSSValueWebkitPagedY:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverflowY: {
    switch (valueID) {
    case CSSValueVisible:
    case CSSValueHidden:
    case CSSValueScroll:
    case CSSValueAuto:
    case CSSValueOverlay:
    case CSSValueWebkitPagedX:
    case CSSValueWebkitPagedY:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverscrollBehaviorX: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueContain:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyOverscrollBehaviorY: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueContain:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyPointerEvents: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueAuto:
    case CSSValueStroke:
    case CSSValueFill:
    case CSSValuePainted:
    case CSSValueVisible:
    case CSSValueVisibleStroke:
    case CSSValueVisibleFill:
    case CSSValueVisiblePainted:
    case CSSValueBoundingBox:
    case CSSValueAll:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyPosition: {
    switch (valueID) {
    case CSSValueStatic:
    case CSSValueRelative:
    case CSSValueAbsolute:
    case CSSValueFixed:
    case CSSValueSticky:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyResize: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueBoth:
    case CSSValueHorizontal:
    case CSSValueVertical:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyRight: {
    switch (valueID) {
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyScrollSnapStop: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueAlways:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertySpeak: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueNormal:
    case CSSValueSpellOut:
    case CSSValueDigits:
    case CSSValueLiteralPunctuation:
    case CSSValueNoPunctuation:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTableLayout: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueFixed:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextAlign: {
    switch (valueID) {
    case CSSValueLeft:
    case CSSValueRight:
    case CSSValueCenter:
    case CSSValueJustify:
    case CSSValueWebkitLeft:
    case CSSValueWebkitRight:
    case CSSValueWebkitCenter:
    case CSSValueStart:
    case CSSValueEnd:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextAlignLast: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueStart:
    case CSSValueEnd:
    case CSSValueLeft:
    case CSSValueRight:
    case CSSValueCenter:
    case CSSValueJustify:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextCombineUpright: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueAll:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextDecorationLine: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueUnderline:
    case CSSValueOverline:
    case CSSValueLineThrough:
    case CSSValueBlink:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextDecorationSkipInk: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextDecorationStyle: {
    switch (valueID) {
    case CSSValueSolid:
    case CSSValueDouble:
    case CSSValueDotted:
    case CSSValueDashed:
    case CSSValueWavy:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextJustify: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueNone:
    case CSSValueInterWord:
    case CSSValueDistribute:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextOverflow: {
    switch (valueID) {
    case CSSValueClip:
    case CSSValueEllipsis:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextTransform: {
    switch (valueID) {
    case CSSValueCapitalize:
    case CSSValueUppercase:
    case CSSValueLowercase:
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTextUnderlinePosition: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueUnder:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTop: {
    switch (valueID) {
    case CSSValueAuto:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTransform: {
    switch (valueID) {
    case CSSValueNone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTransformBox: {
    switch (valueID) {
    case CSSValueFillBox:
    case CSSValueViewBox:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyTransformStyle: {
    switch (valueID) {
    case CSSValueFlat:
    case CSSValuePreserve3d:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyUnicodeBidi: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueEmbed:
    case CSSValueBidiOverride:
    case CSSValueIsolate:
    case CSSValuePlaintext:
    case CSSValueIsolateOverride:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyUserSelect: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueNone:
    case CSSValueText:
    case CSSValueAll:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyVisibility: {
    switch (valueID) {
    case CSSValueVisible:
    case CSSValueHidden:
    case CSSValueCollapse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitAppRegion: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueDrag:
    case CSSValueNoDrag:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxAlign: {
    switch (valueID) {
    case CSSValueStretch:
    case CSSValueStart:
    case CSSValueCenter:
    case CSSValueEnd:
    case CSSValueBaseline:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxDecorationBreak: {
    switch (valueID) {
    case CSSValueSlice:
    case CSSValueClone:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxDirection: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueReverse:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxLines: {
    switch (valueID) {
    case CSSValueSingle:
    case CSSValueMultiple:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxOrient: {
    switch (valueID) {
    case CSSValueHorizontal:
    case CSSValueVertical:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitBoxPack: {
    switch (valueID) {
    case CSSValueStart:
    case CSSValueCenter:
    case CSSValueEnd:
    case CSSValueJustify:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitLineBreak: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueLoose:
    case CSSValueNormal:
    case CSSValueStrict:
    case CSSValueAfterWhiteSpace:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitMarginAfterCollapse: {
    switch (valueID) {
    case CSSValueCollapse:
    case CSSValueSeparate:
    case CSSValueDiscard:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitMarginBeforeCollapse: {
    switch (valueID) {
    case CSSValueCollapse:
    case CSSValueSeparate:
    case CSSValueDiscard:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitPrintColorAdjust: {
    switch (valueID) {
    case CSSValueEconomy:
    case CSSValueExact:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitRtlOrdering: {
    switch (valueID) {
    case CSSValueLogical:
    case CSSValueVisual:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitRubyPosition: {
    switch (valueID) {
    case CSSValueBefore:
    case CSSValueAfter:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitTextSecurity: {
    switch (valueID) {
    case CSSValueNone:
    case CSSValueDisc:
    case CSSValueCircle:
    case CSSValueSquare:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitUserDrag: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueNone:
    case CSSValueElement:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWebkitUserModify: {
    switch (valueID) {
    case CSSValueReadOnly:
    case CSSValueReadWrite:
    case CSSValueReadWritePlaintextOnly:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWhiteSpace: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValuePre:
    case CSSValuePreWrap:
    case CSSValuePreLine:
    case CSSValueNowrap:
    case CSSValueWebkitNowrap:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWidth: {
    switch (valueID) {
    case CSSValueAuto:
    case CSSValueFitContent:
    case CSSValueMinContent:
    case CSSValueMaxContent:
      return true;
    default:
      return false;
    }
  }
  case CSSPropertyWordBreak: {
    switch (valueID) {
    case CSSValueNormal:
    case CSSValueBreakAll:
    case CSSValueKeepAll:
    case CSSValueBreakWord:
      return true;
    default:
      return false;
    }
  }
  default:
    return false;
  }
}

} // namespace blink
