// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   core/css/templates/CSSOMTypes.cpp.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/ComputedStyleFieldAliases.json5
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/CSSProperties.json5


#include "core/css/cssom/CSSOMTypes.h"

#include "core/css/cssom/CSSOMKeywords.h"
#include "core/css/cssom/CSSKeywordValue.h"
#include "core/css/cssom/CSSStyleValue.h"
#include "core/css/cssom/CSSUnsupportedStyleValue.h"
#include "core/css/properties/CSSProperty.h"

namespace blink {

bool CSSOMTypes::PropertyCanTake(CSSPropertyID id,
                                 const CSSStyleValue& styleValue) {
  if (styleValue.GetType() == CSSStyleValue::kKeywordType) {
    return CSSOMKeywords::ValidKeywordForProperty(
        id, ToCSSKeywordValue(styleValue));
  }
  if (styleValue.ContainsPercent() &&
      !CSSProperty::Get(id).SupportsPercentage()) {
    return false;
  }
  if (styleValue.GetType() == CSSStyleValue::kUnknownType) {
    return ToCSSUnsupportedStyleValue(styleValue).GetProperty() == id;
  }

  return CSSOMTypes::PropertyCanTakeType(id, styleValue.GetType());
}

bool CSSOMTypes::PropertyCanTakeType(CSSPropertyID id,
                                     CSSStyleValue::StyleValueType type) {
  switch (id) {
    case CSSPropertyVariable:
      return type == CSSStyleValue::kUnparsedType;
    case CSSPropertyAnimationDuration:
      return (
          type == CSSStyleValue::kTimeType
      );
    case CSSPropertyTransitionDuration:
      return (
          type == CSSStyleValue::kTimeType
      );
    case CSSPropertyBackgroundImage:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderBottomStyle:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderBottomWidth:
      return (
          type == CSSStyleValue::kLengthType
      );
    case CSSPropertyBorderImageSource:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderLeftStyle:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderLeftWidth:
      return (
          type == CSSStyleValue::kLengthType
      );
    case CSSPropertyBorderRightStyle:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderRightWidth:
      return (
          type == CSSStyleValue::kLengthType
      );
    case CSSPropertyBorderTopStyle:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyBorderTopWidth:
      return (
          type == CSSStyleValue::kLengthType
      );
    case CSSPropertyBottom:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyContent:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyHeight:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyLeft:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyListStyleImage:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyRight:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingBlockEnd:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingBlockStart:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingBottom:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingInlineEnd:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingInlineStart:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingLeft:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingRight:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyScrollPaddingTop:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyShapeOutside:
      return (
          type == CSSStyleValue::kURLImageType
      );
    case CSSPropertyTop:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    case CSSPropertyTransform:
      return (
          type == CSSStyleValue::kTransformType
      );
    case CSSPropertyWidth:
      return (
          type == CSSStyleValue::kLengthType
          || type == CSSStyleValue::kPercentType
      );
    default:
      return false;
  }
}

}  // namespace blink
