// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CSSPropertyNames_h
#define CSSPropertyNames_h

#include "core/CoreExport.h"
#include "platform/wtf/Assertions.h"
#include <stddef.h>

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum CSSPropertyID {
    CSSPropertyInvalid = 0,
    CSSPropertyVariable = 1,
    CSSPropertyAnimationDelay = 2,
    CSSPropertyAnimationDirection = 3,
    CSSPropertyAnimationDuration = 4,
    CSSPropertyAnimationFillMode = 5,
    CSSPropertyAnimationIterationCount = 6,
    CSSPropertyAnimationName = 7,
    CSSPropertyAnimationPlayState = 8,
    CSSPropertyAnimationTimingFunction = 9,
    CSSPropertyTransitionDelay = 10,
    CSSPropertyTransitionDuration = 11,
    CSSPropertyTransitionProperty = 12,
    CSSPropertyTransitionTimingFunction = 13,
    CSSPropertyColor = 14,
    CSSPropertyDirection = 15,
    CSSPropertyFontFamily = 16,
    CSSPropertyFontFeatureSettings = 17,
    CSSPropertyFontKerning = 18,
    CSSPropertyFontSize = 19,
    CSSPropertyFontSizeAdjust = 20,
    CSSPropertyFontStretch = 21,
    CSSPropertyFontStyle = 22,
    CSSPropertyFontVariantCaps = 23,
    CSSPropertyFontVariantEastAsian = 24,
    CSSPropertyFontVariantLigatures = 25,
    CSSPropertyFontVariantNumeric = 26,
    CSSPropertyFontVariationSettings = 27,
    CSSPropertyFontWeight = 28,
    CSSPropertyTextOrientation = 29,
    CSSPropertyTextRendering = 30,
    CSSPropertyWebkitFontSmoothing = 31,
    CSSPropertyWebkitLocale = 32,
    CSSPropertyWebkitTextOrientation = 33,
    CSSPropertyWebkitWritingMode = 34,
    CSSPropertyWritingMode = 35,
    CSSPropertyZoom = 36,
    CSSPropertyAlignContent = 37,
    CSSPropertyAlignItems = 38,
    CSSPropertyAlignSelf = 39,
    CSSPropertyAlignmentBaseline = 40,
    CSSPropertyAll = 41,
    CSSPropertyBackdropFilter = 42,
    CSSPropertyBackfaceVisibility = 43,
    CSSPropertyBackgroundAttachment = 44,
    CSSPropertyBackgroundBlendMode = 45,
    CSSPropertyBackgroundClip = 46,
    CSSPropertyBackgroundColor = 47,
    CSSPropertyBackgroundImage = 48,
    CSSPropertyBackgroundOrigin = 49,
    CSSPropertyBackgroundPositionX = 50,
    CSSPropertyBackgroundPositionY = 51,
    CSSPropertyBackgroundRepeatX = 52,
    CSSPropertyBackgroundRepeatY = 53,
    CSSPropertyBackgroundSize = 54,
    CSSPropertyBaselineShift = 55,
    CSSPropertyBlockSize = 56,
    CSSPropertyBorderBottomColor = 57,
    CSSPropertyBorderBottomLeftRadius = 58,
    CSSPropertyBorderBottomRightRadius = 59,
    CSSPropertyBorderBottomStyle = 60,
    CSSPropertyBorderBottomWidth = 61,
    CSSPropertyBorderCollapse = 62,
    CSSPropertyBorderImageOutset = 63,
    CSSPropertyBorderImageRepeat = 64,
    CSSPropertyBorderImageSlice = 65,
    CSSPropertyBorderImageSource = 66,
    CSSPropertyBorderImageWidth = 67,
    CSSPropertyBorderLeftColor = 68,
    CSSPropertyBorderLeftStyle = 69,
    CSSPropertyBorderLeftWidth = 70,
    CSSPropertyBorderRightColor = 71,
    CSSPropertyBorderRightStyle = 72,
    CSSPropertyBorderRightWidth = 73,
    CSSPropertyBorderTopColor = 74,
    CSSPropertyBorderTopLeftRadius = 75,
    CSSPropertyBorderTopRightRadius = 76,
    CSSPropertyBorderTopStyle = 77,
    CSSPropertyBorderTopWidth = 78,
    CSSPropertyBottom = 79,
    CSSPropertyBoxShadow = 80,
    CSSPropertyBoxSizing = 81,
    CSSPropertyBreakAfter = 82,
    CSSPropertyBreakBefore = 83,
    CSSPropertyBreakInside = 84,
    CSSPropertyBufferedRendering = 85,
    CSSPropertyCaptionSide = 86,
    CSSPropertyCaretColor = 87,
    CSSPropertyClear = 88,
    CSSPropertyClip = 89,
    CSSPropertyClipPath = 90,
    CSSPropertyClipRule = 91,
    CSSPropertyColorInterpolation = 92,
    CSSPropertyColorInterpolationFilters = 93,
    CSSPropertyColorRendering = 94,
    CSSPropertyColumnCount = 95,
    CSSPropertyColumnFill = 96,
    CSSPropertyColumnGap = 97,
    CSSPropertyColumnRuleColor = 98,
    CSSPropertyColumnRuleStyle = 99,
    CSSPropertyColumnRuleWidth = 100,
    CSSPropertyColumnSpan = 101,
    CSSPropertyColumnWidth = 102,
    CSSPropertyContain = 103,
    CSSPropertyContent = 104,
    CSSPropertyCounterIncrement = 105,
    CSSPropertyCounterReset = 106,
    CSSPropertyCursor = 107,
    CSSPropertyCx = 108,
    CSSPropertyCy = 109,
    CSSPropertyD = 110,
    CSSPropertyDisplay = 111,
    CSSPropertyDominantBaseline = 112,
    CSSPropertyEmptyCells = 113,
    CSSPropertyFill = 114,
    CSSPropertyFillOpacity = 115,
    CSSPropertyFillRule = 116,
    CSSPropertyFilter = 117,
    CSSPropertyFlexBasis = 118,
    CSSPropertyFlexDirection = 119,
    CSSPropertyFlexGrow = 120,
    CSSPropertyFlexShrink = 121,
    CSSPropertyFlexWrap = 122,
    CSSPropertyFloat = 123,
    CSSPropertyFloodColor = 124,
    CSSPropertyFloodOpacity = 125,
    CSSPropertyFontDisplay = 126,
    CSSPropertyGridAutoColumns = 127,
    CSSPropertyGridAutoFlow = 128,
    CSSPropertyGridAutoRows = 129,
    CSSPropertyGridColumnEnd = 130,
    CSSPropertyGridColumnGap = 131,
    CSSPropertyGridColumnStart = 132,
    CSSPropertyGridRowEnd = 133,
    CSSPropertyGridRowGap = 134,
    CSSPropertyGridRowStart = 135,
    CSSPropertyGridTemplateAreas = 136,
    CSSPropertyGridTemplateColumns = 137,
    CSSPropertyGridTemplateRows = 138,
    CSSPropertyHeight = 139,
    CSSPropertyHyphens = 140,
    CSSPropertyImageOrientation = 141,
    CSSPropertyImageRendering = 142,
    CSSPropertyInlineSize = 143,
    CSSPropertyIsolation = 144,
    CSSPropertyJustifyContent = 145,
    CSSPropertyJustifyItems = 146,
    CSSPropertyJustifySelf = 147,
    CSSPropertyLeft = 148,
    CSSPropertyLetterSpacing = 149,
    CSSPropertyLightingColor = 150,
    CSSPropertyLineBreak = 151,
    CSSPropertyLineHeight = 152,
    CSSPropertyLineHeightStep = 153,
    CSSPropertyListStyleImage = 154,
    CSSPropertyListStylePosition = 155,
    CSSPropertyListStyleType = 156,
    CSSPropertyMarginBottom = 157,
    CSSPropertyMarginLeft = 158,
    CSSPropertyMarginRight = 159,
    CSSPropertyMarginTop = 160,
    CSSPropertyMarkerEnd = 161,
    CSSPropertyMarkerMid = 162,
    CSSPropertyMarkerStart = 163,
    CSSPropertyMask = 164,
    CSSPropertyMaskSourceType = 165,
    CSSPropertyMaskType = 166,
    CSSPropertyMaxBlockSize = 167,
    CSSPropertyMaxHeight = 168,
    CSSPropertyMaxInlineSize = 169,
    CSSPropertyMaxWidth = 170,
    CSSPropertyMaxZoom = 171,
    CSSPropertyMinBlockSize = 172,
    CSSPropertyMinHeight = 173,
    CSSPropertyMinInlineSize = 174,
    CSSPropertyMinWidth = 175,
    CSSPropertyMinZoom = 176,
    CSSPropertyMixBlendMode = 177,
    CSSPropertyObjectFit = 178,
    CSSPropertyObjectPosition = 179,
    CSSPropertyOffsetAnchor = 180,
    CSSPropertyOffsetDistance = 181,
    CSSPropertyOffsetPath = 182,
    CSSPropertyOffsetPosition = 183,
    CSSPropertyOffsetRotate = 184,
    CSSPropertyOpacity = 185,
    CSSPropertyOrder = 186,
    CSSPropertyOrientation = 187,
    CSSPropertyOrphans = 188,
    CSSPropertyOutlineColor = 189,
    CSSPropertyOutlineOffset = 190,
    CSSPropertyOutlineStyle = 191,
    CSSPropertyOutlineWidth = 192,
    CSSPropertyOverflowAnchor = 193,
    CSSPropertyOverflowWrap = 194,
    CSSPropertyOverflowX = 195,
    CSSPropertyOverflowY = 196,
    CSSPropertyOverscrollBehaviorX = 197,
    CSSPropertyOverscrollBehaviorY = 198,
    CSSPropertyPaddingBottom = 199,
    CSSPropertyPaddingLeft = 200,
    CSSPropertyPaddingRight = 201,
    CSSPropertyPaddingTop = 202,
    CSSPropertyPage = 203,
    CSSPropertyPaintOrder = 204,
    CSSPropertyPerspective = 205,
    CSSPropertyPerspectiveOrigin = 206,
    CSSPropertyPointerEvents = 207,
    CSSPropertyPosition = 208,
    CSSPropertyQuotes = 209,
    CSSPropertyR = 210,
    CSSPropertyResize = 211,
    CSSPropertyRight = 212,
    CSSPropertyRotate = 213,
    CSSPropertyRx = 214,
    CSSPropertyRy = 215,
    CSSPropertyScale = 216,
    CSSPropertyScrollBehavior = 217,
    CSSPropertyScrollMarginBlockEnd = 218,
    CSSPropertyScrollMarginBlockStart = 219,
    CSSPropertyScrollMarginBottom = 220,
    CSSPropertyScrollMarginInlineEnd = 221,
    CSSPropertyScrollMarginInlineStart = 222,
    CSSPropertyScrollMarginLeft = 223,
    CSSPropertyScrollMarginRight = 224,
    CSSPropertyScrollMarginTop = 225,
    CSSPropertyScrollPaddingBlockEnd = 226,
    CSSPropertyScrollPaddingBlockStart = 227,
    CSSPropertyScrollPaddingBottom = 228,
    CSSPropertyScrollPaddingInlineEnd = 229,
    CSSPropertyScrollPaddingInlineStart = 230,
    CSSPropertyScrollPaddingLeft = 231,
    CSSPropertyScrollPaddingRight = 232,
    CSSPropertyScrollPaddingTop = 233,
    CSSPropertyScrollSnapAlign = 234,
    CSSPropertyScrollSnapStop = 235,
    CSSPropertyScrollSnapType = 236,
    CSSPropertyShapeImageThreshold = 237,
    CSSPropertyShapeMargin = 238,
    CSSPropertyShapeOutside = 239,
    CSSPropertyShapeRendering = 240,
    CSSPropertySize = 241,
    CSSPropertySpeak = 242,
    CSSPropertySrc = 243,
    CSSPropertyStopColor = 244,
    CSSPropertyStopOpacity = 245,
    CSSPropertyStroke = 246,
    CSSPropertyStrokeDasharray = 247,
    CSSPropertyStrokeDashoffset = 248,
    CSSPropertyStrokeLinecap = 249,
    CSSPropertyStrokeLinejoin = 250,
    CSSPropertyStrokeMiterlimit = 251,
    CSSPropertyStrokeOpacity = 252,
    CSSPropertyStrokeWidth = 253,
    CSSPropertyTabSize = 254,
    CSSPropertyTableLayout = 255,
    CSSPropertyTextAlign = 256,
    CSSPropertyTextAlignLast = 257,
    CSSPropertyTextAnchor = 258,
    CSSPropertyTextCombineUpright = 259,
    CSSPropertyTextDecorationColor = 260,
    CSSPropertyTextDecorationLine = 261,
    CSSPropertyTextDecorationSkipInk = 262,
    CSSPropertyTextDecorationStyle = 263,
    CSSPropertyTextIndent = 264,
    CSSPropertyTextJustify = 265,
    CSSPropertyTextOverflow = 266,
    CSSPropertyTextShadow = 267,
    CSSPropertyTextSizeAdjust = 268,
    CSSPropertyTextTransform = 269,
    CSSPropertyTextUnderlinePosition = 270,
    CSSPropertyTop = 271,
    CSSPropertyTouchAction = 272,
    CSSPropertyTransform = 273,
    CSSPropertyTransformBox = 274,
    CSSPropertyTransformOrigin = 275,
    CSSPropertyTransformStyle = 276,
    CSSPropertyTranslate = 277,
    CSSPropertyUnicodeBidi = 278,
    CSSPropertyUnicodeRange = 279,
    CSSPropertyUserSelect = 280,
    CSSPropertyUserZoom = 281,
    CSSPropertyVectorEffect = 282,
    CSSPropertyVerticalAlign = 283,
    CSSPropertyVisibility = 284,
    CSSPropertyWebkitAppRegion = 285,
    CSSPropertyWebkitAppearance = 286,
    CSSPropertyWebkitBorderAfterColor = 287,
    CSSPropertyWebkitBorderAfterStyle = 288,
    CSSPropertyWebkitBorderAfterWidth = 289,
    CSSPropertyWebkitBorderBeforeColor = 290,
    CSSPropertyWebkitBorderBeforeStyle = 291,
    CSSPropertyWebkitBorderBeforeWidth = 292,
    CSSPropertyWebkitBorderEndColor = 293,
    CSSPropertyWebkitBorderEndStyle = 294,
    CSSPropertyWebkitBorderEndWidth = 295,
    CSSPropertyWebkitBorderHorizontalSpacing = 296,
    CSSPropertyWebkitBorderImage = 297,
    CSSPropertyWebkitBorderStartColor = 298,
    CSSPropertyWebkitBorderStartStyle = 299,
    CSSPropertyWebkitBorderStartWidth = 300,
    CSSPropertyWebkitBorderVerticalSpacing = 301,
    CSSPropertyWebkitBoxAlign = 302,
    CSSPropertyWebkitBoxDecorationBreak = 303,
    CSSPropertyWebkitBoxDirection = 304,
    CSSPropertyWebkitBoxFlex = 305,
    CSSPropertyWebkitBoxFlexGroup = 306,
    CSSPropertyWebkitBoxLines = 307,
    CSSPropertyWebkitBoxOrdinalGroup = 308,
    CSSPropertyWebkitBoxOrient = 309,
    CSSPropertyWebkitBoxPack = 310,
    CSSPropertyWebkitBoxReflect = 311,
    CSSPropertyWebkitFontSizeDelta = 312,
    CSSPropertyWebkitHighlight = 313,
    CSSPropertyWebkitHyphenateCharacter = 314,
    CSSPropertyWebkitLineBreak = 315,
    CSSPropertyWebkitLineClamp = 316,
    CSSPropertyWebkitLogicalHeight = 317,
    CSSPropertyWebkitLogicalWidth = 318,
    CSSPropertyWebkitMarginAfter = 319,
    CSSPropertyWebkitMarginAfterCollapse = 320,
    CSSPropertyWebkitMarginBefore = 321,
    CSSPropertyWebkitMarginBeforeCollapse = 322,
    CSSPropertyWebkitMarginBottomCollapse = 323,
    CSSPropertyWebkitMarginEnd = 324,
    CSSPropertyWebkitMarginStart = 325,
    CSSPropertyWebkitMarginTopCollapse = 326,
    CSSPropertyWebkitMaskBoxImageOutset = 327,
    CSSPropertyWebkitMaskBoxImageRepeat = 328,
    CSSPropertyWebkitMaskBoxImageSlice = 329,
    CSSPropertyWebkitMaskBoxImageSource = 330,
    CSSPropertyWebkitMaskBoxImageWidth = 331,
    CSSPropertyWebkitMaskClip = 332,
    CSSPropertyWebkitMaskComposite = 333,
    CSSPropertyWebkitMaskImage = 334,
    CSSPropertyWebkitMaskOrigin = 335,
    CSSPropertyWebkitMaskPositionX = 336,
    CSSPropertyWebkitMaskPositionY = 337,
    CSSPropertyWebkitMaskRepeatX = 338,
    CSSPropertyWebkitMaskRepeatY = 339,
    CSSPropertyWebkitMaskSize = 340,
    CSSPropertyWebkitMaxLogicalHeight = 341,
    CSSPropertyWebkitMaxLogicalWidth = 342,
    CSSPropertyWebkitMinLogicalHeight = 343,
    CSSPropertyWebkitMinLogicalWidth = 344,
    CSSPropertyWebkitPaddingAfter = 345,
    CSSPropertyWebkitPaddingBefore = 346,
    CSSPropertyWebkitPaddingEnd = 347,
    CSSPropertyWebkitPaddingStart = 348,
    CSSPropertyWebkitPerspectiveOriginX = 349,
    CSSPropertyWebkitPerspectiveOriginY = 350,
    CSSPropertyWebkitPrintColorAdjust = 351,
    CSSPropertyWebkitRtlOrdering = 352,
    CSSPropertyWebkitRubyPosition = 353,
    CSSPropertyWebkitTapHighlightColor = 354,
    CSSPropertyWebkitTextCombine = 355,
    CSSPropertyWebkitTextDecorationsInEffect = 356,
    CSSPropertyWebkitTextEmphasisColor = 357,
    CSSPropertyWebkitTextEmphasisPosition = 358,
    CSSPropertyWebkitTextEmphasisStyle = 359,
    CSSPropertyWebkitTextFillColor = 360,
    CSSPropertyWebkitTextSecurity = 361,
    CSSPropertyWebkitTextStrokeColor = 362,
    CSSPropertyWebkitTextStrokeWidth = 363,
    CSSPropertyWebkitTransformOriginX = 364,
    CSSPropertyWebkitTransformOriginY = 365,
    CSSPropertyWebkitTransformOriginZ = 366,
    CSSPropertyWebkitUserDrag = 367,
    CSSPropertyWebkitUserModify = 368,
    CSSPropertyWhiteSpace = 369,
    CSSPropertyWidows = 370,
    CSSPropertyWidth = 371,
    CSSPropertyWillChange = 372,
    CSSPropertyWordBreak = 373,
    CSSPropertyWordSpacing = 374,
    CSSPropertyWordWrap = 375,
    CSSPropertyX = 376,
    CSSPropertyY = 377,
    CSSPropertyZIndex = 378,
    CSSPropertyAnimation = 379,
    CSSPropertyBackground = 380,
    CSSPropertyBackgroundPosition = 381,
    CSSPropertyBackgroundRepeat = 382,
    CSSPropertyBorder = 383,
    CSSPropertyBorderBottom = 384,
    CSSPropertyBorderColor = 385,
    CSSPropertyBorderImage = 386,
    CSSPropertyBorderLeft = 387,
    CSSPropertyBorderRadius = 388,
    CSSPropertyBorderRight = 389,
    CSSPropertyBorderSpacing = 390,
    CSSPropertyBorderStyle = 391,
    CSSPropertyBorderTop = 392,
    CSSPropertyBorderWidth = 393,
    CSSPropertyColumnRule = 394,
    CSSPropertyColumns = 395,
    CSSPropertyFlex = 396,
    CSSPropertyFlexFlow = 397,
    CSSPropertyFont = 398,
    CSSPropertyFontVariant = 399,
    CSSPropertyGrid = 400,
    CSSPropertyGridArea = 401,
    CSSPropertyGridColumn = 402,
    CSSPropertyGridGap = 403,
    CSSPropertyGridRow = 404,
    CSSPropertyGridTemplate = 405,
    CSSPropertyListStyle = 406,
    CSSPropertyMargin = 407,
    CSSPropertyMarker = 408,
    CSSPropertyOffset = 409,
    CSSPropertyOutline = 410,
    CSSPropertyOverflow = 411,
    CSSPropertyOverscrollBehavior = 412,
    CSSPropertyPadding = 413,
    CSSPropertyPageBreakAfter = 414,
    CSSPropertyPageBreakBefore = 415,
    CSSPropertyPageBreakInside = 416,
    CSSPropertyPlaceContent = 417,
    CSSPropertyPlaceItems = 418,
    CSSPropertyPlaceSelf = 419,
    CSSPropertyScrollMargin = 420,
    CSSPropertyScrollMarginBlock = 421,
    CSSPropertyScrollMarginInline = 422,
    CSSPropertyScrollPadding = 423,
    CSSPropertyScrollPaddingBlock = 424,
    CSSPropertyScrollPaddingInline = 425,
    CSSPropertyTextDecoration = 426,
    CSSPropertyTransition = 427,
    CSSPropertyWebkitBorderAfter = 428,
    CSSPropertyWebkitBorderBefore = 429,
    CSSPropertyWebkitBorderEnd = 430,
    CSSPropertyWebkitBorderStart = 431,
    CSSPropertyWebkitColumnBreakAfter = 432,
    CSSPropertyWebkitColumnBreakBefore = 433,
    CSSPropertyWebkitColumnBreakInside = 434,
    CSSPropertyWebkitMarginCollapse = 435,
    CSSPropertyWebkitMask = 436,
    CSSPropertyWebkitMaskBoxImage = 437,
    CSSPropertyWebkitMaskPosition = 438,
    CSSPropertyWebkitMaskRepeat = 439,
    CSSPropertyWebkitTextEmphasis = 440,
    CSSPropertyWebkitTextStroke = 441,
    CSSPropertyAliasEpubCaptionSide = 598,
    CSSPropertyAliasEpubTextCombine = 867,
    CSSPropertyAliasEpubTextEmphasis = 952,
    CSSPropertyAliasEpubTextEmphasisColor = 869,
    CSSPropertyAliasEpubTextEmphasisStyle = 871,
    CSSPropertyAliasEpubTextOrientation = 545,
    CSSPropertyAliasEpubTextTransform = 781,
    CSSPropertyAliasEpubWordBreak = 885,
    CSSPropertyAliasEpubWritingMode = 546,
    CSSPropertyAliasWebkitAlignContent = 549,
    CSSPropertyAliasWebkitAlignItems = 550,
    CSSPropertyAliasWebkitAlignSelf = 551,
    CSSPropertyAliasWebkitAnimation = 891,
    CSSPropertyAliasWebkitAnimationDelay = 514,
    CSSPropertyAliasWebkitAnimationDirection = 515,
    CSSPropertyAliasWebkitAnimationDuration = 516,
    CSSPropertyAliasWebkitAnimationFillMode = 517,
    CSSPropertyAliasWebkitAnimationIterationCount = 518,
    CSSPropertyAliasWebkitAnimationName = 519,
    CSSPropertyAliasWebkitAnimationPlayState = 520,
    CSSPropertyAliasWebkitAnimationTimingFunction = 521,
    CSSPropertyAliasWebkitBackfaceVisibility = 555,
    CSSPropertyAliasWebkitBackgroundClip = 558,
    CSSPropertyAliasWebkitBackgroundOrigin = 561,
    CSSPropertyAliasWebkitBackgroundSize = 566,
    CSSPropertyAliasWebkitBorderBottomLeftRadius = 570,
    CSSPropertyAliasWebkitBorderBottomRightRadius = 571,
    CSSPropertyAliasWebkitBorderRadius = 900,
    CSSPropertyAliasWebkitBorderTopLeftRadius = 587,
    CSSPropertyAliasWebkitBorderTopRightRadius = 588,
    CSSPropertyAliasWebkitBoxShadow = 592,
    CSSPropertyAliasWebkitBoxSizing = 593,
    CSSPropertyAliasWebkitClipPath = 602,
    CSSPropertyAliasWebkitColumnCount = 607,
    CSSPropertyAliasWebkitColumnGap = 609,
    CSSPropertyAliasWebkitColumnRule = 906,
    CSSPropertyAliasWebkitColumnRuleColor = 610,
    CSSPropertyAliasWebkitColumnRuleStyle = 611,
    CSSPropertyAliasWebkitColumnRuleWidth = 612,
    CSSPropertyAliasWebkitColumnSpan = 613,
    CSSPropertyAliasWebkitColumnWidth = 614,
    CSSPropertyAliasWebkitColumns = 907,
    CSSPropertyAliasWebkitFilter = 629,
    CSSPropertyAliasWebkitFlex = 908,
    CSSPropertyAliasWebkitFlexBasis = 630,
    CSSPropertyAliasWebkitFlexDirection = 631,
    CSSPropertyAliasWebkitFlexFlow = 909,
    CSSPropertyAliasWebkitFlexGrow = 632,
    CSSPropertyAliasWebkitFlexShrink = 633,
    CSSPropertyAliasWebkitFlexWrap = 634,
    CSSPropertyAliasWebkitFontFeatureSettings = 529,
    CSSPropertyAliasWebkitJustifyContent = 657,
    CSSPropertyAliasWebkitOpacity = 697,
    CSSPropertyAliasWebkitOrder = 698,
    CSSPropertyAliasWebkitPerspective = 717,
    CSSPropertyAliasWebkitPerspectiveOrigin = 718,
    CSSPropertyAliasWebkitShapeImageThreshold = 749,
    CSSPropertyAliasWebkitShapeMargin = 750,
    CSSPropertyAliasWebkitShapeOutside = 751,
    CSSPropertyAliasWebkitTextSizeAdjust = 780,
    CSSPropertyAliasWebkitTransform = 785,
    CSSPropertyAliasWebkitTransformOrigin = 787,
    CSSPropertyAliasWebkitTransformStyle = 788,
    CSSPropertyAliasWebkitTransition = 939,
    CSSPropertyAliasWebkitTransitionDelay = 522,
    CSSPropertyAliasWebkitTransitionDuration = 523,
    CSSPropertyAliasWebkitTransitionProperty = 524,
    CSSPropertyAliasWebkitTransitionTimingFunction = 525,
    CSSPropertyAliasWebkitUserSelect = 792,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    static_cast<CSSPropertyID>(598), // CSSPropertyAliasEpubCaptionSide
    static_cast<CSSPropertyID>(867), // CSSPropertyAliasEpubTextCombine
    static_cast<CSSPropertyID>(952), // CSSPropertyAliasEpubTextEmphasis
    static_cast<CSSPropertyID>(869), // CSSPropertyAliasEpubTextEmphasisColor
    static_cast<CSSPropertyID>(871), // CSSPropertyAliasEpubTextEmphasisStyle
    static_cast<CSSPropertyID>(545), // CSSPropertyAliasEpubTextOrientation
    static_cast<CSSPropertyID>(781), // CSSPropertyAliasEpubTextTransform
    static_cast<CSSPropertyID>(885), // CSSPropertyAliasEpubWordBreak
    static_cast<CSSPropertyID>(546), // CSSPropertyAliasEpubWritingMode
    static_cast<CSSPropertyID>(549), // CSSPropertyAliasWebkitAlignContent
    static_cast<CSSPropertyID>(550), // CSSPropertyAliasWebkitAlignItems
    static_cast<CSSPropertyID>(551), // CSSPropertyAliasWebkitAlignSelf
    static_cast<CSSPropertyID>(891), // CSSPropertyAliasWebkitAnimation
    static_cast<CSSPropertyID>(514), // CSSPropertyAliasWebkitAnimationDelay
    static_cast<CSSPropertyID>(515), // CSSPropertyAliasWebkitAnimationDirection
    static_cast<CSSPropertyID>(516), // CSSPropertyAliasWebkitAnimationDuration
    static_cast<CSSPropertyID>(517), // CSSPropertyAliasWebkitAnimationFillMode
    static_cast<CSSPropertyID>(518), // CSSPropertyAliasWebkitAnimationIterationCount
    static_cast<CSSPropertyID>(519), // CSSPropertyAliasWebkitAnimationName
    static_cast<CSSPropertyID>(520), // CSSPropertyAliasWebkitAnimationPlayState
    static_cast<CSSPropertyID>(521), // CSSPropertyAliasWebkitAnimationTimingFunction
    static_cast<CSSPropertyID>(555), // CSSPropertyAliasWebkitBackfaceVisibility
    static_cast<CSSPropertyID>(558), // CSSPropertyAliasWebkitBackgroundClip
    static_cast<CSSPropertyID>(561), // CSSPropertyAliasWebkitBackgroundOrigin
    static_cast<CSSPropertyID>(566), // CSSPropertyAliasWebkitBackgroundSize
    static_cast<CSSPropertyID>(570), // CSSPropertyAliasWebkitBorderBottomLeftRadius
    static_cast<CSSPropertyID>(571), // CSSPropertyAliasWebkitBorderBottomRightRadius
    static_cast<CSSPropertyID>(900), // CSSPropertyAliasWebkitBorderRadius
    static_cast<CSSPropertyID>(587), // CSSPropertyAliasWebkitBorderTopLeftRadius
    static_cast<CSSPropertyID>(588), // CSSPropertyAliasWebkitBorderTopRightRadius
    static_cast<CSSPropertyID>(592), // CSSPropertyAliasWebkitBoxShadow
    static_cast<CSSPropertyID>(593), // CSSPropertyAliasWebkitBoxSizing
    static_cast<CSSPropertyID>(602), // CSSPropertyAliasWebkitClipPath
    static_cast<CSSPropertyID>(607), // CSSPropertyAliasWebkitColumnCount
    static_cast<CSSPropertyID>(609), // CSSPropertyAliasWebkitColumnGap
    static_cast<CSSPropertyID>(906), // CSSPropertyAliasWebkitColumnRule
    static_cast<CSSPropertyID>(610), // CSSPropertyAliasWebkitColumnRuleColor
    static_cast<CSSPropertyID>(611), // CSSPropertyAliasWebkitColumnRuleStyle
    static_cast<CSSPropertyID>(612), // CSSPropertyAliasWebkitColumnRuleWidth
    static_cast<CSSPropertyID>(613), // CSSPropertyAliasWebkitColumnSpan
    static_cast<CSSPropertyID>(614), // CSSPropertyAliasWebkitColumnWidth
    static_cast<CSSPropertyID>(907), // CSSPropertyAliasWebkitColumns
    static_cast<CSSPropertyID>(629), // CSSPropertyAliasWebkitFilter
    static_cast<CSSPropertyID>(908), // CSSPropertyAliasWebkitFlex
    static_cast<CSSPropertyID>(630), // CSSPropertyAliasWebkitFlexBasis
    static_cast<CSSPropertyID>(631), // CSSPropertyAliasWebkitFlexDirection
    static_cast<CSSPropertyID>(909), // CSSPropertyAliasWebkitFlexFlow
    static_cast<CSSPropertyID>(632), // CSSPropertyAliasWebkitFlexGrow
    static_cast<CSSPropertyID>(633), // CSSPropertyAliasWebkitFlexShrink
    static_cast<CSSPropertyID>(634), // CSSPropertyAliasWebkitFlexWrap
    static_cast<CSSPropertyID>(529), // CSSPropertyAliasWebkitFontFeatureSettings
    static_cast<CSSPropertyID>(657), // CSSPropertyAliasWebkitJustifyContent
    static_cast<CSSPropertyID>(697), // CSSPropertyAliasWebkitOpacity
    static_cast<CSSPropertyID>(698), // CSSPropertyAliasWebkitOrder
    static_cast<CSSPropertyID>(717), // CSSPropertyAliasWebkitPerspective
    static_cast<CSSPropertyID>(718), // CSSPropertyAliasWebkitPerspectiveOrigin
    static_cast<CSSPropertyID>(749), // CSSPropertyAliasWebkitShapeImageThreshold
    static_cast<CSSPropertyID>(750), // CSSPropertyAliasWebkitShapeMargin
    static_cast<CSSPropertyID>(751), // CSSPropertyAliasWebkitShapeOutside
    static_cast<CSSPropertyID>(780), // CSSPropertyAliasWebkitTextSizeAdjust
    static_cast<CSSPropertyID>(785), // CSSPropertyAliasWebkitTransform
    static_cast<CSSPropertyID>(787), // CSSPropertyAliasWebkitTransformOrigin
    static_cast<CSSPropertyID>(788), // CSSPropertyAliasWebkitTransformStyle
    static_cast<CSSPropertyID>(939), // CSSPropertyAliasWebkitTransition
    static_cast<CSSPropertyID>(522), // CSSPropertyAliasWebkitTransitionDelay
    static_cast<CSSPropertyID>(523), // CSSPropertyAliasWebkitTransitionDuration
    static_cast<CSSPropertyID>(524), // CSSPropertyAliasWebkitTransitionProperty
    static_cast<CSSPropertyID>(525), // CSSPropertyAliasWebkitTransitionTimingFunction
    static_cast<CSSPropertyID>(792), // CSSPropertyAliasWebkitUserSelect
};

const CSSPropertyID firstCSSProperty = static_cast<CSSPropertyID>(2);
const int numCSSProperties = 509;
const CSSPropertyID lastCSSProperty = static_cast<CSSPropertyID>(441);
const CSSPropertyID lastUnresolvedCSSProperty = static_cast<CSSPropertyID>(952);
const int numCSSPropertyIDs = lastUnresolvedCSSProperty + 1;
const size_t maxCSSPropertyNameLength = 40;

inline bool isCSSPropertyIDWithName(int id)
{
    return id >= firstCSSProperty && id <= lastUnresolvedCSSProperty;
}

inline bool isValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyInvalid;
}

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    DCHECK_GE(value, CSSPropertyInvalid);
    DCHECK_LE(value, lastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(id & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return id & 512; }

CSSPropertyID unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID CORE_EXPORT cssPropertyID(const WTF::String&);

} // namespace blink

#endif // CSSPropertyNames_h
