// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   core/css/properties/templates/CSSPropertySubclass.h.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/ComputedStyleFieldAliases.json5
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/CSSProperties.json5
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/css/properties/CSSPropertyMethods.json5


#ifndef GEN_BLINK_CORE_CSS_PROPERTIES_LONGHAND_BorderImageSlice_h
#define GEN_BLINK_CORE_CSS_PROPERTIES_LONGHAND_BorderImageSlice_h

#include "core/css/CSSIdentifierValue.h"
#include "core/css/CSSPrimitiveValueMappings.h"
#include "core/css/properties/Longhand.h"
#include "core/css/properties/StyleBuildingUtils.h"
#include "core/css/resolver/StyleResolverState.h"
#include "core/style/ComputedStyle.h"

namespace blink {
namespace CSSLonghand {

// Implements the 'border-image-slice' CSS property
// See src/third_party/WebKit/Source/core/css/properties/README.md
class BorderImageSlice final : public Longhand {
 public:
  constexpr BorderImageSlice() : Longhand() {}
  const char* GetPropertyName() const override { return "border-image-slice\0"; }
  const WTF::AtomicString& GetPropertyNameAtomicString() const override {
    static const WTF::AtomicString& name = WTF::AtomicString("border-image-slice\0");
    return name;
  }
  const WTF::String GetJSPropertyName() const override {
    return WTF::String("borderImageSlice\0");
  }
  CSSPropertyID PropertyID() const override { return CSSPropertyBorderImageSlice; }
  const CSSValue* ParseSingleValue(CSSParserTokenRange&, const CSSParserContext&, const CSSParserLocalContext&) const override;
  const CSSValue* CSSValueFromComputedStyleInternal(const ComputedStyle&, const SVGComputedStyle&, const LayoutObject*, Node*, bool allow_visited_style) const override;
  bool IsInterpolable() const override { return true; }

  // Style builder functions
  void ApplyInitial(StyleResolverState& state) const override {
    const NinePieceImage& currentImage = state.Style()->BorderImage();
    if (currentImage.Fill() == false &&
        StyleBuildingUtils::lengthMatchesAllSides(currentImage.ImageSlices(), Length(100, kPercent)))
      return;
    NinePieceImage image(currentImage);
    image.SetImageSlices(LengthBox(Length(100, kPercent), Length(100, kPercent), Length(100, kPercent), Length(100, kPercent)));
    image.SetFill(false);
    state.Style()->SetBorderImage(image);
  }
  void ApplyInherit(StyleResolverState& state) const override {
    NinePieceImage image(state.Style()->BorderImage());
    image.CopyImageSlicesFrom(state.ParentStyle()->BorderImage());
    state.Style()->SetBorderImage(image);
  }
  void ApplyValue(StyleResolverState& state, const CSSValue& value) const override {
    NinePieceImage image(state.Style()->BorderImage());
    CSSToStyleMap::MapNinePieceImageSlice(state, value, image);
    state.Style()->SetBorderImage(image);
  }
};

}  // namespace CSSLonghand
}  // namespace blink

#endif  // GEN_BLINK_CORE_CSS_PROPERTIES_LONGHAND_BorderImageSlice_h
