// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/EventFactory.cpp.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/WebKit/Source/core/events/EventAliases.json5
//   gen/blink/core/event_names.json5


#include "core/events/EventFactory.h"

#include "core/frame/Deprecation.h"
#include "core/css/FontFaceSetLoadEvent.h"
#include "core/css/MediaQueryListEvent.h"
#include "core/dom/events/CustomEvent.h"
#include "core/dom/events/Event.h"
#include "core/events/AnimationEvent.h"
#include "core/events/AnimationPlaybackEvent.h"
#include "core/events/ApplicationCacheErrorEvent.h"
#include "core/events/BeforeUnloadEvent.h"
#include "core/events/ClipboardEvent.h"
#include "core/events/CompositionEvent.h"
#include "core/events/DragEvent.h"
#include "core/events/ErrorEvent.h"
#include "core/events/FocusEvent.h"
#include "core/events/HashChangeEvent.h"
#include "core/events/InputEvent.h"
#include "core/events/KeyboardEvent.h"
#include "core/events/MessageEvent.h"
#include "core/events/MouseEvent.h"
#include "core/events/MutationEvent.h"
#include "core/events/PageTransitionEvent.h"
#include "core/events/PointerEvent.h"
#include "core/events/PopStateEvent.h"
#include "core/events/ProgressEvent.h"
#include "core/events/PromiseRejectionEvent.h"
#include "core/events/ResourceProgressEvent.h"
#include "core/events/SecurityPolicyViolationEvent.h"
#include "core/events/TextEvent.h"
#include "core/events/TouchEvent.h"
#include "core/events/TransitionEvent.h"
#include "core/events/UIEvent.h"
#include "core/events/WheelEvent.h"
#include "core/html/track/TrackEvent.h"
#include "core/mojo/test/MojoInterfaceRequestEvent.h"
#include "platform/runtime_enabled_features.h"

namespace blink {

Event* EventFactory::Create(ExecutionContext* executionContext, const String& type) {
  if (DeprecatedEqualIgnoringCase(type, "CustomEvent")) {
    return CustomEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "Event")) {
    return Event::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "AnimationEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventAnimationEvent);
    return AnimationEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "BeforeUnloadEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventBeforeUnloadEvent);
    return BeforeUnloadEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "CompositionEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventCompositionEvent);
    return CompositionEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "DragEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventDragEvent);
    return DragEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "ErrorEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventErrorEvent);
    return ErrorEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "FocusEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventFocusEvent);
    return FocusEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "HashChangeEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventHashChangeEvent);
    return HashChangeEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "KeyboardEvent")) {
    return KeyboardEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "MessageEvent")) {
    return MessageEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "MouseEvent")) {
    return MouseEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "MutationEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventMutationEvent);
    return MutationEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "PageTransitionEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventPageTransitionEvent);
    return PageTransitionEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "PopStateEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventPopStateEvent);
    return PopStateEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "TextEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventTextEvent);
    return TextEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "TouchEvent")) {
    return TouchEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "TransitionEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventTransitionEvent);
    return TransitionEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "UIEvent")) {
    return UIEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "WheelEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventWheelEvent);
    return WheelEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "TrackEvent")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventTrackEvent);
    return TrackEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "Events")) {
    return Event::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "HTMLEvents")) {
    return Event::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "KeyboardEvents")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventKeyboardEvents);
    return KeyboardEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "MouseEvents")) {
    return MouseEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "MutationEvents")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventMutationEvents);
    return MutationEvent::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "SVGEvents")) {
    UseCounter::Count(executionContext, WebFeature::kDocumentCreateEventSVGEvents);
    return Event::Create();
  }
  if (DeprecatedEqualIgnoringCase(type, "UIEvents")) {
    return UIEvent::Create();
  }
  return nullptr;
}

} // namespace blink
