// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/origin_trials.cc.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/WebKit/Source/platform/runtime_enabled_features.json5


#include "core/origin_trials/origin_trials.h"

#include "core/origin_trials/OriginTrialContext.h"
#include "platform/runtime_enabled_features.h"

namespace blink {

const char OriginTrials::kAudioWorkletTrialName[] = "AudioWorklet";
const char OriginTrials::kBudgetQueryTrialName[] = "BudgetQuery";
const char OriginTrials::kDisableHardwareNoiseSuppressionTrialName[] = "DisableHardwareNoiseSuppression";
const char OriginTrials::kGamepadExtensionsTrialName[] = "WebVR1.1M62";
const char OriginTrials::kInstalledAppTrialName[] = "InstalledApp";
const char OriginTrials::kLowLatencyCanvasTrialName[] = "LowLatencyCanvas";
const char OriginTrials::kMediaCapabilitiesTrialName[] = "MediaCapabilities";
const char OriginTrials::kOriginTrialsSampleAPITrialName[] = "Frobulate";
const char OriginTrials::kSensorTrialName[] = "GenericSensor";
const char OriginTrials::kTouchEventFeatureDetectionTrialName[] = "ForceTouchEventFeatureDetectionForInspector";
const char OriginTrials::kWebVRTrialName[] = "WebVR1.1M62";


bool OriginTrials::audioWorkletEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::AudioWorkletEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kAudioWorkletTrialName);
}


bool OriginTrials::budgetQueryEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::BudgetQueryEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kBudgetQueryTrialName);
}


bool OriginTrials::disableHardwareNoiseSuppressionEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::DisableHardwareNoiseSuppressionEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kDisableHardwareNoiseSuppressionTrialName);
}


bool OriginTrials::gamepadExtensionsEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::GamepadExtensionsEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kGamepadExtensionsTrialName);
}


bool OriginTrials::installedAppEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::InstalledAppEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kInstalledAppTrialName);
}


bool OriginTrials::lowLatencyCanvasEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::LowLatencyCanvasEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kLowLatencyCanvasTrialName);
}


bool OriginTrials::mediaCapabilitiesEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::MediaCapabilitiesEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kMediaCapabilitiesTrialName);
}


bool OriginTrials::originTrialsSampleAPIEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::OriginTrialsSampleAPIEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kOriginTrialsSampleAPITrialName);
}


bool OriginTrials::sensorEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::SensorEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kSensorTrialName);
}


bool OriginTrials::touchEventFeatureDetectionEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::TouchEventFeatureDetectionEnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kTouchEventFeatureDetectionTrialName);
}


bool OriginTrials::webVREnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::WebVREnabled())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  return context && context->IsTrialEnabled(OriginTrials::kWebVRTrialName);
}


} // namespace blink
