// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   /InstrumentingProbesImpl.cpp.tmpl
// and input files:
//   PlatformProbes.pidl


#include "platform/wtf/ThreadingPrimitives.h"

#include "PlatformProbesInl.h"
#include "PlatformProbeSink.h"
#include "platform/probe/PlatformTraceEventsAgent.h"

namespace blink {

namespace {

Mutex& AgentCountMutex() {
  DEFINE_THREAD_SAFE_STATIC_LOCAL(Mutex, mutex, ());
  return mutex;
}

}  // namespace

// static
std::atomic<unsigned> PlatformProbeSink::s_existingAgents;

PlatformProbeSink::PlatformProbeSink() {}

PlatformProbeSink::~PlatformProbeSink() {
  MutexLocker lock(AgentCountMutex());
  if (hasPlatformTraceEventsAgents() && --s_numSinksWithPlatformTraceEventsAgent == 0)
    s_existingAgents &= ~kPlatformTraceEventsAgent;
}


// static
unsigned PlatformProbeSink::s_numSinksWithPlatformTraceEventsAgent = 0;

void PlatformProbeSink::addPlatformTraceEventsAgent(PlatformTraceEventsAgent* agent) {
  bool already_had_agent = hasPlatformTraceEventsAgents();
  m_platformTraceEventsAgents.insert(agent);

  if (!already_had_agent) {
    MutexLocker lock(AgentCountMutex());
    if (++s_numSinksWithPlatformTraceEventsAgent == 1)
      s_existingAgents |= kPlatformTraceEventsAgent;
  }

  DCHECK(HasAgentsGlobal(kPlatformTraceEventsAgent));
}

void PlatformProbeSink::removePlatformTraceEventsAgent(PlatformTraceEventsAgent* agent) {
  if (!hasPlatformTraceEventsAgents())
    return;

  m_platformTraceEventsAgents.erase(agent);

  if (!hasPlatformTraceEventsAgents()) {
    MutexLocker lock(AgentCountMutex());
    if (--s_numSinksWithPlatformTraceEventsAgent == 0)
      s_existingAgents &= ~kPlatformTraceEventsAgent;
  }

  if (hasPlatformTraceEventsAgents())
    DCHECK(HasAgentsGlobal(kPlatformTraceEventsAgent));
}

void PlatformProbeSink::Trace(Visitor* visitor)
{
  visitor->Trace(m_platformTraceEventsAgents);
}

namespace probe {

PlatformSendRequest::PlatformSendRequest(FetchContext* paramFetchContext, unsigned long identifier, ResourceRequest& paramResourceRequest, const ResourceResponse& redirectResponse, const FetchInitiatorInfo& paramFetchInitiatorInfo) :
    paramFetchContext(paramFetchContext),
    identifier(identifier),
    paramResourceRequest(paramResourceRequest),
    redirectResponse(redirectResponse),
    paramFetchInitiatorInfo(paramFetchInitiatorInfo) {
  probe_sink = ToPlatformProbeSink(paramFetchContext);
  if (!probe_sink)
    return;
  if (probe_sink->hasPlatformTraceEventsAgents()) {
    for (PlatformTraceEventsAgent* agent : probe_sink->platformTraceEventsAgents())
      agent->Will(*this);
  }
}

PlatformSendRequest::~PlatformSendRequest() {
  if (!probe_sink)
    return;
  if (probe_sink->hasPlatformTraceEventsAgents()) {
    for (PlatformTraceEventsAgent* agent : probe_sink->platformTraceEventsAgents())
      agent->Did(*this);
  }
}


} // namespace probe
} // namespace blink
