// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEVICE_SENSORS_PUBLIC_INTERFACES_MOTION_MOJOM_H_
#define DEVICE_SENSORS_PUBLIC_INTERFACES_MOTION_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "device/sensors/public/interfaces/motion.mojom-shared.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace device {
namespace mojom {
class MotionSensor;
using MotionSensorPtr = mojo::InterfacePtr<MotionSensor>;
using MotionSensorPtrInfo = mojo::InterfacePtrInfo<MotionSensor>;
using ThreadSafeMotionSensorPtr =
    mojo::ThreadSafeInterfacePtr<MotionSensor>;
using MotionSensorRequest = mojo::InterfaceRequest<MotionSensor>;
using MotionSensorAssociatedPtr =
    mojo::AssociatedInterfacePtr<MotionSensor>;
using ThreadSafeMotionSensorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MotionSensor>;
using MotionSensorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MotionSensor>;
using MotionSensorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MotionSensor>;


class MotionSensorProxy;

template <typename ImplRefTraits>
class MotionSensorStub;

class MotionSensorRequestValidator;
class MotionSensorResponseValidator;

class  MotionSensor
    : public MotionSensorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MotionSensorProxy;

  template <typename ImplRefTraits>
  using Stub_ = MotionSensorStub<ImplRefTraits>;

  using RequestValidator_ = MotionSensorRequestValidator;
  using ResponseValidator_ = MotionSensorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartPollingMinVersion = 0,
    kStopPollingMinVersion = 0,
  };
  virtual ~MotionSensor() {}


  using StartPollingCallback = base::OnceCallback<void(mojo::ScopedSharedBufferHandle)>;
  virtual void StartPolling(StartPollingCallback callback) = 0;

  virtual void StopPolling() = 0;
};
class  MotionSensorInterceptorForTesting : public MotionSensor {
  virtual MotionSensor* GetForwardingInterface() = 0;
  void StartPolling(StartPollingCallback callback) override;
  void StopPolling() override;
};
class  MotionSensorAsyncWaiter {
 public:
  explicit MotionSensorAsyncWaiter(MotionSensor* proxy);
  ~MotionSensorAsyncWaiter();
  void StartPolling(
      mojo::ScopedSharedBufferHandle* out_memory_handle);

 private:
  MotionSensor* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(MotionSensorAsyncWaiter);
};

class  MotionSensorProxy
    : public MotionSensor {
 public:
  explicit MotionSensorProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartPolling(StartPollingCallback callback) final;
  void StopPolling() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  MotionSensorStubDispatch {
 public:
  static bool Accept(MotionSensor* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MotionSensor* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MotionSensor>>
class MotionSensorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MotionSensorStub() {}
  ~MotionSensorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MotionSensorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MotionSensorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MotionSensorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  MotionSensorResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace device

namespace mojo {

}  // namespace mojo

#endif  // DEVICE_SENSORS_PUBLIC_INTERFACES_MOTION_MOJOM_H_