// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_INTERFACES_CDM_SERVICE_MOJOM_H_
#define MEDIA_MOJO_INTERFACES_CDM_SERVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "media/mojo/interfaces/cdm_service.mojom-shared.h"
#include "media/mojo/interfaces/interface_factory.mojom.h"
#include "mojo/common/file_path.mojom.h"
#include "services/service_manager/public/interfaces/interface_provider.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace media {
namespace mojom {
class CdmService;
using CdmServicePtr = mojo::InterfacePtr<CdmService>;
using CdmServicePtrInfo = mojo::InterfacePtrInfo<CdmService>;
using ThreadSafeCdmServicePtr =
    mojo::ThreadSafeInterfacePtr<CdmService>;
using CdmServiceRequest = mojo::InterfaceRequest<CdmService>;
using CdmServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<CdmService>;
using ThreadSafeCdmServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CdmService>;
using CdmServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CdmService>;
using CdmServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CdmService>;


class CdmServiceProxy;

template <typename ImplRefTraits>
class CdmServiceStub;

class CdmServiceRequestValidator;

class  CdmService
    : public CdmServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CdmServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = CdmServiceStub<ImplRefTraits>;

  using RequestValidator_ = CdmServiceRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kLoadCdmMinVersion = 0,
    kCreateInterfaceFactoryMinVersion = 0,
  };
  virtual ~CdmService() {}

  virtual void LoadCdm(const base::FilePath& cdm_path) = 0;

  virtual void CreateInterfaceFactory(::media::mojom::InterfaceFactoryRequest factory, ::service_manager::mojom::InterfaceProviderPtr host_interfaces) = 0;
};
class  CdmServiceInterceptorForTesting : public CdmService {
  virtual CdmService* GetForwardingInterface() = 0;
  void LoadCdm(const base::FilePath& cdm_path) override;
  void CreateInterfaceFactory(::media::mojom::InterfaceFactoryRequest factory, ::service_manager::mojom::InterfaceProviderPtr host_interfaces) override;
};
class  CdmServiceAsyncWaiter {
 public:
  explicit CdmServiceAsyncWaiter(CdmService* proxy);
  ~CdmServiceAsyncWaiter();

 private:
  CdmService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(CdmServiceAsyncWaiter);
};

class  CdmServiceProxy
    : public CdmService {
 public:
  explicit CdmServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void LoadCdm(const base::FilePath& cdm_path) final;
  void CreateInterfaceFactory(::media::mojom::InterfaceFactoryRequest factory, ::service_manager::mojom::InterfaceProviderPtr host_interfaces) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  CdmServiceStubDispatch {
 public:
  static bool Accept(CdmService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CdmService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CdmService>>
class CdmServiceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CdmServiceStub() {}
  ~CdmServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CdmServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CdmServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CdmServiceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace media

namespace mojo {

}  // namespace mojo

#endif  // MEDIA_MOJO_INTERFACES_CDM_SERVICE_MOJOM_H_