// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "media/mojo/interfaces/video_decode_perf_history.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "media/base/ipc/media_param_traits_macros.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace media {
namespace mojom {
const char VideoDecodePerfHistory::Name_[] = "media::mojom::VideoDecodePerfHistory";

class VideoDecodePerfHistory_GetPerfInfo_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  VideoDecodePerfHistory_GetPerfInfo_ForwardToCallback(
      VideoDecodePerfHistory::GetPerfInfoCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  VideoDecodePerfHistory::GetPerfInfoCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(VideoDecodePerfHistory_GetPerfInfo_ForwardToCallback);
};

VideoDecodePerfHistoryProxy::VideoDecodePerfHistoryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VideoDecodePerfHistoryProxy::GetPerfInfo(
    media::VideoCodecProfile in_profile, const gfx::Size& in_video_size, int32_t in_frames_per_sec, GetPerfInfoCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "media::mojom::VideoDecodePerfHistory::GetPerfInfo");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kVideoDecodePerfHistory_GetPerfInfo_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::media::mojom::internal::VideoDecodePerfHistory_GetPerfInfo_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::media::mojom::VideoCodecProfile>(
      in_profile, &params->profile);
  typename decltype(params->video_size)::BaseType::BufferWriter
      video_size_writer;
  mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
      in_video_size, buffer, &video_size_writer, &serialization_context);
  params->video_size.Set(
      video_size_writer.is_null() ? nullptr : video_size_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->video_size.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null video_size in VideoDecodePerfHistory.GetPerfInfo request");
  params->frames_per_sec = in_frames_per_sec;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new VideoDecodePerfHistory_GetPerfInfo_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder {
 public:
  static VideoDecodePerfHistory::GetPerfInfoCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder> proxy(
        new VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "VideoDecodePerfHistory::GetPerfInfoCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_is_smooth, bool in_is_power_efficient);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder);
};

bool VideoDecodePerfHistory_GetPerfInfo_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "media::mojom::VideoDecodePerfHistory::GetPerfInfoCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::VideoDecodePerfHistory_GetPerfInfo_ResponseParams_Data* params =
      reinterpret_cast<
          internal::VideoDecodePerfHistory_GetPerfInfo_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_is_smooth{};
  bool p_is_power_efficient{};
  VideoDecodePerfHistory_GetPerfInfo_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_is_smooth = input_data_view.is_smooth();
  p_is_power_efficient = input_data_view.is_power_efficient();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "VideoDecodePerfHistory::GetPerfInfo response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_is_smooth), 
std::move(p_is_power_efficient));
  return true;
}

void VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder::Run(
    bool in_is_smooth, bool in_is_power_efficient) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kVideoDecodePerfHistory_GetPerfInfo_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::media::mojom::internal::VideoDecodePerfHistory_GetPerfInfo_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->is_smooth = in_is_smooth;
  params->is_power_efficient = in_is_power_efficient;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodePerfHistory::GetPerfInfoCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool VideoDecodePerfHistoryStubDispatch::Accept(
    VideoDecodePerfHistory* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVideoDecodePerfHistory_GetPerfInfo_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodePerfHistory::GetPerfInfo",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool VideoDecodePerfHistoryStubDispatch::AcceptWithResponder(
    VideoDecodePerfHistory* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVideoDecodePerfHistory_GetPerfInfo_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodePerfHistory::GetPerfInfo",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::VideoDecodePerfHistory_GetPerfInfo_Params_Data* params =
          reinterpret_cast<
              internal::VideoDecodePerfHistory_GetPerfInfo_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      media::VideoCodecProfile p_profile{};
      gfx::Size p_video_size{};
      int32_t p_frames_per_sec{};
      VideoDecodePerfHistory_GetPerfInfo_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProfile(&p_profile))
        success = false;
      if (!input_data_view.ReadVideoSize(&p_video_size))
        success = false;
      p_frames_per_sec = input_data_view.frames_per_sec();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VideoDecodePerfHistory::GetPerfInfo deserializer");
        return false;
      }
      VideoDecodePerfHistory::GetPerfInfoCallback callback =
          VideoDecodePerfHistory_GetPerfInfo_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetPerfInfo(
std::move(p_profile), 
std::move(p_video_size), 
std::move(p_frames_per_sec), std::move(callback));
      return true;
    }
  }
  return false;
}

bool VideoDecodePerfHistoryRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VideoDecodePerfHistory RequestValidator");

  switch (message->header()->name) {
    case internal::kVideoDecodePerfHistory_GetPerfInfo_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VideoDecodePerfHistory_GetPerfInfo_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool VideoDecodePerfHistoryResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VideoDecodePerfHistory ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kVideoDecodePerfHistory_GetPerfInfo_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::VideoDecodePerfHistory_GetPerfInfo_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void VideoDecodePerfHistoryInterceptorForTesting::GetPerfInfo(media::VideoCodecProfile profile, const gfx::Size& video_size, int32_t frames_per_sec, GetPerfInfoCallback callback) {
  GetForwardingInterface()->GetPerfInfo(std::move(profile), std::move(video_size), std::move(frames_per_sec), std::move(callback));
}
VideoDecodePerfHistoryAsyncWaiter::VideoDecodePerfHistoryAsyncWaiter(
    VideoDecodePerfHistory* proxy) : proxy_(proxy) {}

VideoDecodePerfHistoryAsyncWaiter::~VideoDecodePerfHistoryAsyncWaiter() = default;

void VideoDecodePerfHistoryAsyncWaiter::GetPerfInfo(
    media::VideoCodecProfile profile, const gfx::Size& video_size, int32_t frames_per_sec, bool* out_is_smooth, bool* out_is_power_efficient) {
  base::RunLoop loop;
  proxy_->GetPerfInfo(std::move(profile),std::move(video_size),std::move(frames_per_sec),
      base::Bind(
          [](base::RunLoop* loop,
             bool* out_is_smooth
,
             bool* out_is_power_efficient
,
             bool is_smooth,
             bool is_power_efficient) {*out_is_smooth = std::move(is_smooth);*out_is_power_efficient = std::move(is_power_efficient);
            loop->Quit();
          },
          &loop,
          out_is_smooth,
          out_is_power_efficient));
  loop.Run();
}

}  // namespace mojom
}  // namespace media

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif