// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "media/mojo/interfaces/video_decode_stats_recorder.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "third_party/WebKit/Source/platform/mojo/GeometryStructTraits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace media {
namespace mojom {
namespace blink {
const char VideoDecodeStatsRecorder::Name_[] = "media::mojom::VideoDecodeStatsRecorder";

VideoDecodeStatsRecorderProxy::VideoDecodeStatsRecorderProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void VideoDecodeStatsRecorderProxy::StartNewRecord(
    ::media::mojom::blink::VideoCodecProfile in_profile, const ::blink::WebSize& in_video_size, int32_t in_frames_per_sec) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "media::mojom::VideoDecodeStatsRecorder::StartNewRecord");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kVideoDecodeStatsRecorder_StartNewRecord_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::media::mojom::internal::VideoDecodeStatsRecorder_StartNewRecord_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::media::mojom::VideoCodecProfile>(
      in_profile, &params->profile);
  typename decltype(params->video_size)::BaseType::BufferWriter
      video_size_writer;
  mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
      in_video_size, buffer, &video_size_writer, &serialization_context);
  params->video_size.Set(
      video_size_writer.is_null() ? nullptr : video_size_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->video_size.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null video_size in VideoDecodeStatsRecorder.StartNewRecord request");
  params->frames_per_sec = in_frames_per_sec;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void VideoDecodeStatsRecorderProxy::UpdateRecord(
    uint32_t in_frames_decoded, uint32_t in_frames_dropped, uint32_t in_frames_decoded_power_efficient) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "media::mojom::VideoDecodeStatsRecorder::UpdateRecord");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kVideoDecodeStatsRecorder_UpdateRecord_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::media::mojom::internal::VideoDecodeStatsRecorder_UpdateRecord_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->frames_decoded = in_frames_decoded;
  params->frames_dropped = in_frames_dropped;
  params->frames_decoded_power_efficient = in_frames_decoded_power_efficient;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool VideoDecodeStatsRecorderStubDispatch::Accept(
    VideoDecodeStatsRecorder* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kVideoDecodeStatsRecorder_StartNewRecord_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodeStatsRecorder::StartNewRecord",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VideoDecodeStatsRecorder_StartNewRecord_Params_Data* params =
          reinterpret_cast<internal::VideoDecodeStatsRecorder_StartNewRecord_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::media::mojom::blink::VideoCodecProfile p_profile{};
      ::blink::WebSize p_video_size{};
      int32_t p_frames_per_sec{};
      VideoDecodeStatsRecorder_StartNewRecord_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProfile(&p_profile))
        success = false;
      if (!input_data_view.ReadVideoSize(&p_video_size))
        success = false;
      p_frames_per_sec = input_data_view.frames_per_sec();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VideoDecodeStatsRecorder::StartNewRecord deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StartNewRecord(
std::move(p_profile), 
std::move(p_video_size), 
std::move(p_frames_per_sec));
      return true;
    }
    case internal::kVideoDecodeStatsRecorder_UpdateRecord_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodeStatsRecorder::UpdateRecord",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::VideoDecodeStatsRecorder_UpdateRecord_Params_Data* params =
          reinterpret_cast<internal::VideoDecodeStatsRecorder_UpdateRecord_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_frames_decoded{};
      uint32_t p_frames_dropped{};
      uint32_t p_frames_decoded_power_efficient{};
      VideoDecodeStatsRecorder_UpdateRecord_ParamsDataView input_data_view(params, &serialization_context);
      
      p_frames_decoded = input_data_view.frames_decoded();
      p_frames_dropped = input_data_view.frames_dropped();
      p_frames_decoded_power_efficient = input_data_view.frames_decoded_power_efficient();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "VideoDecodeStatsRecorder::UpdateRecord deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UpdateRecord(
std::move(p_frames_decoded), 
std::move(p_frames_dropped), 
std::move(p_frames_decoded_power_efficient));
      return true;
    }
  }
  return false;
}

// static
bool VideoDecodeStatsRecorderStubDispatch::AcceptWithResponder(
    VideoDecodeStatsRecorder* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kVideoDecodeStatsRecorder_StartNewRecord_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodeStatsRecorder::StartNewRecord",
               "message", message->name());
#endif
      break;
    }
    case internal::kVideoDecodeStatsRecorder_UpdateRecord_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)media::mojom::VideoDecodeStatsRecorder::UpdateRecord",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool VideoDecodeStatsRecorderRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "VideoDecodeStatsRecorder RequestValidator");

  switch (message->header()->name) {
    case internal::kVideoDecodeStatsRecorder_StartNewRecord_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VideoDecodeStatsRecorder_StartNewRecord_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kVideoDecodeStatsRecorder_UpdateRecord_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::VideoDecodeStatsRecorder_UpdateRecord_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void VideoDecodeStatsRecorderInterceptorForTesting::StartNewRecord(::media::mojom::blink::VideoCodecProfile profile, const ::blink::WebSize& video_size, int32_t frames_per_sec) {
  GetForwardingInterface()->StartNewRecord(std::move(profile), std::move(video_size), std::move(frames_per_sec));
}
void VideoDecodeStatsRecorderInterceptorForTesting::UpdateRecord(uint32_t frames_decoded, uint32_t frames_dropped, uint32_t frames_decoded_power_efficient) {
  GetForwardingInterface()->UpdateRecord(std::move(frames_decoded), std::move(frames_dropped), std::move(frames_decoded_power_efficient));
}
VideoDecodeStatsRecorderAsyncWaiter::VideoDecodeStatsRecorderAsyncWaiter(
    VideoDecodeStatsRecorder* proxy) : proxy_(proxy) {}

VideoDecodeStatsRecorderAsyncWaiter::~VideoDecodeStatsRecorderAsyncWaiter() = default;


}  // namespace blink
}  // namespace mojom
}  // namespace media

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif