// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_BLINK_H_
#define SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/network/public/interfaces/network_service.mojom-shared.h"
#include "services/network/public/interfaces/cookie_manager.mojom-blink.h"
#include "services/network/public/interfaces/network_change_manager.mojom-blink.h"
#include "services/network/public/interfaces/proxy_config.mojom-blink.h"
#include "services/network/public/interfaces/url_loader.mojom-blink.h"
#include "services/network/public/interfaces/url_loader_factory.mojom-blink.h"
#include "services/network/public/interfaces/restricted_cookie_manager.mojom-blink.h"
#include "mojo/common/file_path.mojom-blink.h"
#include "mojo/common/time.mojom-blink.h"
#include "url/mojo/url.mojom-blink.h"
#include "services/proxy_resolver/public/interfaces/proxy_resolver.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace network {
namespace mojom {
namespace blink {
class NetworkContext;
using NetworkContextPtr = mojo::InterfacePtr<NetworkContext>;
using NetworkContextPtrInfo = mojo::InterfacePtrInfo<NetworkContext>;
using ThreadSafeNetworkContextPtr =
    mojo::ThreadSafeInterfacePtr<NetworkContext>;
using NetworkContextRequest = mojo::InterfaceRequest<NetworkContext>;
using NetworkContextAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkContext>;
using ThreadSafeNetworkContextAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkContext>;
using NetworkContextAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkContext>;
using NetworkContextAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkContext>;

class NetworkServiceClient;
using NetworkServiceClientPtr = mojo::InterfacePtr<NetworkServiceClient>;
using NetworkServiceClientPtrInfo = mojo::InterfacePtrInfo<NetworkServiceClient>;
using ThreadSafeNetworkServiceClientPtr =
    mojo::ThreadSafeInterfacePtr<NetworkServiceClient>;
using NetworkServiceClientRequest = mojo::InterfaceRequest<NetworkServiceClient>;
using NetworkServiceClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkServiceClient>;
using ThreadSafeNetworkServiceClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkServiceClient>;
using NetworkServiceClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkServiceClient>;
using NetworkServiceClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkServiceClient>;

class NetworkService;
using NetworkServicePtr = mojo::InterfacePtr<NetworkService>;
using NetworkServicePtrInfo = mojo::InterfacePtrInfo<NetworkService>;
using ThreadSafeNetworkServicePtr =
    mojo::ThreadSafeInterfacePtr<NetworkService>;
using NetworkServiceRequest = mojo::InterfaceRequest<NetworkService>;
using NetworkServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkService>;
using ThreadSafeNetworkServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkService>;
using NetworkServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkService>;
using NetworkServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkService>;

class NetworkContextParams;
using NetworkContextParamsPtr = mojo::StructPtr<NetworkContextParams>;

class NetworkConditions;
using NetworkConditionsPtr = mojo::StructPtr<NetworkConditions>;


class NetworkContextProxy;

template <typename ImplRefTraits>
class NetworkContextStub;

class NetworkContextRequestValidator;
class NetworkContextResponseValidator;

class BLINK_PLATFORM_EXPORT NetworkContext
    : public NetworkContextInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = NetworkContextProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkContextStub<ImplRefTraits>;

  using RequestValidator_ = NetworkContextRequestValidator;
  using ResponseValidator_ = NetworkContextResponseValidator;
  enum MethodMinVersions : uint32_t {
    kCreateURLLoaderFactoryMinVersion = 0,
    kHandleViewCacheRequestMinVersion = 0,
    kGetCookieManagerMinVersion = 0,
    kGetRestrictedCookieManagerMinVersion = 0,
    kClearNetworkingHistorySinceMinVersion = 0,
    kSetNetworkConditionsMinVersion = 0,
    kAddHSTSForTestingMinVersion = 0,
  };
  virtual ~NetworkContext() {}

  virtual void CreateURLLoaderFactory(::network::mojom::blink::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) = 0;

  virtual void HandleViewCacheRequest(const ::blink::KURL& url, ::network::mojom::blink::URLLoaderClientPtr client) = 0;

  virtual void GetCookieManager(::network::mojom::blink::CookieManagerRequest cookie_manager) = 0;

  virtual void GetRestrictedCookieManager(::network::mojom::blink::RestrictedCookieManagerRequest restricted_cookie_manager, int32_t render_process_id, int32_t render_frame_id) = 0;


  using ClearNetworkingHistorySinceCallback = base::OnceCallback<void()>;
  virtual void ClearNetworkingHistorySince(WTF::Time start_time, ClearNetworkingHistorySinceCallback callback) = 0;

  virtual void SetNetworkConditions(const WTF::String& profile_id, NetworkConditionsPtr conditions) = 0;

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool AddHSTSForTesting(const WTF::String& host, WTF::Time expiry, bool include_subdomains);

  using AddHSTSForTestingCallback = base::OnceCallback<void()>;
  virtual void AddHSTSForTesting(const WTF::String& host, WTF::Time expiry, bool include_subdomains, AddHSTSForTestingCallback callback) = 0;
};
class BLINK_PLATFORM_EXPORT NetworkContextInterceptorForTesting : public NetworkContext {
  virtual NetworkContext* GetForwardingInterface() = 0;
  void CreateURLLoaderFactory(::network::mojom::blink::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) override;
  void HandleViewCacheRequest(const ::blink::KURL& url, ::network::mojom::blink::URLLoaderClientPtr client) override;
  void GetCookieManager(::network::mojom::blink::CookieManagerRequest cookie_manager) override;
  void GetRestrictedCookieManager(::network::mojom::blink::RestrictedCookieManagerRequest restricted_cookie_manager, int32_t render_process_id, int32_t render_frame_id) override;
  void ClearNetworkingHistorySince(WTF::Time start_time, ClearNetworkingHistorySinceCallback callback) override;
  void SetNetworkConditions(const WTF::String& profile_id, NetworkConditionsPtr conditions) override;
  void AddHSTSForTesting(const WTF::String& host, WTF::Time expiry, bool include_subdomains, AddHSTSForTestingCallback callback) override;
};
class BLINK_PLATFORM_EXPORT NetworkContextAsyncWaiter {
 public:
  explicit NetworkContextAsyncWaiter(NetworkContext* proxy);
  ~NetworkContextAsyncWaiter();
  void ClearNetworkingHistorySince(
      WTF::Time start_time);
  void AddHSTSForTesting(
      const WTF::String& host, WTF::Time expiry, bool include_subdomains);

 private:
  NetworkContext* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextAsyncWaiter);
};

class NetworkServiceClientProxy;

template <typename ImplRefTraits>
class NetworkServiceClientStub;

class NetworkServiceClientRequestValidator;
class NetworkServiceClientResponseValidator;

class BLINK_PLATFORM_EXPORT NetworkServiceClient
    : public NetworkServiceClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NetworkServiceClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkServiceClientStub<ImplRefTraits>;

  using RequestValidator_ = NetworkServiceClientRequestValidator;
  using ResponseValidator_ = NetworkServiceClientResponseValidator;
  enum MethodMinVersions : uint32_t {
    kOnSSLCertificateErrorMinVersion = 0,
  };
  virtual ~NetworkServiceClient() {}


  using OnSSLCertificateErrorCallback = base::OnceCallback<void(int32_t)>;
  virtual void OnSSLCertificateError(int32_t resource_type, const ::blink::KURL& url, uint32_t process_id, uint32_t routing_id, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, OnSSLCertificateErrorCallback callback) = 0;
};
class BLINK_PLATFORM_EXPORT NetworkServiceClientInterceptorForTesting : public NetworkServiceClient {
  virtual NetworkServiceClient* GetForwardingInterface() = 0;
  void OnSSLCertificateError(int32_t resource_type, const ::blink::KURL& url, uint32_t process_id, uint32_t routing_id, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, OnSSLCertificateErrorCallback callback) override;
};
class BLINK_PLATFORM_EXPORT NetworkServiceClientAsyncWaiter {
 public:
  explicit NetworkServiceClientAsyncWaiter(NetworkServiceClient* proxy);
  ~NetworkServiceClientAsyncWaiter();
  void OnSSLCertificateError(
      int32_t resource_type, const ::blink::KURL& url, uint32_t process_id, uint32_t routing_id, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, int32_t* out_net_error);

 private:
  NetworkServiceClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientAsyncWaiter);
};

class NetworkServiceProxy;

template <typename ImplRefTraits>
class NetworkServiceStub;

class NetworkServiceRequestValidator;

class BLINK_PLATFORM_EXPORT NetworkService
    : public NetworkServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NetworkServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkServiceStub<ImplRefTraits>;

  using RequestValidator_ = NetworkServiceRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetClientMinVersion = 0,
    kCreateNetworkContextMinVersion = 0,
    kDisableQuicMinVersion = 0,
    kSetRawHeadersAccessMinVersion = 0,
    kGetNetworkChangeManagerMinVersion = 0,
  };
  virtual ~NetworkService() {}

  virtual void SetClient(NetworkServiceClientPtr client) = 0;

  virtual void CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) = 0;

  virtual void DisableQuic() = 0;

  virtual void SetRawHeadersAccess(uint32_t process_id, bool allow) = 0;

  virtual void GetNetworkChangeManager(::network::mojom::blink::NetworkChangeManagerRequest network_change_manager) = 0;
};
class BLINK_PLATFORM_EXPORT NetworkServiceInterceptorForTesting : public NetworkService {
  virtual NetworkService* GetForwardingInterface() = 0;
  void SetClient(NetworkServiceClientPtr client) override;
  void CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) override;
  void DisableQuic() override;
  void SetRawHeadersAccess(uint32_t process_id, bool allow) override;
  void GetNetworkChangeManager(::network::mojom::blink::NetworkChangeManagerRequest network_change_manager) override;
};
class BLINK_PLATFORM_EXPORT NetworkServiceAsyncWaiter {
 public:
  explicit NetworkServiceAsyncWaiter(NetworkService* proxy);
  ~NetworkServiceAsyncWaiter();

 private:
  NetworkService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceAsyncWaiter);
};

class BLINK_PLATFORM_EXPORT NetworkContextProxy
    : public NetworkContext {
 public:
  explicit NetworkContextProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateURLLoaderFactory(::network::mojom::blink::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) final;
  void HandleViewCacheRequest(const ::blink::KURL& url, ::network::mojom::blink::URLLoaderClientPtr client) final;
  void GetCookieManager(::network::mojom::blink::CookieManagerRequest cookie_manager) final;
  void GetRestrictedCookieManager(::network::mojom::blink::RestrictedCookieManagerRequest restricted_cookie_manager, int32_t render_process_id, int32_t render_frame_id) final;
  void ClearNetworkingHistorySince(WTF::Time start_time, ClearNetworkingHistorySinceCallback callback) final;
  void SetNetworkConditions(const WTF::String& profile_id, NetworkConditionsPtr conditions) final;
  bool AddHSTSForTesting(const WTF::String& host, WTF::Time expiry, bool include_subdomains) final;
  void AddHSTSForTesting(const WTF::String& host, WTF::Time expiry, bool include_subdomains, AddHSTSForTestingCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT NetworkServiceClientProxy
    : public NetworkServiceClient {
 public:
  explicit NetworkServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnSSLCertificateError(int32_t resource_type, const ::blink::KURL& url, uint32_t process_id, uint32_t routing_id, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, OnSSLCertificateErrorCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT NetworkServiceProxy
    : public NetworkService {
 public:
  explicit NetworkServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetClient(NetworkServiceClientPtr client) final;
  void CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) final;
  void DisableQuic() final;
  void SetRawHeadersAccess(uint32_t process_id, bool allow) final;
  void GetNetworkChangeManager(::network::mojom::blink::NetworkChangeManagerRequest network_change_manager) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT NetworkContextStubDispatch {
 public:
  static bool Accept(NetworkContext* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkContext* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkContext>>
class NetworkContextStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkContextStub() {}
  ~NetworkContextStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkContextStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkContextStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT NetworkServiceClientStubDispatch {
 public:
  static bool Accept(NetworkServiceClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkServiceClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkServiceClient>>
class NetworkServiceClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkServiceClientStub() {}
  ~NetworkServiceClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT NetworkServiceStubDispatch {
 public:
  static bool Accept(NetworkService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkService>>
class NetworkServiceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkServiceStub() {}
  ~NetworkServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT NetworkContextRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT NetworkServiceClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT NetworkServiceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT NetworkContextResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT NetworkServiceClientResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};






class BLINK_PLATFORM_EXPORT NetworkContextParams {
 public:
  using DataView = NetworkContextParamsDataView;
  using Data_ = internal::NetworkContextParams_Data;

  template <typename... Args>
  static NetworkContextParamsPtr New(Args&&... args) {
    return NetworkContextParamsPtr(
        WTF::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static NetworkContextParamsPtr From(const U& u) {
    return mojo::TypeConverter<NetworkContextParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, NetworkContextParams>::Convert(*this);
  }


  NetworkContextParams();

  NetworkContextParams(
      const WTF::String& context_name,
      bool enable_brotli,
      const WTF::String& quic_user_agent_id,
      ::proxy_resolver::mojom::blink::ProxyResolverFactoryPtrInfo proxy_resolver_factory,
      const WTF::Optional<base::FilePath>& cookie_path,
      bool restore_old_session_cookies,
      bool persist_session_cookies,
      const WTF::Optional<base::FilePath>& channel_id_path,
      bool http_cache_enabled,
      int32_t http_cache_max_size,
      const WTF::Optional<base::FilePath>& http_cache_path,
      const WTF::Optional<base::FilePath>& http_server_properties_path,
      bool enable_data_url_support,
      bool enable_file_url_support,
      bool enable_ftp_url_support,
      bool http_09_on_non_default_ports_enabled,
      ::network::mojom::blink::ProxyConfigPtr initial_proxy_config,
      ::network::mojom::blink::ProxyConfigClientRequest proxy_config_client_request,
      ::network::mojom::blink::ProxyConfigPollerClientPtrInfo proxy_config_poller_client);

  ~NetworkContextParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = NetworkContextParamsPtr>
  NetworkContextParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, NetworkContextParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        NetworkContextParams::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        NetworkContextParams::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::NetworkContextParams_UnserializedMessageContext<
            UserType, NetworkContextParams::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<NetworkContextParams::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return NetworkContextParams::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::NetworkContextParams_UnserializedMessageContext<
            UserType, NetworkContextParams::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<NetworkContextParams::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  WTF::String context_name;
  bool enable_brotli;
  WTF::String quic_user_agent_id;
  ::proxy_resolver::mojom::blink::ProxyResolverFactoryPtrInfo proxy_resolver_factory;
  WTF::Optional<base::FilePath> cookie_path;
  bool restore_old_session_cookies;
  bool persist_session_cookies;
  WTF::Optional<base::FilePath> channel_id_path;
  bool http_cache_enabled;
  int32_t http_cache_max_size;
  WTF::Optional<base::FilePath> http_cache_path;
  WTF::Optional<base::FilePath> http_server_properties_path;
  bool enable_data_url_support;
  bool enable_file_url_support;
  bool enable_ftp_url_support;
  bool http_09_on_non_default_ports_enabled;
  ::network::mojom::blink::ProxyConfigPtr initial_proxy_config;
  ::network::mojom::blink::ProxyConfigClientRequest proxy_config_client_request;
  ::network::mojom::blink::ProxyConfigPollerClientPtrInfo proxy_config_poller_client;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(NetworkContextParams);
};


class BLINK_PLATFORM_EXPORT NetworkConditions {
 public:
  using DataView = NetworkConditionsDataView;
  using Data_ = internal::NetworkConditions_Data;

  template <typename... Args>
  static NetworkConditionsPtr New(Args&&... args) {
    return NetworkConditionsPtr(
        WTF::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static NetworkConditionsPtr From(const U& u) {
    return mojo::TypeConverter<NetworkConditionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, NetworkConditions>::Convert(*this);
  }


  NetworkConditions();

  NetworkConditions(
      bool offline,
      WTF::TimeDelta latency,
      double download_throughput,
      double upload_throughput);

  ~NetworkConditions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = NetworkConditionsPtr>
  NetworkConditionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, NetworkConditions>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        NetworkConditions::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        NetworkConditions::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::NetworkConditions_UnserializedMessageContext<
            UserType, NetworkConditions::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<NetworkConditions::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return NetworkConditions::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::NetworkConditions_UnserializedMessageContext<
            UserType, NetworkConditions::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<NetworkConditions::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  bool offline;
  WTF::TimeDelta latency;
  double download_throughput;
  double upload_throughput;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
NetworkContextParamsPtr NetworkContextParams::Clone() const {
  return New(
      mojo::Clone(context_name),
      mojo::Clone(enable_brotli),
      mojo::Clone(quic_user_agent_id),
      mojo::Clone(proxy_resolver_factory),
      mojo::Clone(cookie_path),
      mojo::Clone(restore_old_session_cookies),
      mojo::Clone(persist_session_cookies),
      mojo::Clone(channel_id_path),
      mojo::Clone(http_cache_enabled),
      mojo::Clone(http_cache_max_size),
      mojo::Clone(http_cache_path),
      mojo::Clone(http_server_properties_path),
      mojo::Clone(enable_data_url_support),
      mojo::Clone(enable_file_url_support),
      mojo::Clone(enable_ftp_url_support),
      mojo::Clone(http_09_on_non_default_ports_enabled),
      mojo::Clone(initial_proxy_config),
      mojo::Clone(proxy_config_client_request),
      mojo::Clone(proxy_config_poller_client)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, NetworkContextParams>::value>::type*>
bool NetworkContextParams::Equals(const T& other) const {
  if (!mojo::Equals(this->context_name, other.context_name))
    return false;
  if (!mojo::Equals(this->enable_brotli, other.enable_brotli))
    return false;
  if (!mojo::Equals(this->quic_user_agent_id, other.quic_user_agent_id))
    return false;
  if (!mojo::Equals(this->proxy_resolver_factory, other.proxy_resolver_factory))
    return false;
  if (!mojo::Equals(this->cookie_path, other.cookie_path))
    return false;
  if (!mojo::Equals(this->restore_old_session_cookies, other.restore_old_session_cookies))
    return false;
  if (!mojo::Equals(this->persist_session_cookies, other.persist_session_cookies))
    return false;
  if (!mojo::Equals(this->channel_id_path, other.channel_id_path))
    return false;
  if (!mojo::Equals(this->http_cache_enabled, other.http_cache_enabled))
    return false;
  if (!mojo::Equals(this->http_cache_max_size, other.http_cache_max_size))
    return false;
  if (!mojo::Equals(this->http_cache_path, other.http_cache_path))
    return false;
  if (!mojo::Equals(this->http_server_properties_path, other.http_server_properties_path))
    return false;
  if (!mojo::Equals(this->enable_data_url_support, other.enable_data_url_support))
    return false;
  if (!mojo::Equals(this->enable_file_url_support, other.enable_file_url_support))
    return false;
  if (!mojo::Equals(this->enable_ftp_url_support, other.enable_ftp_url_support))
    return false;
  if (!mojo::Equals(this->http_09_on_non_default_ports_enabled, other.http_09_on_non_default_ports_enabled))
    return false;
  if (!mojo::Equals(this->initial_proxy_config, other.initial_proxy_config))
    return false;
  if (!mojo::Equals(this->proxy_config_client_request, other.proxy_config_client_request))
    return false;
  if (!mojo::Equals(this->proxy_config_poller_client, other.proxy_config_poller_client))
    return false;
  return true;
}
template <typename StructPtrType>
NetworkConditionsPtr NetworkConditions::Clone() const {
  return New(
      mojo::Clone(offline),
      mojo::Clone(latency),
      mojo::Clone(download_throughput),
      mojo::Clone(upload_throughput)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, NetworkConditions>::value>::type*>
bool NetworkConditions::Equals(const T& other) const {
  if (!mojo::Equals(this->offline, other.offline))
    return false;
  if (!mojo::Equals(this->latency, other.latency))
    return false;
  if (!mojo::Equals(this->download_throughput, other.download_throughput))
    return false;
  if (!mojo::Equals(this->upload_throughput, other.upload_throughput))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace network

namespace mojo {


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::network::mojom::blink::NetworkContextParams::DataView,
                                         ::network::mojom::blink::NetworkContextParamsPtr> {
  static bool IsNull(const ::network::mojom::blink::NetworkContextParamsPtr& input) { return !input; }
  static void SetToNull(::network::mojom::blink::NetworkContextParamsPtr* output) { output->reset(); }

  static const decltype(::network::mojom::blink::NetworkContextParams::context_name)& context_name(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->context_name;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::enable_brotli) enable_brotli(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->enable_brotli;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::quic_user_agent_id)& quic_user_agent_id(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->quic_user_agent_id;
  }

  static  decltype(::network::mojom::blink::NetworkContextParams::proxy_resolver_factory)& proxy_resolver_factory(
       ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->proxy_resolver_factory;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::cookie_path)& cookie_path(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->cookie_path;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::restore_old_session_cookies) restore_old_session_cookies(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->restore_old_session_cookies;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::persist_session_cookies) persist_session_cookies(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->persist_session_cookies;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::channel_id_path)& channel_id_path(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->channel_id_path;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::http_cache_enabled) http_cache_enabled(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->http_cache_enabled;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::http_cache_max_size) http_cache_max_size(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->http_cache_max_size;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::http_cache_path)& http_cache_path(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->http_cache_path;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::http_server_properties_path)& http_server_properties_path(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->http_server_properties_path;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::enable_data_url_support) enable_data_url_support(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->enable_data_url_support;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::enable_file_url_support) enable_file_url_support(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->enable_file_url_support;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::enable_ftp_url_support) enable_ftp_url_support(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->enable_ftp_url_support;
  }

  static decltype(::network::mojom::blink::NetworkContextParams::http_09_on_non_default_ports_enabled) http_09_on_non_default_ports_enabled(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->http_09_on_non_default_ports_enabled;
  }

  static const decltype(::network::mojom::blink::NetworkContextParams::initial_proxy_config)& initial_proxy_config(
      const ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->initial_proxy_config;
  }

  static  decltype(::network::mojom::blink::NetworkContextParams::proxy_config_client_request)& proxy_config_client_request(
       ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->proxy_config_client_request;
  }

  static  decltype(::network::mojom::blink::NetworkContextParams::proxy_config_poller_client)& proxy_config_poller_client(
       ::network::mojom::blink::NetworkContextParamsPtr& input) {
    return input->proxy_config_poller_client;
  }

  static bool Read(::network::mojom::blink::NetworkContextParams::DataView input, ::network::mojom::blink::NetworkContextParamsPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::network::mojom::blink::NetworkConditions::DataView,
                                         ::network::mojom::blink::NetworkConditionsPtr> {
  static bool IsNull(const ::network::mojom::blink::NetworkConditionsPtr& input) { return !input; }
  static void SetToNull(::network::mojom::blink::NetworkConditionsPtr* output) { output->reset(); }

  static decltype(::network::mojom::blink::NetworkConditions::offline) offline(
      const ::network::mojom::blink::NetworkConditionsPtr& input) {
    return input->offline;
  }

  static const decltype(::network::mojom::blink::NetworkConditions::latency)& latency(
      const ::network::mojom::blink::NetworkConditionsPtr& input) {
    return input->latency;
  }

  static decltype(::network::mojom::blink::NetworkConditions::download_throughput) download_throughput(
      const ::network::mojom::blink::NetworkConditionsPtr& input) {
    return input->download_throughput;
  }

  static decltype(::network::mojom::blink::NetworkConditions::upload_throughput) upload_throughput(
      const ::network::mojom::blink::NetworkConditionsPtr& input) {
    return input->upload_throughput;
  }

  static bool Read(::network::mojom::blink::NetworkConditions::DataView input, ::network::mojom::blink::NetworkConditionsPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_BLINK_H_