// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_SHARED_INTERNAL_H_
#define SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "services/network/public/interfaces/cookie_manager.mojom-shared-internal.h"
#include "services/network/public/interfaces/network_change_manager.mojom-shared-internal.h"
#include "services/network/public/interfaces/proxy_config.mojom-shared-internal.h"
#include "services/network/public/interfaces/url_loader.mojom-shared-internal.h"
#include "services/network/public/interfaces/url_loader_factory.mojom-shared-internal.h"
#include "services/network/public/interfaces/restricted_cookie_manager.mojom-shared-internal.h"
#include "mojo/common/file_path.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared-internal.h"
#include "url/mojo/url.mojom-shared-internal.h"
#include "services/proxy_resolver/public/interfaces/proxy_resolver.mojom-shared-internal.h"
#include "mojo/public/interfaces/bindings/native_struct.mojom-shared-internal.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace network {
namespace mojom {
namespace internal {
class NetworkContextParams_Data;
class NetworkConditions_Data;

#pragma pack(push, 1)
class  NetworkContextParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContextParams_Data));
      new (data()) NetworkContextParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContextParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContextParams_Data>(index_);
    }
    NetworkContextParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> context_name;
  uint8_t enable_brotli : 1;
  uint8_t restore_old_session_cookies : 1;
  uint8_t persist_session_cookies : 1;
  uint8_t http_cache_enabled : 1;
  uint8_t enable_data_url_support : 1;
  uint8_t enable_file_url_support : 1;
  uint8_t enable_ftp_url_support : 1;
  uint8_t http_09_on_non_default_ports_enabled : 1;
  uint8_t pad8_[3];
  int32_t http_cache_max_size;
  mojo::internal::Pointer<mojo::internal::String_Data> quic_user_agent_id;
  mojo::internal::Interface_Data proxy_resolver_factory;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> cookie_path;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> channel_id_path;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> http_cache_path;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> http_server_properties_path;
  mojo::internal::Pointer<::network::mojom::internal::ProxyConfig_Data> initial_proxy_config;
  mojo::internal::Handle_Data proxy_config_client_request;
  mojo::internal::Interface_Data proxy_config_poller_client;
  uint8_t padfinal_[4];

 private:
  NetworkContextParams_Data();
  ~NetworkContextParams_Data() = delete;
};
static_assert(sizeof(NetworkContextParams_Data) == 96,
              "Bad sizeof(NetworkContextParams_Data)");
// Used by NetworkContextParams::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct NetworkContextParams_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  NetworkContextParams_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~NetworkContextParams_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    NetworkContextParams_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    NetworkContextParams_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class  NetworkConditions_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkConditions_Data));
      new (data()) NetworkConditions_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkConditions_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkConditions_Data>(index_);
    }
    NetworkConditions_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t offline : 1;
  uint8_t pad0_[7];
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> latency;
  double download_throughput;
  double upload_throughput;

 private:
  NetworkConditions_Data();
  ~NetworkConditions_Data() = delete;
};
static_assert(sizeof(NetworkConditions_Data) == 40,
              "Bad sizeof(NetworkConditions_Data)");
// Used by NetworkConditions::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct NetworkConditions_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  NetworkConditions_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~NetworkConditions_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    NetworkConditions_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    NetworkConditions_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
// The 1136152652 value is based on sha256(salt + "NetworkContext1").
constexpr uint32_t kNetworkContext_CreateURLLoaderFactory_Name = 1136152652;
class  NetworkContext_CreateURLLoaderFactory_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_CreateURLLoaderFactory_Params_Data));
      new (data()) NetworkContext_CreateURLLoaderFactory_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_CreateURLLoaderFactory_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_CreateURLLoaderFactory_Params_Data>(index_);
    }
    NetworkContext_CreateURLLoaderFactory_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data url_loader_factory;
  uint32_t process_id;

 private:
  NetworkContext_CreateURLLoaderFactory_Params_Data();
  ~NetworkContext_CreateURLLoaderFactory_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_CreateURLLoaderFactory_Params_Data) == 16,
              "Bad sizeof(NetworkContext_CreateURLLoaderFactory_Params_Data)");
// The 866141801 value is based on sha256(salt + "NetworkContext2").
constexpr uint32_t kNetworkContext_HandleViewCacheRequest_Name = 866141801;
class  NetworkContext_HandleViewCacheRequest_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_HandleViewCacheRequest_Params_Data));
      new (data()) NetworkContext_HandleViewCacheRequest_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_HandleViewCacheRequest_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_HandleViewCacheRequest_Params_Data>(index_);
    }
    NetworkContext_HandleViewCacheRequest_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> url;
  mojo::internal::Interface_Data client;

 private:
  NetworkContext_HandleViewCacheRequest_Params_Data();
  ~NetworkContext_HandleViewCacheRequest_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_HandleViewCacheRequest_Params_Data) == 24,
              "Bad sizeof(NetworkContext_HandleViewCacheRequest_Params_Data)");
// The 117534600 value is based on sha256(salt + "NetworkContext3").
constexpr uint32_t kNetworkContext_GetCookieManager_Name = 117534600;
class  NetworkContext_GetCookieManager_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_GetCookieManager_Params_Data));
      new (data()) NetworkContext_GetCookieManager_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_GetCookieManager_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_GetCookieManager_Params_Data>(index_);
    }
    NetworkContext_GetCookieManager_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data cookie_manager;
  uint8_t padfinal_[4];

 private:
  NetworkContext_GetCookieManager_Params_Data();
  ~NetworkContext_GetCookieManager_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_GetCookieManager_Params_Data) == 16,
              "Bad sizeof(NetworkContext_GetCookieManager_Params_Data)");
// The 1557038038 value is based on sha256(salt + "NetworkContext4").
constexpr uint32_t kNetworkContext_GetRestrictedCookieManager_Name = 1557038038;
class  NetworkContext_GetRestrictedCookieManager_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_GetRestrictedCookieManager_Params_Data));
      new (data()) NetworkContext_GetRestrictedCookieManager_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_GetRestrictedCookieManager_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_GetRestrictedCookieManager_Params_Data>(index_);
    }
    NetworkContext_GetRestrictedCookieManager_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data restricted_cookie_manager;
  int32_t render_process_id;
  int32_t render_frame_id;
  uint8_t padfinal_[4];

 private:
  NetworkContext_GetRestrictedCookieManager_Params_Data();
  ~NetworkContext_GetRestrictedCookieManager_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_GetRestrictedCookieManager_Params_Data) == 24,
              "Bad sizeof(NetworkContext_GetRestrictedCookieManager_Params_Data)");
// The 1763555324 value is based on sha256(salt + "NetworkContext5").
constexpr uint32_t kNetworkContext_ClearNetworkingHistorySince_Name = 1763555324;
class  NetworkContext_ClearNetworkingHistorySince_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_ClearNetworkingHistorySince_Params_Data));
      new (data()) NetworkContext_ClearNetworkingHistorySince_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_ClearNetworkingHistorySince_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_ClearNetworkingHistorySince_Params_Data>(index_);
    }
    NetworkContext_ClearNetworkingHistorySince_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> start_time;

 private:
  NetworkContext_ClearNetworkingHistorySince_Params_Data();
  ~NetworkContext_ClearNetworkingHistorySince_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_ClearNetworkingHistorySince_Params_Data) == 16,
              "Bad sizeof(NetworkContext_ClearNetworkingHistorySince_Params_Data)");
class  NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data));
      new (data()) NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data>(index_);
    }
    NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data();
  ~NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data() = delete;
};
static_assert(sizeof(NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data) == 8,
              "Bad sizeof(NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data)");
// The 1947291256 value is based on sha256(salt + "NetworkContext6").
constexpr uint32_t kNetworkContext_SetNetworkConditions_Name = 1947291256;
class  NetworkContext_SetNetworkConditions_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_SetNetworkConditions_Params_Data));
      new (data()) NetworkContext_SetNetworkConditions_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_SetNetworkConditions_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_SetNetworkConditions_Params_Data>(index_);
    }
    NetworkContext_SetNetworkConditions_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> profile_id;
  mojo::internal::Pointer<internal::NetworkConditions_Data> conditions;

 private:
  NetworkContext_SetNetworkConditions_Params_Data();
  ~NetworkContext_SetNetworkConditions_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_SetNetworkConditions_Params_Data) == 24,
              "Bad sizeof(NetworkContext_SetNetworkConditions_Params_Data)");
// The 1661453500 value is based on sha256(salt + "NetworkContext7").
constexpr uint32_t kNetworkContext_AddHSTSForTesting_Name = 1661453500;
class  NetworkContext_AddHSTSForTesting_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_AddHSTSForTesting_Params_Data));
      new (data()) NetworkContext_AddHSTSForTesting_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_AddHSTSForTesting_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_AddHSTSForTesting_Params_Data>(index_);
    }
    NetworkContext_AddHSTSForTesting_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> host;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> expiry;
  uint8_t include_subdomains : 1;
  uint8_t padfinal_[7];

 private:
  NetworkContext_AddHSTSForTesting_Params_Data();
  ~NetworkContext_AddHSTSForTesting_Params_Data() = delete;
};
static_assert(sizeof(NetworkContext_AddHSTSForTesting_Params_Data) == 32,
              "Bad sizeof(NetworkContext_AddHSTSForTesting_Params_Data)");
class  NetworkContext_AddHSTSForTesting_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkContext_AddHSTSForTesting_ResponseParams_Data));
      new (data()) NetworkContext_AddHSTSForTesting_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkContext_AddHSTSForTesting_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkContext_AddHSTSForTesting_ResponseParams_Data>(index_);
    }
    NetworkContext_AddHSTSForTesting_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  NetworkContext_AddHSTSForTesting_ResponseParams_Data();
  ~NetworkContext_AddHSTSForTesting_ResponseParams_Data() = delete;
};
static_assert(sizeof(NetworkContext_AddHSTSForTesting_ResponseParams_Data) == 8,
              "Bad sizeof(NetworkContext_AddHSTSForTesting_ResponseParams_Data)");
// The 1071859417 value is based on sha256(salt + "NetworkServiceClient1").
constexpr uint32_t kNetworkServiceClient_OnSSLCertificateError_Name = 1071859417;
class  NetworkServiceClient_OnSSLCertificateError_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkServiceClient_OnSSLCertificateError_Params_Data));
      new (data()) NetworkServiceClient_OnSSLCertificateError_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkServiceClient_OnSSLCertificateError_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkServiceClient_OnSSLCertificateError_Params_Data>(index_);
    }
    NetworkServiceClient_OnSSLCertificateError_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t resource_type;
  uint32_t process_id;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> url;
  uint32_t routing_id;
  uint8_t fatal : 1;
  uint8_t pad4_[3];
  mojo::internal::Pointer<::network::mojom::internal::SSLInfo_Data> ssl_info;

 private:
  NetworkServiceClient_OnSSLCertificateError_Params_Data();
  ~NetworkServiceClient_OnSSLCertificateError_Params_Data() = delete;
};
static_assert(sizeof(NetworkServiceClient_OnSSLCertificateError_Params_Data) == 40,
              "Bad sizeof(NetworkServiceClient_OnSSLCertificateError_Params_Data)");
class  NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data));
      new (data()) NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data>(index_);
    }
    NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t net_error;
  uint8_t padfinal_[4];

 private:
  NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data();
  ~NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data() = delete;
};
static_assert(sizeof(NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data) == 16,
              "Bad sizeof(NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data)");
// The 1313026358 value is based on sha256(salt + "NetworkService1").
constexpr uint32_t kNetworkService_SetClient_Name = 1313026358;
class  NetworkService_SetClient_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkService_SetClient_Params_Data));
      new (data()) NetworkService_SetClient_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkService_SetClient_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkService_SetClient_Params_Data>(index_);
    }
    NetworkService_SetClient_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data client;

 private:
  NetworkService_SetClient_Params_Data();
  ~NetworkService_SetClient_Params_Data() = delete;
};
static_assert(sizeof(NetworkService_SetClient_Params_Data) == 16,
              "Bad sizeof(NetworkService_SetClient_Params_Data)");
// The 788528108 value is based on sha256(salt + "NetworkService2").
constexpr uint32_t kNetworkService_CreateNetworkContext_Name = 788528108;
class  NetworkService_CreateNetworkContext_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkService_CreateNetworkContext_Params_Data));
      new (data()) NetworkService_CreateNetworkContext_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkService_CreateNetworkContext_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkService_CreateNetworkContext_Params_Data>(index_);
    }
    NetworkService_CreateNetworkContext_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data context;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::NetworkContextParams_Data> params;

 private:
  NetworkService_CreateNetworkContext_Params_Data();
  ~NetworkService_CreateNetworkContext_Params_Data() = delete;
};
static_assert(sizeof(NetworkService_CreateNetworkContext_Params_Data) == 24,
              "Bad sizeof(NetworkService_CreateNetworkContext_Params_Data)");
// The 1143572736 value is based on sha256(salt + "NetworkService3").
constexpr uint32_t kNetworkService_DisableQuic_Name = 1143572736;
class  NetworkService_DisableQuic_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkService_DisableQuic_Params_Data));
      new (data()) NetworkService_DisableQuic_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkService_DisableQuic_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkService_DisableQuic_Params_Data>(index_);
    }
    NetworkService_DisableQuic_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  NetworkService_DisableQuic_Params_Data();
  ~NetworkService_DisableQuic_Params_Data() = delete;
};
static_assert(sizeof(NetworkService_DisableQuic_Params_Data) == 8,
              "Bad sizeof(NetworkService_DisableQuic_Params_Data)");
// The 1987496256 value is based on sha256(salt + "NetworkService4").
constexpr uint32_t kNetworkService_SetRawHeadersAccess_Name = 1987496256;
class  NetworkService_SetRawHeadersAccess_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkService_SetRawHeadersAccess_Params_Data));
      new (data()) NetworkService_SetRawHeadersAccess_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkService_SetRawHeadersAccess_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkService_SetRawHeadersAccess_Params_Data>(index_);
    }
    NetworkService_SetRawHeadersAccess_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t process_id;
  uint8_t allow : 1;
  uint8_t padfinal_[3];

 private:
  NetworkService_SetRawHeadersAccess_Params_Data();
  ~NetworkService_SetRawHeadersAccess_Params_Data() = delete;
};
static_assert(sizeof(NetworkService_SetRawHeadersAccess_Params_Data) == 16,
              "Bad sizeof(NetworkService_SetRawHeadersAccess_Params_Data)");
// The 646322008 value is based on sha256(salt + "NetworkService5").
constexpr uint32_t kNetworkService_GetNetworkChangeManager_Name = 646322008;
class  NetworkService_GetNetworkChangeManager_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(NetworkService_GetNetworkChangeManager_Params_Data));
      new (data()) NetworkService_GetNetworkChangeManager_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    NetworkService_GetNetworkChangeManager_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<NetworkService_GetNetworkChangeManager_Params_Data>(index_);
    }
    NetworkService_GetNetworkChangeManager_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data network_change_manager;
  uint8_t padfinal_[4];

 private:
  NetworkService_GetNetworkChangeManager_Params_Data();
  ~NetworkService_GetNetworkChangeManager_Params_Data() = delete;
};
static_assert(sizeof(NetworkService_GetNetworkChangeManager_Params_Data) == 16,
              "Bad sizeof(NetworkService_GetNetworkChangeManager_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace network

#endif  // SERVICES_NETWORK_PUBLIC_INTERFACES_NETWORK_SERVICE_MOJOM_SHARED_INTERNAL_H_