// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_INTERFACES_PROXY_CONFIG_MOJOM_H_
#define SERVICES_NETWORK_PUBLIC_INTERFACES_PROXY_CONFIG_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/network/public/interfaces/proxy_config.mojom-shared.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "net/proxy/proxy_bypass_rules.h"
#include "net/proxy/proxy_config.h"
#include "net/proxy/proxy_config_source.h"
#include "net/proxy/proxy_list.h"


namespace network {
namespace mojom {
class ProxyConfigClient;
using ProxyConfigClientPtr = mojo::InterfacePtr<ProxyConfigClient>;
using ProxyConfigClientPtrInfo = mojo::InterfacePtrInfo<ProxyConfigClient>;
using ThreadSafeProxyConfigClientPtr =
    mojo::ThreadSafeInterfacePtr<ProxyConfigClient>;
using ProxyConfigClientRequest = mojo::InterfaceRequest<ProxyConfigClient>;
using ProxyConfigClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProxyConfigClient>;
using ThreadSafeProxyConfigClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProxyConfigClient>;
using ProxyConfigClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProxyConfigClient>;
using ProxyConfigClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProxyConfigClient>;

class ProxyConfigPollerClient;
using ProxyConfigPollerClientPtr = mojo::InterfacePtr<ProxyConfigPollerClient>;
using ProxyConfigPollerClientPtrInfo = mojo::InterfacePtrInfo<ProxyConfigPollerClient>;
using ThreadSafeProxyConfigPollerClientPtr =
    mojo::ThreadSafeInterfacePtr<ProxyConfigPollerClient>;
using ProxyConfigPollerClientRequest = mojo::InterfaceRequest<ProxyConfigPollerClient>;
using ProxyConfigPollerClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProxyConfigPollerClient>;
using ThreadSafeProxyConfigPollerClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProxyConfigPollerClient>;
using ProxyConfigPollerClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProxyConfigPollerClient>;
using ProxyConfigPollerClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProxyConfigPollerClient>;

class ProxyBypassRules;
using ProxyBypassRulesPtr = mojo::StructPtr<ProxyBypassRules>;

class ProxyList;
using ProxyListPtr = mojo::StructPtr<ProxyList>;

class ProxyRules;
using ProxyRulesPtr = mojo::StructPtr<ProxyRules>;

class ProxyConfig;
using ProxyConfigPtr = mojo::StructPtr<ProxyConfig>;


class ProxyConfigClientProxy;

template <typename ImplRefTraits>
class ProxyConfigClientStub;

class ProxyConfigClientRequestValidator;

class  ProxyConfigClient
    : public ProxyConfigClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProxyConfigClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProxyConfigClientStub<ImplRefTraits>;

  using RequestValidator_ = ProxyConfigClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnProxyConfigUpdatedMinVersion = 0,
  };
  virtual ~ProxyConfigClient() {}

  virtual void OnProxyConfigUpdated(const net::ProxyConfig& proxy_config) = 0;
};
class  ProxyConfigClientInterceptorForTesting : public ProxyConfigClient {
  virtual ProxyConfigClient* GetForwardingInterface() = 0;
  void OnProxyConfigUpdated(const net::ProxyConfig& proxy_config) override;
};
class  ProxyConfigClientAsyncWaiter {
 public:
  explicit ProxyConfigClientAsyncWaiter(ProxyConfigClient* proxy);
  ~ProxyConfigClientAsyncWaiter();

 private:
  ProxyConfigClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProxyConfigClientAsyncWaiter);
};

class ProxyConfigPollerClientProxy;

template <typename ImplRefTraits>
class ProxyConfigPollerClientStub;

class ProxyConfigPollerClientRequestValidator;

class  ProxyConfigPollerClient
    : public ProxyConfigPollerClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProxyConfigPollerClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProxyConfigPollerClientStub<ImplRefTraits>;

  using RequestValidator_ = ProxyConfigPollerClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnLazyProxyConfigPollMinVersion = 0,
  };
  virtual ~ProxyConfigPollerClient() {}

  virtual void OnLazyProxyConfigPoll() = 0;
};
class  ProxyConfigPollerClientInterceptorForTesting : public ProxyConfigPollerClient {
  virtual ProxyConfigPollerClient* GetForwardingInterface() = 0;
  void OnLazyProxyConfigPoll() override;
};
class  ProxyConfigPollerClientAsyncWaiter {
 public:
  explicit ProxyConfigPollerClientAsyncWaiter(ProxyConfigPollerClient* proxy);
  ~ProxyConfigPollerClientAsyncWaiter();

 private:
  ProxyConfigPollerClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProxyConfigPollerClientAsyncWaiter);
};

class  ProxyConfigClientProxy
    : public ProxyConfigClient {
 public:
  explicit ProxyConfigClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnProxyConfigUpdated(const net::ProxyConfig& proxy_config) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  ProxyConfigPollerClientProxy
    : public ProxyConfigPollerClient {
 public:
  explicit ProxyConfigPollerClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnLazyProxyConfigPoll() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ProxyConfigClientStubDispatch {
 public:
  static bool Accept(ProxyConfigClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProxyConfigClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProxyConfigClient>>
class ProxyConfigClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProxyConfigClientStub() {}
  ~ProxyConfigClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyConfigClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyConfigClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ProxyConfigPollerClientStubDispatch {
 public:
  static bool Accept(ProxyConfigPollerClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProxyConfigPollerClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProxyConfigPollerClient>>
class ProxyConfigPollerClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProxyConfigPollerClientStub() {}
  ~ProxyConfigPollerClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyConfigPollerClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyConfigPollerClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ProxyConfigClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ProxyConfigPollerClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};








class  ProxyBypassRules {
 public:
  using DataView = ProxyBypassRulesDataView;
  using Data_ = internal::ProxyBypassRules_Data;

  template <typename... Args>
  static ProxyBypassRulesPtr New(Args&&... args) {
    return ProxyBypassRulesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ProxyBypassRulesPtr From(const U& u) {
    return mojo::TypeConverter<ProxyBypassRulesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ProxyBypassRules>::Convert(*this);
  }


  ProxyBypassRules();

  explicit ProxyBypassRules(
      const std::vector<std::string>& rules);

  ~ProxyBypassRules();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ProxyBypassRulesPtr>
  ProxyBypassRulesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ProxyBypassRules>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ProxyBypassRules::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ProxyBypassRules::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ProxyBypassRules_UnserializedMessageContext<
            UserType, ProxyBypassRules::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ProxyBypassRules::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ProxyBypassRules::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ProxyBypassRules_UnserializedMessageContext<
            UserType, ProxyBypassRules::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ProxyBypassRules::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<std::string> rules;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class  ProxyList {
 public:
  using DataView = ProxyListDataView;
  using Data_ = internal::ProxyList_Data;

  template <typename... Args>
  static ProxyListPtr New(Args&&... args) {
    return ProxyListPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ProxyListPtr From(const U& u) {
    return mojo::TypeConverter<ProxyListPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ProxyList>::Convert(*this);
  }


  ProxyList();

  explicit ProxyList(
      const std::vector<std::string>& proxies);

  ~ProxyList();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ProxyListPtr>
  ProxyListPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ProxyList>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ProxyList::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ProxyList::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ProxyList_UnserializedMessageContext<
            UserType, ProxyList::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ProxyList::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ProxyList::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ProxyList_UnserializedMessageContext<
            UserType, ProxyList::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ProxyList::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<std::string> proxies;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class  ProxyRules {
 public:
  using DataView = ProxyRulesDataView;
  using Data_ = internal::ProxyRules_Data;

  template <typename... Args>
  static ProxyRulesPtr New(Args&&... args) {
    return ProxyRulesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ProxyRulesPtr From(const U& u) {
    return mojo::TypeConverter<ProxyRulesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ProxyRules>::Convert(*this);
  }


  ProxyRules();

  ProxyRules(
      const net::ProxyBypassRules& bypass_rules,
      bool reverse_bypass,
      net::ProxyConfig::ProxyRules::Type type,
      const net::ProxyList& single_proxies,
      const net::ProxyList& proxies_for_http,
      const net::ProxyList& proxies_for_https,
      const net::ProxyList& proxies_for_ftp,
      const net::ProxyList& fallback_proxies);

  ~ProxyRules();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ProxyRulesPtr>
  ProxyRulesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ProxyRules>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ProxyRules::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ProxyRules::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ProxyRules_UnserializedMessageContext<
            UserType, ProxyRules::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ProxyRules::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ProxyRules::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ProxyRules_UnserializedMessageContext<
            UserType, ProxyRules::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ProxyRules::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  net::ProxyBypassRules bypass_rules;
  bool reverse_bypass;
  net::ProxyConfig::ProxyRules::Type type;
  net::ProxyList single_proxies;
  net::ProxyList proxies_for_http;
  net::ProxyList proxies_for_https;
  net::ProxyList proxies_for_ftp;
  net::ProxyList fallback_proxies;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class  ProxyConfig {
 public:
  using DataView = ProxyConfigDataView;
  using Data_ = internal::ProxyConfig_Data;

  template <typename... Args>
  static ProxyConfigPtr New(Args&&... args) {
    return ProxyConfigPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ProxyConfigPtr From(const U& u) {
    return mojo::TypeConverter<ProxyConfigPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ProxyConfig>::Convert(*this);
  }


  ProxyConfig();

  ProxyConfig(
      bool auto_detect,
      const GURL& pac_url,
      bool pac_mandatory,
      ProxyRulesPtr proxy_rules,
      net::ProxyConfigSource source,
      int32_t id);

  ~ProxyConfig();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ProxyConfigPtr>
  ProxyConfigPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ProxyConfig>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ProxyConfig::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ProxyConfig::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ProxyConfig_UnserializedMessageContext<
            UserType, ProxyConfig::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ProxyConfig::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ProxyConfig::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ProxyConfig_UnserializedMessageContext<
            UserType, ProxyConfig::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ProxyConfig::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  bool auto_detect;
  GURL pac_url;
  bool pac_mandatory;
  ProxyRulesPtr proxy_rules;
  net::ProxyConfigSource source;
  int32_t id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ProxyConfig);
};

template <typename StructPtrType>
ProxyBypassRulesPtr ProxyBypassRules::Clone() const {
  return New(
      mojo::Clone(rules)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ProxyBypassRules>::value>::type*>
bool ProxyBypassRules::Equals(const T& other) const {
  if (!mojo::Equals(this->rules, other.rules))
    return false;
  return true;
}
template <typename StructPtrType>
ProxyListPtr ProxyList::Clone() const {
  return New(
      mojo::Clone(proxies)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ProxyList>::value>::type*>
bool ProxyList::Equals(const T& other) const {
  if (!mojo::Equals(this->proxies, other.proxies))
    return false;
  return true;
}
template <typename StructPtrType>
ProxyRulesPtr ProxyRules::Clone() const {
  return New(
      mojo::Clone(bypass_rules),
      mojo::Clone(reverse_bypass),
      mojo::Clone(type),
      mojo::Clone(single_proxies),
      mojo::Clone(proxies_for_http),
      mojo::Clone(proxies_for_https),
      mojo::Clone(proxies_for_ftp),
      mojo::Clone(fallback_proxies)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ProxyRules>::value>::type*>
bool ProxyRules::Equals(const T& other) const {
  if (!mojo::Equals(this->bypass_rules, other.bypass_rules))
    return false;
  if (!mojo::Equals(this->reverse_bypass, other.reverse_bypass))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->single_proxies, other.single_proxies))
    return false;
  if (!mojo::Equals(this->proxies_for_http, other.proxies_for_http))
    return false;
  if (!mojo::Equals(this->proxies_for_https, other.proxies_for_https))
    return false;
  if (!mojo::Equals(this->proxies_for_ftp, other.proxies_for_ftp))
    return false;
  if (!mojo::Equals(this->fallback_proxies, other.fallback_proxies))
    return false;
  return true;
}
template <typename StructPtrType>
ProxyConfigPtr ProxyConfig::Clone() const {
  return New(
      mojo::Clone(auto_detect),
      mojo::Clone(pac_url),
      mojo::Clone(pac_mandatory),
      mojo::Clone(proxy_rules),
      mojo::Clone(source),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ProxyConfig>::value>::type*>
bool ProxyConfig::Equals(const T& other) const {
  if (!mojo::Equals(this->auto_detect, other.auto_detect))
    return false;
  if (!mojo::Equals(this->pac_url, other.pac_url))
    return false;
  if (!mojo::Equals(this->pac_mandatory, other.pac_mandatory))
    return false;
  if (!mojo::Equals(this->proxy_rules, other.proxy_rules))
    return false;
  if (!mojo::Equals(this->source, other.source))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace network

namespace mojo {


template <>
struct  StructTraits<::network::mojom::ProxyBypassRules::DataView,
                                         ::network::mojom::ProxyBypassRulesPtr> {
  static bool IsNull(const ::network::mojom::ProxyBypassRulesPtr& input) { return !input; }
  static void SetToNull(::network::mojom::ProxyBypassRulesPtr* output) { output->reset(); }

  static const decltype(::network::mojom::ProxyBypassRules::rules)& rules(
      const ::network::mojom::ProxyBypassRulesPtr& input) {
    return input->rules;
  }

  static bool Read(::network::mojom::ProxyBypassRules::DataView input, ::network::mojom::ProxyBypassRulesPtr* output);
};


template <>
struct  StructTraits<::network::mojom::ProxyList::DataView,
                                         ::network::mojom::ProxyListPtr> {
  static bool IsNull(const ::network::mojom::ProxyListPtr& input) { return !input; }
  static void SetToNull(::network::mojom::ProxyListPtr* output) { output->reset(); }

  static const decltype(::network::mojom::ProxyList::proxies)& proxies(
      const ::network::mojom::ProxyListPtr& input) {
    return input->proxies;
  }

  static bool Read(::network::mojom::ProxyList::DataView input, ::network::mojom::ProxyListPtr* output);
};


template <>
struct  StructTraits<::network::mojom::ProxyRules::DataView,
                                         ::network::mojom::ProxyRulesPtr> {
  static bool IsNull(const ::network::mojom::ProxyRulesPtr& input) { return !input; }
  static void SetToNull(::network::mojom::ProxyRulesPtr* output) { output->reset(); }

  static const decltype(::network::mojom::ProxyRules::bypass_rules)& bypass_rules(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->bypass_rules;
  }

  static decltype(::network::mojom::ProxyRules::reverse_bypass) reverse_bypass(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->reverse_bypass;
  }

  static decltype(::network::mojom::ProxyRules::type) type(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->type;
  }

  static const decltype(::network::mojom::ProxyRules::single_proxies)& single_proxies(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->single_proxies;
  }

  static const decltype(::network::mojom::ProxyRules::proxies_for_http)& proxies_for_http(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->proxies_for_http;
  }

  static const decltype(::network::mojom::ProxyRules::proxies_for_https)& proxies_for_https(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->proxies_for_https;
  }

  static const decltype(::network::mojom::ProxyRules::proxies_for_ftp)& proxies_for_ftp(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->proxies_for_ftp;
  }

  static const decltype(::network::mojom::ProxyRules::fallback_proxies)& fallback_proxies(
      const ::network::mojom::ProxyRulesPtr& input) {
    return input->fallback_proxies;
  }

  static bool Read(::network::mojom::ProxyRules::DataView input, ::network::mojom::ProxyRulesPtr* output);
};


template <>
struct  StructTraits<::network::mojom::ProxyConfig::DataView,
                                         ::network::mojom::ProxyConfigPtr> {
  static bool IsNull(const ::network::mojom::ProxyConfigPtr& input) { return !input; }
  static void SetToNull(::network::mojom::ProxyConfigPtr* output) { output->reset(); }

  static decltype(::network::mojom::ProxyConfig::auto_detect) auto_detect(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->auto_detect;
  }

  static const decltype(::network::mojom::ProxyConfig::pac_url)& pac_url(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->pac_url;
  }

  static decltype(::network::mojom::ProxyConfig::pac_mandatory) pac_mandatory(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->pac_mandatory;
  }

  static const decltype(::network::mojom::ProxyConfig::proxy_rules)& proxy_rules(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->proxy_rules;
  }

  static decltype(::network::mojom::ProxyConfig::source) source(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->source;
  }

  static decltype(::network::mojom::ProxyConfig::id) id(
      const ::network::mojom::ProxyConfigPtr& input) {
    return input->id;
  }

  static bool Read(::network::mojom::ProxyConfig::DataView input, ::network::mojom::ProxyConfigPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_NETWORK_PUBLIC_INTERFACES_PROXY_CONFIG_MOJOM_H_