// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/signals.mojom-shared.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "services/resource_coordinator/public/interfaces/interfaces_shared_export.h"


namespace resource_coordinator {
namespace mojom {
class CoordinationUnitIDDataView;



}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::CoordinationUnitIDDataView> {
  using Data = ::resource_coordinator::mojom::internal::CoordinationUnitID_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace resource_coordinator {
namespace mojom {

enum class CoordinationUnitType : int32_t {
  kFrame,
  kPage,
  kProcess,
};

inline std::ostream& operator<<(std::ostream& os, CoordinationUnitType value) {
  switch(value) {
    case CoordinationUnitType::kFrame:
      return os << "CoordinationUnitType::kFrame";
    case CoordinationUnitType::kPage:
      return os << "CoordinationUnitType::kPage";
    case CoordinationUnitType::kProcess:
      return os << "CoordinationUnitType::kProcess";
    default:
      return os << "Unknown CoordinationUnitType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(CoordinationUnitType value) {
  return internal::CoordinationUnitType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class FrameCoordinationUnitInterfaceBase {};

using FrameCoordinationUnitPtrDataView =
    mojo::InterfacePtrDataView<FrameCoordinationUnitInterfaceBase>;
using FrameCoordinationUnitRequestDataView =
    mojo::InterfaceRequestDataView<FrameCoordinationUnitInterfaceBase>;
using FrameCoordinationUnitAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FrameCoordinationUnitInterfaceBase>;
using FrameCoordinationUnitAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FrameCoordinationUnitInterfaceBase>;
class PageCoordinationUnitInterfaceBase {};

using PageCoordinationUnitPtrDataView =
    mojo::InterfacePtrDataView<PageCoordinationUnitInterfaceBase>;
using PageCoordinationUnitRequestDataView =
    mojo::InterfaceRequestDataView<PageCoordinationUnitInterfaceBase>;
using PageCoordinationUnitAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageCoordinationUnitInterfaceBase>;
using PageCoordinationUnitAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageCoordinationUnitInterfaceBase>;
class ProcessCoordinationUnitInterfaceBase {};

using ProcessCoordinationUnitPtrDataView =
    mojo::InterfacePtrDataView<ProcessCoordinationUnitInterfaceBase>;
using ProcessCoordinationUnitRequestDataView =
    mojo::InterfaceRequestDataView<ProcessCoordinationUnitInterfaceBase>;
using ProcessCoordinationUnitAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ProcessCoordinationUnitInterfaceBase>;
using ProcessCoordinationUnitAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ProcessCoordinationUnitInterfaceBase>;
class CoordinationUnitIDDataView {
 public:
  CoordinationUnitIDDataView() {}

  CoordinationUnitIDDataView(
      internal::CoordinationUnitID_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitType>(
        data_value, output);
  }

  CoordinationUnitType type() const {
    return static_cast<CoordinationUnitType>(data_->type);
  }
  int64_t id() const {
    return data_->id;
  }
 private:
  internal::CoordinationUnitID_Data* data_ = nullptr;
};

class FrameCoordinationUnit_GetID_ParamsDataView {
 public:
  FrameCoordinationUnit_GetID_ParamsDataView() {}

  FrameCoordinationUnit_GetID_ParamsDataView(
      internal::FrameCoordinationUnit_GetID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::FrameCoordinationUnit_GetID_Params_Data* data_ = nullptr;
};

class FrameCoordinationUnit_GetID_ResponseParamsDataView {
 public:
  FrameCoordinationUnit_GetID_ResponseParamsDataView() {}

  FrameCoordinationUnit_GetID_ResponseParamsDataView(
      internal::FrameCoordinationUnit_GetID_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::FrameCoordinationUnit_GetID_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FrameCoordinationUnit_AddBinding_ParamsDataView {
 public:
  FrameCoordinationUnit_AddBinding_ParamsDataView() {}

  FrameCoordinationUnit_AddBinding_ParamsDataView(
      internal::FrameCoordinationUnit_AddBinding_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::resource_coordinator::mojom::FrameCoordinationUnitRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FrameCoordinationUnit_AddBinding_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FrameCoordinationUnit_AddChildFrame_ParamsDataView {
 public:
  FrameCoordinationUnit_AddChildFrame_ParamsDataView() {}

  FrameCoordinationUnit_AddChildFrame_ParamsDataView(
      internal::FrameCoordinationUnit_AddChildFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::FrameCoordinationUnit_AddChildFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FrameCoordinationUnit_RemoveChildFrame_ParamsDataView {
 public:
  FrameCoordinationUnit_RemoveChildFrame_ParamsDataView() {}

  FrameCoordinationUnit_RemoveChildFrame_ParamsDataView(
      internal::FrameCoordinationUnit_RemoveChildFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::FrameCoordinationUnit_RemoveChildFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FrameCoordinationUnit_SetAudibility_ParamsDataView {
 public:
  FrameCoordinationUnit_SetAudibility_ParamsDataView() {}

  FrameCoordinationUnit_SetAudibility_ParamsDataView(
      internal::FrameCoordinationUnit_SetAudibility_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool audible() const {
    return data_->audible;
  }
 private:
  internal::FrameCoordinationUnit_SetAudibility_Params_Data* data_ = nullptr;
};

class FrameCoordinationUnit_SetNetworkAlmostIdle_ParamsDataView {
 public:
  FrameCoordinationUnit_SetNetworkAlmostIdle_ParamsDataView() {}

  FrameCoordinationUnit_SetNetworkAlmostIdle_ParamsDataView(
      internal::FrameCoordinationUnit_SetNetworkAlmostIdle_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool idle() const {
    return data_->idle;
  }
 private:
  internal::FrameCoordinationUnit_SetNetworkAlmostIdle_Params_Data* data_ = nullptr;
};

class FrameCoordinationUnit_OnAlertFired_ParamsDataView {
 public:
  FrameCoordinationUnit_OnAlertFired_ParamsDataView() {}

  FrameCoordinationUnit_OnAlertFired_ParamsDataView(
      internal::FrameCoordinationUnit_OnAlertFired_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::FrameCoordinationUnit_OnAlertFired_Params_Data* data_ = nullptr;
};

class FrameCoordinationUnit_OnNonPersistentNotificationCreated_ParamsDataView {
 public:
  FrameCoordinationUnit_OnNonPersistentNotificationCreated_ParamsDataView() {}

  FrameCoordinationUnit_OnNonPersistentNotificationCreated_ParamsDataView(
      internal::FrameCoordinationUnit_OnNonPersistentNotificationCreated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::FrameCoordinationUnit_OnNonPersistentNotificationCreated_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_GetID_ParamsDataView {
 public:
  PageCoordinationUnit_GetID_ParamsDataView() {}

  PageCoordinationUnit_GetID_ParamsDataView(
      internal::PageCoordinationUnit_GetID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PageCoordinationUnit_GetID_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_GetID_ResponseParamsDataView {
 public:
  PageCoordinationUnit_GetID_ResponseParamsDataView() {}

  PageCoordinationUnit_GetID_ResponseParamsDataView(
      internal::PageCoordinationUnit_GetID_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::PageCoordinationUnit_GetID_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PageCoordinationUnit_AddBinding_ParamsDataView {
 public:
  PageCoordinationUnit_AddBinding_ParamsDataView() {}

  PageCoordinationUnit_AddBinding_ParamsDataView(
      internal::PageCoordinationUnit_AddBinding_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::resource_coordinator::mojom::PageCoordinationUnitRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::PageCoordinationUnit_AddBinding_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PageCoordinationUnit_AddFrame_ParamsDataView {
 public:
  PageCoordinationUnit_AddFrame_ParamsDataView() {}

  PageCoordinationUnit_AddFrame_ParamsDataView(
      internal::PageCoordinationUnit_AddFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::PageCoordinationUnit_AddFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PageCoordinationUnit_RemoveFrame_ParamsDataView {
 public:
  PageCoordinationUnit_RemoveFrame_ParamsDataView() {}

  PageCoordinationUnit_RemoveFrame_ParamsDataView(
      internal::PageCoordinationUnit_RemoveFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::PageCoordinationUnit_RemoveFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PageCoordinationUnit_SetVisibility_ParamsDataView {
 public:
  PageCoordinationUnit_SetVisibility_ParamsDataView() {}

  PageCoordinationUnit_SetVisibility_ParamsDataView(
      internal::PageCoordinationUnit_SetVisibility_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool visible() const {
    return data_->visible;
  }
 private:
  internal::PageCoordinationUnit_SetVisibility_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_SetUKMSourceId_ParamsDataView {
 public:
  PageCoordinationUnit_SetUKMSourceId_ParamsDataView() {}

  PageCoordinationUnit_SetUKMSourceId_ParamsDataView(
      internal::PageCoordinationUnit_SetUKMSourceId_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int64_t ukm_source_id() const {
    return data_->ukm_source_id;
  }
 private:
  internal::PageCoordinationUnit_SetUKMSourceId_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_OnFaviconUpdated_ParamsDataView {
 public:
  PageCoordinationUnit_OnFaviconUpdated_ParamsDataView() {}

  PageCoordinationUnit_OnFaviconUpdated_ParamsDataView(
      internal::PageCoordinationUnit_OnFaviconUpdated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PageCoordinationUnit_OnFaviconUpdated_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_OnTitleUpdated_ParamsDataView {
 public:
  PageCoordinationUnit_OnTitleUpdated_ParamsDataView() {}

  PageCoordinationUnit_OnTitleUpdated_ParamsDataView(
      internal::PageCoordinationUnit_OnTitleUpdated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PageCoordinationUnit_OnTitleUpdated_Params_Data* data_ = nullptr;
};

class PageCoordinationUnit_OnMainFrameNavigationCommitted_ParamsDataView {
 public:
  PageCoordinationUnit_OnMainFrameNavigationCommitted_ParamsDataView() {}

  PageCoordinationUnit_OnMainFrameNavigationCommitted_ParamsDataView(
      internal::PageCoordinationUnit_OnMainFrameNavigationCommitted_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PageCoordinationUnit_OnMainFrameNavigationCommitted_Params_Data* data_ = nullptr;
};

class ProcessCoordinationUnit_GetID_ParamsDataView {
 public:
  ProcessCoordinationUnit_GetID_ParamsDataView() {}

  ProcessCoordinationUnit_GetID_ParamsDataView(
      internal::ProcessCoordinationUnit_GetID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ProcessCoordinationUnit_GetID_Params_Data* data_ = nullptr;
};

class ProcessCoordinationUnit_GetID_ResponseParamsDataView {
 public:
  ProcessCoordinationUnit_GetID_ResponseParamsDataView() {}

  ProcessCoordinationUnit_GetID_ResponseParamsDataView(
      internal::ProcessCoordinationUnit_GetID_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessCoordinationUnit_GetID_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_AddBinding_ParamsDataView {
 public:
  ProcessCoordinationUnit_AddBinding_ParamsDataView() {}

  ProcessCoordinationUnit_AddBinding_ParamsDataView(
      internal::ProcessCoordinationUnit_AddBinding_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::resource_coordinator::mojom::ProcessCoordinationUnitRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ProcessCoordinationUnit_AddBinding_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_AddFrame_ParamsDataView {
 public:
  ProcessCoordinationUnit_AddFrame_ParamsDataView() {}

  ProcessCoordinationUnit_AddFrame_ParamsDataView(
      internal::ProcessCoordinationUnit_AddFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessCoordinationUnit_AddFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_RemoveFrame_ParamsDataView {
 public:
  ProcessCoordinationUnit_RemoveFrame_ParamsDataView() {}

  ProcessCoordinationUnit_RemoveFrame_ParamsDataView(
      internal::ProcessCoordinationUnit_RemoveFrame_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCuIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCuId(UserType* output) {
    auto* pointer = data_->cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessCoordinationUnit_RemoveFrame_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_SetCPUUsage_ParamsDataView {
 public:
  ProcessCoordinationUnit_SetCPUUsage_ParamsDataView() {}

  ProcessCoordinationUnit_SetCPUUsage_ParamsDataView(
      internal::ProcessCoordinationUnit_SetCPUUsage_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  double cpu_usage() const {
    return data_->cpu_usage;
  }
 private:
  internal::ProcessCoordinationUnit_SetCPUUsage_Params_Data* data_ = nullptr;
};

class ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_ParamsDataView {
 public:
  ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_ParamsDataView() {}

  ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_ParamsDataView(
      internal::ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDurationDataView(
      ::mojo::common::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDuration(UserType* output) {
    auto* pointer = data_->duration.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_SetLaunchTime_ParamsDataView {
 public:
  ProcessCoordinationUnit_SetLaunchTime_ParamsDataView() {}

  ProcessCoordinationUnit_SetLaunchTime_ParamsDataView(
      internal::ProcessCoordinationUnit_SetLaunchTime_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLaunchTimeDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLaunchTime(UserType* output) {
    auto* pointer = data_->launch_time.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessCoordinationUnit_SetLaunchTime_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ProcessCoordinationUnit_SetMainThreadTaskLoadIsLow_ParamsDataView {
 public:
  ProcessCoordinationUnit_SetMainThreadTaskLoadIsLow_ParamsDataView() {}

  ProcessCoordinationUnit_SetMainThreadTaskLoadIsLow_ParamsDataView(
      internal::ProcessCoordinationUnit_SetMainThreadTaskLoadIsLow_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool main_thread_task_load_is_low() const {
    return data_->main_thread_task_load_is_low;
  }
 private:
  internal::ProcessCoordinationUnit_SetMainThreadTaskLoadIsLow_Params_Data* data_ = nullptr;
};

class ProcessCoordinationUnit_SetPID_ParamsDataView {
 public:
  ProcessCoordinationUnit_SetPID_ParamsDataView() {}

  ProcessCoordinationUnit_SetPID_ParamsDataView(
      internal::ProcessCoordinationUnit_SetPID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int64_t pid() const {
    return data_->pid;
  }
 private:
  internal::ProcessCoordinationUnit_SetPID_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace resource_coordinator

namespace std {

template <>
struct hash<::resource_coordinator::mojom::CoordinationUnitType>
    : public mojo::internal::EnumHashImpl<::resource_coordinator::mojom::CoordinationUnitType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::resource_coordinator::mojom::CoordinationUnitType, ::resource_coordinator::mojom::CoordinationUnitType> {
  static ::resource_coordinator::mojom::CoordinationUnitType ToMojom(::resource_coordinator::mojom::CoordinationUnitType input) { return input; }
  static bool FromMojom(::resource_coordinator::mojom::CoordinationUnitType input, ::resource_coordinator::mojom::CoordinationUnitType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::CoordinationUnitType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::resource_coordinator::mojom::CoordinationUnitType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::resource_coordinator::mojom::CoordinationUnitType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::CoordinationUnitIDDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::CoordinationUnitIDDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::CoordinationUnitID_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::resource_coordinator::mojom::CoordinationUnitType>(
        CallWithContext(Traits::type, input, custom_context), &(*output)->type);
    (*output)->id = CallWithContext(Traits::id, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::CoordinationUnitID_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::CoordinationUnitIDDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace resource_coordinator {
namespace mojom {





inline void FrameCoordinationUnit_GetID_ResponseParamsDataView::GetIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void FrameCoordinationUnit_AddChildFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}


inline void FrameCoordinationUnit_RemoveChildFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}












inline void PageCoordinationUnit_GetID_ResponseParamsDataView::GetIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void PageCoordinationUnit_AddFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}


inline void PageCoordinationUnit_RemoveFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}














inline void ProcessCoordinationUnit_GetID_ResponseParamsDataView::GetIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void ProcessCoordinationUnit_AddFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}


inline void ProcessCoordinationUnit_RemoveFrame_ParamsDataView::GetCuIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->cu_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void ProcessCoordinationUnit_SetExpectedTaskQueueingDuration_ParamsDataView::GetDurationDataView(
    ::mojo::common::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->duration.Get();
  *output = ::mojo::common::mojom::TimeDeltaDataView(pointer, context_);
}


inline void ProcessCoordinationUnit_SetLaunchTime_ParamsDataView::GetLaunchTimeDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->launch_time.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}







}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_
