// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include "services/resource_coordinator/public/interfaces/signals.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_id.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_types.h"
#include "services/resource_coordinator/public/interfaces/interfaces_export.h"


namespace resource_coordinator {
namespace mojom {
class FrameCoordinationUnit;
using FrameCoordinationUnitPtr = mojo::InterfacePtr<FrameCoordinationUnit>;
using FrameCoordinationUnitPtrInfo = mojo::InterfacePtrInfo<FrameCoordinationUnit>;
using ThreadSafeFrameCoordinationUnitPtr =
    mojo::ThreadSafeInterfacePtr<FrameCoordinationUnit>;
using FrameCoordinationUnitRequest = mojo::InterfaceRequest<FrameCoordinationUnit>;
using FrameCoordinationUnitAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameCoordinationUnit>;
using ThreadSafeFrameCoordinationUnitAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameCoordinationUnit>;
using FrameCoordinationUnitAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameCoordinationUnit>;
using FrameCoordinationUnitAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameCoordinationUnit>;

class PageCoordinationUnit;
using PageCoordinationUnitPtr = mojo::InterfacePtr<PageCoordinationUnit>;
using PageCoordinationUnitPtrInfo = mojo::InterfacePtrInfo<PageCoordinationUnit>;
using ThreadSafePageCoordinationUnitPtr =
    mojo::ThreadSafeInterfacePtr<PageCoordinationUnit>;
using PageCoordinationUnitRequest = mojo::InterfaceRequest<PageCoordinationUnit>;
using PageCoordinationUnitAssociatedPtr =
    mojo::AssociatedInterfacePtr<PageCoordinationUnit>;
using ThreadSafePageCoordinationUnitAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PageCoordinationUnit>;
using PageCoordinationUnitAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PageCoordinationUnit>;
using PageCoordinationUnitAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PageCoordinationUnit>;

class ProcessCoordinationUnit;
using ProcessCoordinationUnitPtr = mojo::InterfacePtr<ProcessCoordinationUnit>;
using ProcessCoordinationUnitPtrInfo = mojo::InterfacePtrInfo<ProcessCoordinationUnit>;
using ThreadSafeProcessCoordinationUnitPtr =
    mojo::ThreadSafeInterfacePtr<ProcessCoordinationUnit>;
using ProcessCoordinationUnitRequest = mojo::InterfaceRequest<ProcessCoordinationUnit>;
using ProcessCoordinationUnitAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProcessCoordinationUnit>;
using ThreadSafeProcessCoordinationUnitAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProcessCoordinationUnit>;
using ProcessCoordinationUnitAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProcessCoordinationUnit>;
using ProcessCoordinationUnitAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProcessCoordinationUnit>;

class CoordinationUnitID;
using CoordinationUnitIDPtr = mojo::InlinedStructPtr<CoordinationUnitID>;


class FrameCoordinationUnitProxy;

template <typename ImplRefTraits>
class FrameCoordinationUnitStub;

class FrameCoordinationUnitRequestValidator;
class FrameCoordinationUnitResponseValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnit
    : public FrameCoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameCoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameCoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = FrameCoordinationUnitRequestValidator;
  using ResponseValidator_ = FrameCoordinationUnitResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetIDMinVersion = 0,
    kAddBindingMinVersion = 0,
    kAddChildFrameMinVersion = 0,
    kRemoveChildFrameMinVersion = 0,
    kSetAudibilityMinVersion = 0,
    kSetNetworkAlmostIdleMinVersion = 0,
    kOnAlertFiredMinVersion = 0,
    kOnNonPersistentNotificationCreatedMinVersion = 0,
  };
  virtual ~FrameCoordinationUnit() {}


  using GetIDCallback = base::Callback<void(const ::resource_coordinator::CoordinationUnitID&)>;
  virtual void GetID(const GetIDCallback& callback) = 0;

  virtual void AddBinding(FrameCoordinationUnitRequest request) = 0;

  virtual void AddChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void RemoveChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void SetAudibility(bool audible) = 0;

  virtual void SetNetworkAlmostIdle(bool idle) = 0;

  virtual void OnAlertFired() = 0;

  virtual void OnNonPersistentNotificationCreated() = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitInterceptorForTesting : public FrameCoordinationUnit {
  virtual FrameCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(const GetIDCallback& callback) override;
  void AddBinding(FrameCoordinationUnitRequest request) override;
  void AddChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void RemoveChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void SetAudibility(bool audible) override;
  void SetNetworkAlmostIdle(bool idle) override;
  void OnAlertFired() override;
  void OnNonPersistentNotificationCreated() override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitAsyncWaiter {
 public:
  explicit FrameCoordinationUnitAsyncWaiter(FrameCoordinationUnit* proxy);
  ~FrameCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  FrameCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameCoordinationUnitAsyncWaiter);
};

class PageCoordinationUnitProxy;

template <typename ImplRefTraits>
class PageCoordinationUnitStub;

class PageCoordinationUnitRequestValidator;
class PageCoordinationUnitResponseValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnit
    : public PageCoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageCoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageCoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = PageCoordinationUnitRequestValidator;
  using ResponseValidator_ = PageCoordinationUnitResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetIDMinVersion = 0,
    kAddBindingMinVersion = 0,
    kAddFrameMinVersion = 0,
    kRemoveFrameMinVersion = 0,
    kSetVisibilityMinVersion = 0,
    kSetUKMSourceIdMinVersion = 0,
    kOnFaviconUpdatedMinVersion = 0,
    kOnTitleUpdatedMinVersion = 0,
    kOnMainFrameNavigationCommittedMinVersion = 0,
  };
  virtual ~PageCoordinationUnit() {}


  using GetIDCallback = base::Callback<void(const ::resource_coordinator::CoordinationUnitID&)>;
  virtual void GetID(const GetIDCallback& callback) = 0;

  virtual void AddBinding(PageCoordinationUnitRequest request) = 0;

  virtual void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void SetVisibility(bool visible) = 0;

  virtual void SetUKMSourceId(int64_t ukm_source_id) = 0;

  virtual void OnFaviconUpdated() = 0;

  virtual void OnTitleUpdated() = 0;

  virtual void OnMainFrameNavigationCommitted() = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitInterceptorForTesting : public PageCoordinationUnit {
  virtual PageCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(const GetIDCallback& callback) override;
  void AddBinding(PageCoordinationUnitRequest request) override;
  void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void SetVisibility(bool visible) override;
  void SetUKMSourceId(int64_t ukm_source_id) override;
  void OnFaviconUpdated() override;
  void OnTitleUpdated() override;
  void OnMainFrameNavigationCommitted() override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitAsyncWaiter {
 public:
  explicit PageCoordinationUnitAsyncWaiter(PageCoordinationUnit* proxy);
  ~PageCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  PageCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageCoordinationUnitAsyncWaiter);
};

class ProcessCoordinationUnitProxy;

template <typename ImplRefTraits>
class ProcessCoordinationUnitStub;

class ProcessCoordinationUnitRequestValidator;
class ProcessCoordinationUnitResponseValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnit
    : public ProcessCoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProcessCoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProcessCoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = ProcessCoordinationUnitRequestValidator;
  using ResponseValidator_ = ProcessCoordinationUnitResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetIDMinVersion = 0,
    kAddBindingMinVersion = 0,
    kAddFrameMinVersion = 0,
    kRemoveFrameMinVersion = 0,
    kSetCPUUsageMinVersion = 0,
    kSetExpectedTaskQueueingDurationMinVersion = 0,
    kSetLaunchTimeMinVersion = 0,
    kSetMainThreadTaskLoadIsLowMinVersion = 0,
    kSetPIDMinVersion = 0,
  };
  virtual ~ProcessCoordinationUnit() {}


  using GetIDCallback = base::Callback<void(const ::resource_coordinator::CoordinationUnitID&)>;
  virtual void GetID(const GetIDCallback& callback) = 0;

  virtual void AddBinding(ProcessCoordinationUnitRequest request) = 0;

  virtual void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) = 0;

  virtual void SetCPUUsage(double cpu_usage) = 0;

  virtual void SetExpectedTaskQueueingDuration(base::TimeDelta duration) = 0;

  virtual void SetLaunchTime(base::Time launch_time) = 0;

  virtual void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) = 0;

  virtual void SetPID(int64_t pid) = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitInterceptorForTesting : public ProcessCoordinationUnit {
  virtual ProcessCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(const GetIDCallback& callback) override;
  void AddBinding(ProcessCoordinationUnitRequest request) override;
  void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void SetCPUUsage(double cpu_usage) override;
  void SetExpectedTaskQueueingDuration(base::TimeDelta duration) override;
  void SetLaunchTime(base::Time launch_time) override;
  void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) override;
  void SetPID(int64_t pid) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitAsyncWaiter {
 public:
  explicit ProcessCoordinationUnitAsyncWaiter(ProcessCoordinationUnit* proxy);
  ~ProcessCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  ProcessCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProcessCoordinationUnitAsyncWaiter);
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitProxy
    : public FrameCoordinationUnit {
 public:
  explicit FrameCoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetID(const GetIDCallback& callback) final;
  void AddBinding(FrameCoordinationUnitRequest request) final;
  void AddChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void RemoveChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void SetAudibility(bool audible) final;
  void SetNetworkAlmostIdle(bool idle) final;
  void OnAlertFired() final;
  void OnNonPersistentNotificationCreated() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitProxy
    : public PageCoordinationUnit {
 public:
  explicit PageCoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetID(const GetIDCallback& callback) final;
  void AddBinding(PageCoordinationUnitRequest request) final;
  void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void SetVisibility(bool visible) final;
  void SetUKMSourceId(int64_t ukm_source_id) final;
  void OnFaviconUpdated() final;
  void OnTitleUpdated() final;
  void OnMainFrameNavigationCommitted() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitProxy
    : public ProcessCoordinationUnit {
 public:
  explicit ProcessCoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetID(const GetIDCallback& callback) final;
  void AddBinding(ProcessCoordinationUnitRequest request) final;
  void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) final;
  void SetCPUUsage(double cpu_usage) final;
  void SetExpectedTaskQueueingDuration(base::TimeDelta duration) final;
  void SetLaunchTime(base::Time launch_time) final;
  void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) final;
  void SetPID(int64_t pid) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitStubDispatch {
 public:
  static bool Accept(FrameCoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameCoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameCoordinationUnit>>
class FrameCoordinationUnitStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameCoordinationUnitStub() {}
  ~FrameCoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameCoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameCoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitStubDispatch {
 public:
  static bool Accept(PageCoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageCoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageCoordinationUnit>>
class PageCoordinationUnitStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageCoordinationUnitStub() {}
  ~PageCoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageCoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageCoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitStubDispatch {
 public:
  static bool Accept(ProcessCoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProcessCoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProcessCoordinationUnit>>
class ProcessCoordinationUnitStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProcessCoordinationUnitStub() {}
  ~ProcessCoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessCoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessCoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT FrameCoordinationUnitResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT PageCoordinationUnitResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT ProcessCoordinationUnitResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};


class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinationUnitID {
 public:
  using DataView = CoordinationUnitIDDataView;
  using Data_ = internal::CoordinationUnitID_Data;

  template <typename... Args>
  static CoordinationUnitIDPtr New(Args&&... args) {
    return CoordinationUnitIDPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CoordinationUnitIDPtr From(const U& u) {
    return mojo::TypeConverter<CoordinationUnitIDPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CoordinationUnitID>::Convert(*this);
  }


  CoordinationUnitID();

  CoordinationUnitID(
      ::resource_coordinator::CoordinationUnitType type,
      int64_t id);

  ~CoordinationUnitID();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CoordinationUnitIDPtr>
  CoordinationUnitIDPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CoordinationUnitID>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CoordinationUnitID::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CoordinationUnitID::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CoordinationUnitID_UnserializedMessageContext<
            UserType, CoordinationUnitID::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CoordinationUnitID::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CoordinationUnitID::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CoordinationUnitID_UnserializedMessageContext<
            UserType, CoordinationUnitID::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CoordinationUnitID::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::resource_coordinator::CoordinationUnitType type;
  int64_t id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
CoordinationUnitIDPtr CoordinationUnitID::Clone() const {
  return New(
      mojo::Clone(type),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CoordinationUnitID>::value>::type*>
bool CoordinationUnitID::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT StructTraits<::resource_coordinator::mojom::CoordinationUnitID::DataView,
                                         ::resource_coordinator::mojom::CoordinationUnitIDPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::CoordinationUnitIDPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::type) type(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->type;
  }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::id) id(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->id;
  }

  static bool Read(::resource_coordinator::mojom::CoordinationUnitID::DataView input, ::resource_coordinator::mojom::CoordinationUnitIDPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_