// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/tracing/tracing.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include "mojo/common/values.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "services/resource_coordinator/public/interfaces/interfaces_export.h"


namespace tracing {
namespace mojom {
class AgentRegistry;
using AgentRegistryPtr = mojo::InterfacePtr<AgentRegistry>;
using AgentRegistryPtrInfo = mojo::InterfacePtrInfo<AgentRegistry>;
using ThreadSafeAgentRegistryPtr =
    mojo::ThreadSafeInterfacePtr<AgentRegistry>;
using AgentRegistryRequest = mojo::InterfaceRequest<AgentRegistry>;
using AgentRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<AgentRegistry>;
using ThreadSafeAgentRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AgentRegistry>;
using AgentRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AgentRegistry>;
using AgentRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AgentRegistry>;

class Agent;
using AgentPtr = mojo::InterfacePtr<Agent>;
using AgentPtrInfo = mojo::InterfacePtrInfo<Agent>;
using ThreadSafeAgentPtr =
    mojo::ThreadSafeInterfacePtr<Agent>;
using AgentRequest = mojo::InterfaceRequest<Agent>;
using AgentAssociatedPtr =
    mojo::AssociatedInterfacePtr<Agent>;
using ThreadSafeAgentAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Agent>;
using AgentAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Agent>;
using AgentAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Agent>;

class Recorder;
using RecorderPtr = mojo::InterfacePtr<Recorder>;
using RecorderPtrInfo = mojo::InterfacePtrInfo<Recorder>;
using ThreadSafeRecorderPtr =
    mojo::ThreadSafeInterfacePtr<Recorder>;
using RecorderRequest = mojo::InterfaceRequest<Recorder>;
using RecorderAssociatedPtr =
    mojo::AssociatedInterfacePtr<Recorder>;
using ThreadSafeRecorderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Recorder>;
using RecorderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Recorder>;
using RecorderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Recorder>;

class Coordinator;
using CoordinatorPtr = mojo::InterfacePtr<Coordinator>;
using CoordinatorPtrInfo = mojo::InterfacePtrInfo<Coordinator>;
using ThreadSafeCoordinatorPtr =
    mojo::ThreadSafeInterfacePtr<Coordinator>;
using CoordinatorRequest = mojo::InterfaceRequest<Coordinator>;
using CoordinatorAssociatedPtr =
    mojo::AssociatedInterfacePtr<Coordinator>;
using ThreadSafeCoordinatorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Coordinator>;
using CoordinatorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Coordinator>;
using CoordinatorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Coordinator>;


class AgentRegistryProxy;

template <typename ImplRefTraits>
class AgentRegistryStub;

class AgentRegistryRequestValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistry
    : public AgentRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentRegistryStub<ImplRefTraits>;

  using RequestValidator_ = AgentRegistryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kRegisterAgentMinVersion = 0,
  };
  virtual ~AgentRegistry() {}

  virtual void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, bool supports_explicit_clock_sync_) = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistryInterceptorForTesting : public AgentRegistry {
  virtual AgentRegistry* GetForwardingInterface() = 0;
  void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, bool supports_explicit_clock_sync_) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistryAsyncWaiter {
 public:
  explicit AgentRegistryAsyncWaiter(AgentRegistry* proxy);
  ~AgentRegistryAsyncWaiter();

 private:
  AgentRegistry* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AgentRegistryAsyncWaiter);
};

class AgentProxy;

template <typename ImplRefTraits>
class AgentStub;

class AgentRequestValidator;
class AgentResponseValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT Agent
    : public AgentInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentStub<ImplRefTraits>;

  using RequestValidator_ = AgentRequestValidator;
  using ResponseValidator_ = AgentResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kRequestClockSyncMarkerMinVersion = 0,
    kRequestBufferStatusMinVersion = 0,
    kGetCategoriesMinVersion = 0,
  };
  virtual ~Agent() {}


  using StartTracingCallback = base::Callback<void(bool)>;
  virtual void StartTracing(const std::string& config, base::TimeTicks coordinator_time, const StartTracingCallback& callback) = 0;

  virtual void StopAndFlush(RecorderPtr recorder) = 0;


  using RequestClockSyncMarkerCallback = base::Callback<void(base::TimeTicks, base::TimeTicks)>;
  virtual void RequestClockSyncMarker(const std::string& sync_id, const RequestClockSyncMarkerCallback& callback) = 0;


  using RequestBufferStatusCallback = base::Callback<void(uint32_t, uint32_t)>;
  virtual void RequestBufferStatus(const RequestBufferStatusCallback& callback) = 0;


  using GetCategoriesCallback = base::Callback<void(const std::string&)>;
  virtual void GetCategories(const GetCategoriesCallback& callback) = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentInterceptorForTesting : public Agent {
  virtual Agent* GetForwardingInterface() = 0;
  void StartTracing(const std::string& config, base::TimeTicks coordinator_time, const StartTracingCallback& callback) override;
  void StopAndFlush(RecorderPtr recorder) override;
  void RequestClockSyncMarker(const std::string& sync_id, const RequestClockSyncMarkerCallback& callback) override;
  void RequestBufferStatus(const RequestBufferStatusCallback& callback) override;
  void GetCategories(const GetCategoriesCallback& callback) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentAsyncWaiter {
 public:
  explicit AgentAsyncWaiter(Agent* proxy);
  ~AgentAsyncWaiter();
  void StartTracing(
      const std::string& config, base::TimeTicks coordinator_time, bool* out_success);
  void RequestClockSyncMarker(
      const std::string& sync_id, base::TimeTicks* out_issue_ts, base::TimeTicks* out_issue_end_ts);
  void RequestBufferStatus(
      uint32_t* out_capacity, uint32_t* out_count);
  void GetCategories(
      std::string* out_categories);

 private:
  Agent* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AgentAsyncWaiter);
};

class RecorderProxy;

template <typename ImplRefTraits>
class RecorderStub;

class RecorderRequestValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT Recorder
    : public RecorderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RecorderProxy;

  template <typename ImplRefTraits>
  using Stub_ = RecorderStub<ImplRefTraits>;

  using RequestValidator_ = RecorderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddChunkMinVersion = 0,
    kAddMetadataMinVersion = 0,
  };
  virtual ~Recorder() {}

  virtual void AddChunk(const std::string& chunk) = 0;

  virtual void AddMetadata(std::unique_ptr<base::DictionaryValue> metadata) = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT RecorderInterceptorForTesting : public Recorder {
  virtual Recorder* GetForwardingInterface() = 0;
  void AddChunk(const std::string& chunk) override;
  void AddMetadata(std::unique_ptr<base::DictionaryValue> metadata) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT RecorderAsyncWaiter {
 public:
  explicit RecorderAsyncWaiter(Recorder* proxy);
  ~RecorderAsyncWaiter();

 private:
  Recorder* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(RecorderAsyncWaiter);
};

class CoordinatorProxy;

template <typename ImplRefTraits>
class CoordinatorStub;

class CoordinatorRequestValidator;
class CoordinatorResponseValidator;

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT Coordinator
    : public CoordinatorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinatorProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinatorStub<ImplRefTraits>;

  using RequestValidator_ = CoordinatorRequestValidator;
  using ResponseValidator_ = CoordinatorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kStopAndFlushAgentMinVersion = 0,
    kIsTracingMinVersion = 0,
    kRequestBufferUsageMinVersion = 0,
    kGetCategoriesMinVersion = 0,
  };
  virtual ~Coordinator() {}


  using StartTracingCallback = base::Callback<void(bool)>;
  virtual void StartTracing(const std::string& config, const StartTracingCallback& callback) = 0;


  using StopAndFlushCallback = base::Callback<void(std::unique_ptr<base::DictionaryValue>)>;
  virtual void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream, const StopAndFlushCallback& callback) = 0;


  using StopAndFlushAgentCallback = base::Callback<void(std::unique_ptr<base::DictionaryValue>)>;
  virtual void StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, const StopAndFlushAgentCallback& callback) = 0;


  using IsTracingCallback = base::Callback<void(bool)>;
  virtual void IsTracing(const IsTracingCallback& callback) = 0;


  using RequestBufferUsageCallback = base::Callback<void(bool, float, uint32_t)>;
  virtual void RequestBufferUsage(const RequestBufferUsageCallback& callback) = 0;


  using GetCategoriesCallback = base::Callback<void(bool, const std::string&)>;
  virtual void GetCategories(const GetCategoriesCallback& callback) = 0;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorInterceptorForTesting : public Coordinator {
  virtual Coordinator* GetForwardingInterface() = 0;
  void StartTracing(const std::string& config, const StartTracingCallback& callback) override;
  void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream, const StopAndFlushCallback& callback) override;
  void StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, const StopAndFlushAgentCallback& callback) override;
  void IsTracing(const IsTracingCallback& callback) override;
  void RequestBufferUsage(const RequestBufferUsageCallback& callback) override;
  void GetCategories(const GetCategoriesCallback& callback) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorAsyncWaiter {
 public:
  explicit CoordinatorAsyncWaiter(Coordinator* proxy);
  ~CoordinatorAsyncWaiter();
  void StartTracing(
      const std::string& config, bool* out_success);
  void StopAndFlush(
      mojo::ScopedDataPipeProducerHandle stream, std::unique_ptr<base::DictionaryValue>* out_metadata);
  void StopAndFlushAgent(
      mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, std::unique_ptr<base::DictionaryValue>* out_metadata);
  void IsTracing(
      bool* out_is_tracing);
  void RequestBufferUsage(
      bool* out_success, float* out_percent_full, uint32_t* out_approximate_count);
  void GetCategories(
      bool* out_success, std::string* out_categories);

 private:
  Coordinator* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(CoordinatorAsyncWaiter);
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistryProxy
    : public AgentRegistry {
 public:
  explicit AgentRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, bool supports_explicit_clock_sync_) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentProxy
    : public Agent {
 public:
  explicit AgentProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config, base::TimeTicks coordinator_time, const StartTracingCallback& callback) final;
  void StopAndFlush(RecorderPtr recorder) final;
  void RequestClockSyncMarker(const std::string& sync_id, const RequestClockSyncMarkerCallback& callback) final;
  void RequestBufferStatus(const RequestBufferStatusCallback& callback) final;
  void GetCategories(const GetCategoriesCallback& callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT RecorderProxy
    : public Recorder {
 public:
  explicit RecorderProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddChunk(const std::string& chunk) final;
  void AddMetadata(std::unique_ptr<base::DictionaryValue> metadata) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorProxy
    : public Coordinator {
 public:
  explicit CoordinatorProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config, const StartTracingCallback& callback) final;
  void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream, const StopAndFlushCallback& callback) final;
  void StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, const StopAndFlushAgentCallback& callback) final;
  void IsTracing(const IsTracingCallback& callback) final;
  void RequestBufferUsage(const RequestBufferUsageCallback& callback) final;
  void GetCategories(const GetCategoriesCallback& callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistryStubDispatch {
 public:
  static bool Accept(AgentRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AgentRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AgentRegistry>>
class AgentRegistryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentRegistryStub() {}
  ~AgentRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentStubDispatch {
 public:
  static bool Accept(Agent* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Agent* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Agent>>
class AgentStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentStub() {}
  ~AgentStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT RecorderStubDispatch {
 public:
  static bool Accept(Recorder* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Recorder* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Recorder>>
class RecorderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RecorderStub() {}
  ~RecorderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorStubDispatch {
 public:
  static bool Accept(Coordinator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Coordinator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Coordinator>>
class CoordinatorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinatorStub() {}
  ~CoordinatorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRegistryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT RecorderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT AgentResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class RESOURCE_COORDINATOR_PUBLIC_INTERFACES_EXPORT CoordinatorResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace tracing

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TRACING_TRACING_MOJOM_H_