// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_DISPLAY_MANAGER_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_DISPLAY_MANAGER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/display_manager.mojom-shared.h"
#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {
class DisplayManagerObserver;
using DisplayManagerObserverPtr = mojo::InterfacePtr<DisplayManagerObserver>;
using DisplayManagerObserverPtrInfo = mojo::InterfacePtrInfo<DisplayManagerObserver>;
using ThreadSafeDisplayManagerObserverPtr =
    mojo::ThreadSafeInterfacePtr<DisplayManagerObserver>;
using DisplayManagerObserverRequest = mojo::InterfaceRequest<DisplayManagerObserver>;
using DisplayManagerObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<DisplayManagerObserver>;
using ThreadSafeDisplayManagerObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DisplayManagerObserver>;
using DisplayManagerObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DisplayManagerObserver>;
using DisplayManagerObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DisplayManagerObserver>;

class DisplayManager;
using DisplayManagerPtr = mojo::InterfacePtr<DisplayManager>;
using DisplayManagerPtrInfo = mojo::InterfacePtrInfo<DisplayManager>;
using ThreadSafeDisplayManagerPtr =
    mojo::ThreadSafeInterfacePtr<DisplayManager>;
using DisplayManagerRequest = mojo::InterfaceRequest<DisplayManager>;
using DisplayManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<DisplayManager>;
using ThreadSafeDisplayManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DisplayManager>;
using DisplayManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DisplayManager>;
using DisplayManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DisplayManager>;


class DisplayManagerObserverProxy;

template <typename ImplRefTraits>
class DisplayManagerObserverStub;

class DisplayManagerObserverRequestValidator;

class  DisplayManagerObserver
    : public DisplayManagerObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DisplayManagerObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = DisplayManagerObserverStub<ImplRefTraits>;

  using RequestValidator_ = DisplayManagerObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnDisplaysChangedMinVersion = 0,
  };
  virtual ~DisplayManagerObserver() {}

  virtual void OnDisplaysChanged(std::vector<::ui::mojom::WsDisplayPtr> displays, int64_t primary_display_id, int64_t internal_display_id) = 0;
};
class  DisplayManagerObserverInterceptorForTesting : public DisplayManagerObserver {
  virtual DisplayManagerObserver* GetForwardingInterface() = 0;
  void OnDisplaysChanged(std::vector<::ui::mojom::WsDisplayPtr> displays, int64_t primary_display_id, int64_t internal_display_id) override;
};
class  DisplayManagerObserverAsyncWaiter {
 public:
  explicit DisplayManagerObserverAsyncWaiter(DisplayManagerObserver* proxy);
  ~DisplayManagerObserverAsyncWaiter();

 private:
  DisplayManagerObserver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DisplayManagerObserverAsyncWaiter);
};

class DisplayManagerProxy;

template <typename ImplRefTraits>
class DisplayManagerStub;

class DisplayManagerRequestValidator;

class  DisplayManager
    : public DisplayManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DisplayManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = DisplayManagerStub<ImplRefTraits>;

  using RequestValidator_ = DisplayManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddObserverMinVersion = 0,
  };
  virtual ~DisplayManager() {}

  virtual void AddObserver(DisplayManagerObserverPtr observer) = 0;
};
class  DisplayManagerInterceptorForTesting : public DisplayManager {
  virtual DisplayManager* GetForwardingInterface() = 0;
  void AddObserver(DisplayManagerObserverPtr observer) override;
};
class  DisplayManagerAsyncWaiter {
 public:
  explicit DisplayManagerAsyncWaiter(DisplayManager* proxy);
  ~DisplayManagerAsyncWaiter();

 private:
  DisplayManager* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DisplayManagerAsyncWaiter);
};

class  DisplayManagerObserverProxy
    : public DisplayManagerObserver {
 public:
  explicit DisplayManagerObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnDisplaysChanged(std::vector<::ui::mojom::WsDisplayPtr> displays, int64_t primary_display_id, int64_t internal_display_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  DisplayManagerProxy
    : public DisplayManager {
 public:
  explicit DisplayManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddObserver(DisplayManagerObserverPtr observer) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  DisplayManagerObserverStubDispatch {
 public:
  static bool Accept(DisplayManagerObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DisplayManagerObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DisplayManagerObserver>>
class DisplayManagerObserverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DisplayManagerObserverStub() {}
  ~DisplayManagerObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayManagerObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayManagerObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  DisplayManagerStubDispatch {
 public:
  static bool Accept(DisplayManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DisplayManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DisplayManager>>
class DisplayManagerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DisplayManagerStub() {}
  ~DisplayManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DisplayManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  DisplayManagerObserverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  DisplayManagerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_DISPLAY_MANAGER_MOJOM_H_