// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_REMOTE_EVENT_DISPATCHER_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_REMOTE_EVENT_DISPATCHER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/remote_event_dispatcher.mojom-shared.h"
#include "ui/events/mojo/event.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {
class RemoteEventDispatcher;
using RemoteEventDispatcherPtr = mojo::InterfacePtr<RemoteEventDispatcher>;
using RemoteEventDispatcherPtrInfo = mojo::InterfacePtrInfo<RemoteEventDispatcher>;
using ThreadSafeRemoteEventDispatcherPtr =
    mojo::ThreadSafeInterfacePtr<RemoteEventDispatcher>;
using RemoteEventDispatcherRequest = mojo::InterfaceRequest<RemoteEventDispatcher>;
using RemoteEventDispatcherAssociatedPtr =
    mojo::AssociatedInterfacePtr<RemoteEventDispatcher>;
using ThreadSafeRemoteEventDispatcherAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RemoteEventDispatcher>;
using RemoteEventDispatcherAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RemoteEventDispatcher>;
using RemoteEventDispatcherAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RemoteEventDispatcher>;


class RemoteEventDispatcherProxy;

template <typename ImplRefTraits>
class RemoteEventDispatcherStub;

class RemoteEventDispatcherRequestValidator;
class RemoteEventDispatcherResponseValidator;

class  RemoteEventDispatcher
    : public RemoteEventDispatcherInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RemoteEventDispatcherProxy;

  template <typename ImplRefTraits>
  using Stub_ = RemoteEventDispatcherStub<ImplRefTraits>;

  using RequestValidator_ = RemoteEventDispatcherRequestValidator;
  using ResponseValidator_ = RemoteEventDispatcherResponseValidator;
  enum MethodMinVersions : uint32_t {
    kDispatchEventMinVersion = 0,
  };
  virtual ~RemoteEventDispatcher() {}


  using DispatchEventCallback = base::Callback<void(bool)>;
  virtual void DispatchEvent(int64_t display_id, std::unique_ptr<ui::Event> event, const DispatchEventCallback& callback) = 0;
};
class  RemoteEventDispatcherInterceptorForTesting : public RemoteEventDispatcher {
  virtual RemoteEventDispatcher* GetForwardingInterface() = 0;
  void DispatchEvent(int64_t display_id, std::unique_ptr<ui::Event> event, const DispatchEventCallback& callback) override;
};
class  RemoteEventDispatcherAsyncWaiter {
 public:
  explicit RemoteEventDispatcherAsyncWaiter(RemoteEventDispatcher* proxy);
  ~RemoteEventDispatcherAsyncWaiter();
  void DispatchEvent(
      int64_t display_id, std::unique_ptr<ui::Event> event, bool* out_result);

 private:
  RemoteEventDispatcher* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(RemoteEventDispatcherAsyncWaiter);
};

class  RemoteEventDispatcherProxy
    : public RemoteEventDispatcher {
 public:
  explicit RemoteEventDispatcherProxy(mojo::MessageReceiverWithResponder* receiver);
  void DispatchEvent(int64_t display_id, std::unique_ptr<ui::Event> event, const DispatchEventCallback& callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  RemoteEventDispatcherStubDispatch {
 public:
  static bool Accept(RemoteEventDispatcher* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RemoteEventDispatcher* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RemoteEventDispatcher>>
class RemoteEventDispatcherStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RemoteEventDispatcherStub() {}
  ~RemoteEventDispatcherStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RemoteEventDispatcherStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RemoteEventDispatcherStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  RemoteEventDispatcherRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  RemoteEventDispatcherResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_REMOTE_EVENT_DISPATCHER_MOJOM_H_