// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_USER_ACCESS_MANAGER_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_USER_ACCESS_MANAGER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/user_access_manager.mojom-shared.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {
class UserAccessManager;
using UserAccessManagerPtr = mojo::InterfacePtr<UserAccessManager>;
using UserAccessManagerPtrInfo = mojo::InterfacePtrInfo<UserAccessManager>;
using ThreadSafeUserAccessManagerPtr =
    mojo::ThreadSafeInterfacePtr<UserAccessManager>;
using UserAccessManagerRequest = mojo::InterfaceRequest<UserAccessManager>;
using UserAccessManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<UserAccessManager>;
using ThreadSafeUserAccessManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<UserAccessManager>;
using UserAccessManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<UserAccessManager>;
using UserAccessManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<UserAccessManager>;


class UserAccessManagerProxy;

template <typename ImplRefTraits>
class UserAccessManagerStub;

class UserAccessManagerRequestValidator;

class  UserAccessManager
    : public UserAccessManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = UserAccessManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = UserAccessManagerStub<ImplRefTraits>;

  using RequestValidator_ = UserAccessManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetActiveUserMinVersion = 0,
  };
  virtual ~UserAccessManager() {}

  virtual void SetActiveUser(const std::string& user_id) = 0;
};
class  UserAccessManagerInterceptorForTesting : public UserAccessManager {
  virtual UserAccessManager* GetForwardingInterface() = 0;
  void SetActiveUser(const std::string& user_id) override;
};
class  UserAccessManagerAsyncWaiter {
 public:
  explicit UserAccessManagerAsyncWaiter(UserAccessManager* proxy);
  ~UserAccessManagerAsyncWaiter();

 private:
  UserAccessManager* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(UserAccessManagerAsyncWaiter);
};

class  UserAccessManagerProxy
    : public UserAccessManager {
 public:
  explicit UserAccessManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetActiveUser(const std::string& user_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  UserAccessManagerStubDispatch {
 public:
  static bool Accept(UserAccessManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      UserAccessManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<UserAccessManager>>
class UserAccessManagerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  UserAccessManagerStub() {}
  ~UserAccessManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return UserAccessManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return UserAccessManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  UserAccessManagerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_USER_ACCESS_MANAGER_MOJOM_H_