// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_SHARED_H_
#define SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/video_capture/public/interfaces/virtual_device.mojom-shared-internal.h"
#include "media/capture/mojo/video_capture_types.mojom-shared.h"
#include "media/mojo/interfaces/media_types.mojom-shared.h"
#include "services/video_capture/public/interfaces/producer.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace video_capture {
namespace mojom {


}  // namespace mojom
}  // namespace video_capture

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace video_capture {
namespace mojom {
// Interface base classes. They are used for type safety check.
class VirtualDeviceInterfaceBase {};

using VirtualDevicePtrDataView =
    mojo::InterfacePtrDataView<VirtualDeviceInterfaceBase>;
using VirtualDeviceRequestDataView =
    mojo::InterfaceRequestDataView<VirtualDeviceInterfaceBase>;
using VirtualDeviceAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<VirtualDeviceInterfaceBase>;
using VirtualDeviceAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<VirtualDeviceInterfaceBase>;
class VirtualDevice_RequestFrameBuffer_ParamsDataView {
 public:
  VirtualDevice_RequestFrameBuffer_ParamsDataView() {}

  VirtualDevice_RequestFrameBuffer_ParamsDataView(
      internal::VirtualDevice_RequestFrameBuffer_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDimensionDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDimension(UserType* output) {
    auto* pointer = data_->dimension.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPixelFormat(UserType* output) const {
    auto data_value = data_->pixel_format;
    return mojo::internal::Deserialize<::media::mojom::VideoPixelFormat>(
        data_value, output);
  }

  ::media::mojom::VideoPixelFormat pixel_format() const {
    return static_cast<::media::mojom::VideoPixelFormat>(data_->pixel_format);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPixelStorage(UserType* output) const {
    auto data_value = data_->pixel_storage;
    return mojo::internal::Deserialize<::media::mojom::VideoPixelStorage>(
        data_value, output);
  }

  ::media::mojom::VideoPixelStorage pixel_storage() const {
    return static_cast<::media::mojom::VideoPixelStorage>(data_->pixel_storage);
  }
 private:
  internal::VirtualDevice_RequestFrameBuffer_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class VirtualDevice_RequestFrameBuffer_ResponseParamsDataView {
 public:
  VirtualDevice_RequestFrameBuffer_ResponseParamsDataView() {}

  VirtualDevice_RequestFrameBuffer_ResponseParamsDataView(
      internal::VirtualDevice_RequestFrameBuffer_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t buffer_id() const {
    return data_->buffer_id;
  }
 private:
  internal::VirtualDevice_RequestFrameBuffer_ResponseParams_Data* data_ = nullptr;
};

class VirtualDevice_OnFrameReadyInBuffer_ParamsDataView {
 public:
  VirtualDevice_OnFrameReadyInBuffer_ParamsDataView() {}

  VirtualDevice_OnFrameReadyInBuffer_ParamsDataView(
      internal::VirtualDevice_OnFrameReadyInBuffer_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t buffer_id() const {
    return data_->buffer_id;
  }
  inline void GetFrameInfoDataView(
      ::media::mojom::VideoFrameInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFrameInfo(UserType* output) {
    auto* pointer = data_->frame_info.Get();
    return mojo::internal::Deserialize<::media::mojom::VideoFrameInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::VirtualDevice_OnFrameReadyInBuffer_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace video_capture

namespace std {

}  // namespace std

namespace mojo {

}  // namespace mojo


namespace video_capture {
namespace mojom {

inline void VirtualDevice_RequestFrameBuffer_ParamsDataView::GetDimensionDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->dimension.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}




inline void VirtualDevice_OnFrameReadyInBuffer_ParamsDataView::GetFrameInfoDataView(
    ::media::mojom::VideoFrameInfoDataView* output) {
  auto pointer = data_->frame_info.Get();
  *output = ::media::mojom::VideoFrameInfoDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace video_capture

#endif  // SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_SHARED_H_
