// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_H_
#define SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/video_capture/public/interfaces/virtual_device.mojom-shared.h"
#include "media/capture/mojo/video_capture_types.mojom.h"
#include "media/mojo/interfaces/media_types.mojom.h"
#include "services/video_capture/public/interfaces/producer.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace video_capture {
namespace mojom {
class VirtualDevice;
using VirtualDevicePtr = mojo::InterfacePtr<VirtualDevice>;
using VirtualDevicePtrInfo = mojo::InterfacePtrInfo<VirtualDevice>;
using ThreadSafeVirtualDevicePtr =
    mojo::ThreadSafeInterfacePtr<VirtualDevice>;
using VirtualDeviceRequest = mojo::InterfaceRequest<VirtualDevice>;
using VirtualDeviceAssociatedPtr =
    mojo::AssociatedInterfacePtr<VirtualDevice>;
using ThreadSafeVirtualDeviceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<VirtualDevice>;
using VirtualDeviceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<VirtualDevice>;
using VirtualDeviceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<VirtualDevice>;


class VirtualDeviceProxy;

template <typename ImplRefTraits>
class VirtualDeviceStub;

class VirtualDeviceRequestValidator;
class VirtualDeviceResponseValidator;

class  VirtualDevice
    : public VirtualDeviceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = VirtualDeviceProxy;

  template <typename ImplRefTraits>
  using Stub_ = VirtualDeviceStub<ImplRefTraits>;

  using RequestValidator_ = VirtualDeviceRequestValidator;
  using ResponseValidator_ = VirtualDeviceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestFrameBufferMinVersion = 0,
    kOnFrameReadyInBufferMinVersion = 0,
  };
  virtual ~VirtualDevice() {}


  using RequestFrameBufferCallback = base::OnceCallback<void(int32_t)>;
  virtual void RequestFrameBuffer(const gfx::Size& dimension, media::VideoPixelFormat pixel_format, media::VideoPixelStorage pixel_storage, RequestFrameBufferCallback callback) = 0;

  virtual void OnFrameReadyInBuffer(int32_t buffer_id, ::media::mojom::VideoFrameInfoPtr frame_info) = 0;
};
class  VirtualDeviceInterceptorForTesting : public VirtualDevice {
  virtual VirtualDevice* GetForwardingInterface() = 0;
  void RequestFrameBuffer(const gfx::Size& dimension, media::VideoPixelFormat pixel_format, media::VideoPixelStorage pixel_storage, RequestFrameBufferCallback callback) override;
  void OnFrameReadyInBuffer(int32_t buffer_id, ::media::mojom::VideoFrameInfoPtr frame_info) override;
};
class  VirtualDeviceAsyncWaiter {
 public:
  explicit VirtualDeviceAsyncWaiter(VirtualDevice* proxy);
  ~VirtualDeviceAsyncWaiter();
  void RequestFrameBuffer(
      const gfx::Size& dimension, media::VideoPixelFormat pixel_format, media::VideoPixelStorage pixel_storage, int32_t* out_buffer_id);

 private:
  VirtualDevice* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(VirtualDeviceAsyncWaiter);
};

class  VirtualDeviceProxy
    : public VirtualDevice {
 public:
  explicit VirtualDeviceProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestFrameBuffer(const gfx::Size& dimension, media::VideoPixelFormat pixel_format, media::VideoPixelStorage pixel_storage, RequestFrameBufferCallback callback) final;
  void OnFrameReadyInBuffer(int32_t buffer_id, ::media::mojom::VideoFrameInfoPtr frame_info) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  VirtualDeviceStubDispatch {
 public:
  static bool Accept(VirtualDevice* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      VirtualDevice* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<VirtualDevice>>
class VirtualDeviceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  VirtualDeviceStub() {}
  ~VirtualDeviceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VirtualDeviceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VirtualDeviceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  VirtualDeviceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  VirtualDeviceResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace video_capture

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_VIRTUAL_DEVICE_MOJOM_H_