// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COMPOSITOR_FRAME_METADATA_MOJOM_H_
#define SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COMPOSITOR_FRAME_METADATA_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/viz/public/interfaces/compositing/compositor_frame_metadata.mojom-shared.h"
#include "services/viz/public/interfaces/compositing/begin_frame_args.mojom.h"
#include "services/viz/public/interfaces/compositing/frame_deadline.mojom.h"
#include "services/viz/public/interfaces/compositing/selection.mojom.h"
#include "services/viz/public/interfaces/compositing/surface_id.mojom.h"
#include "ui/latency/mojo/latency_info.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "components/viz/common/quads/compositor_frame_metadata.h"


namespace viz {
namespace mojom {
class CompositorFrameMetadata;
using CompositorFrameMetadataPtr = mojo::StructPtr<CompositorFrameMetadata>;






class  CompositorFrameMetadata {
 public:
  using DataView = CompositorFrameMetadataDataView;
  using Data_ = internal::CompositorFrameMetadata_Data;

  template <typename... Args>
  static CompositorFrameMetadataPtr New(Args&&... args) {
    return CompositorFrameMetadataPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CompositorFrameMetadataPtr From(const U& u) {
    return mojo::TypeConverter<CompositorFrameMetadataPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CompositorFrameMetadata>::Convert(*this);
  }


  CompositorFrameMetadata();

  CompositorFrameMetadata(
      float device_scale_factor,
      const gfx::Vector2dF& root_scroll_offset,
      float page_scale_factor,
      const gfx::SizeF& scrollable_viewport_size,
      const gfx::SizeF& root_layer_size,
      float min_page_scale_factor,
      float max_page_scale_factor,
      bool root_overflow_x_hidden,
      bool root_overflow_y_hidden,
      bool may_contain_video,
      bool is_resourceless_software_draw_with_scroll_or_animation,
      float top_controls_height,
      float top_controls_shown_ratio,
      float bottom_controls_height,
      float bottom_controls_shown_ratio,
      uint32_t root_background_color,
      const viz::Selection<gfx::SelectionBound>& selection,
      const std::vector<ui::LatencyInfo>& latency_info,
      const std::vector<viz::SurfaceId>& referenced_surfaces,
      const base::Optional<uint32_t>& deadline_in_frames,
      const std::vector<viz::SurfaceId>& activation_dependencies,
      uint32_t content_source_id,
      const viz::BeginFrameAck& begin_frame_ack,
      uint32_t frame_token,
      uint32_t presentation_token);

  ~CompositorFrameMetadata();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CompositorFrameMetadataPtr>
  CompositorFrameMetadataPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CompositorFrameMetadata>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CompositorFrameMetadata::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CompositorFrameMetadata::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CompositorFrameMetadata_UnserializedMessageContext<
            UserType, CompositorFrameMetadata::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CompositorFrameMetadata::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CompositorFrameMetadata::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CompositorFrameMetadata_UnserializedMessageContext<
            UserType, CompositorFrameMetadata::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CompositorFrameMetadata::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  float device_scale_factor;
  gfx::Vector2dF root_scroll_offset;
  float page_scale_factor;
  gfx::SizeF scrollable_viewport_size;
  gfx::SizeF root_layer_size;
  float min_page_scale_factor;
  float max_page_scale_factor;
  bool root_overflow_x_hidden;
  bool root_overflow_y_hidden;
  bool may_contain_video;
  bool is_resourceless_software_draw_with_scroll_or_animation;
  float top_controls_height;
  float top_controls_shown_ratio;
  float bottom_controls_height;
  float bottom_controls_shown_ratio;
  uint32_t root_background_color;
  viz::Selection<gfx::SelectionBound> selection;
  std::vector<ui::LatencyInfo> latency_info;
  std::vector<viz::SurfaceId> referenced_surfaces;
  base::Optional<uint32_t> deadline_in_frames;
  std::vector<viz::SurfaceId> activation_dependencies;
  uint32_t content_source_id;
  viz::BeginFrameAck begin_frame_ack;
  uint32_t frame_token;
  uint32_t presentation_token;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
CompositorFrameMetadataPtr CompositorFrameMetadata::Clone() const {
  return New(
      mojo::Clone(device_scale_factor),
      mojo::Clone(root_scroll_offset),
      mojo::Clone(page_scale_factor),
      mojo::Clone(scrollable_viewport_size),
      mojo::Clone(root_layer_size),
      mojo::Clone(min_page_scale_factor),
      mojo::Clone(max_page_scale_factor),
      mojo::Clone(root_overflow_x_hidden),
      mojo::Clone(root_overflow_y_hidden),
      mojo::Clone(may_contain_video),
      mojo::Clone(is_resourceless_software_draw_with_scroll_or_animation),
      mojo::Clone(top_controls_height),
      mojo::Clone(top_controls_shown_ratio),
      mojo::Clone(bottom_controls_height),
      mojo::Clone(bottom_controls_shown_ratio),
      mojo::Clone(root_background_color),
      mojo::Clone(selection),
      mojo::Clone(latency_info),
      mojo::Clone(referenced_surfaces),
      mojo::Clone(deadline_in_frames),
      mojo::Clone(activation_dependencies),
      mojo::Clone(content_source_id),
      mojo::Clone(begin_frame_ack),
      mojo::Clone(frame_token),
      mojo::Clone(presentation_token)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CompositorFrameMetadata>::value>::type*>
bool CompositorFrameMetadata::Equals(const T& other) const {
  if (!mojo::Equals(this->device_scale_factor, other.device_scale_factor))
    return false;
  if (!mojo::Equals(this->root_scroll_offset, other.root_scroll_offset))
    return false;
  if (!mojo::Equals(this->page_scale_factor, other.page_scale_factor))
    return false;
  if (!mojo::Equals(this->scrollable_viewport_size, other.scrollable_viewport_size))
    return false;
  if (!mojo::Equals(this->root_layer_size, other.root_layer_size))
    return false;
  if (!mojo::Equals(this->min_page_scale_factor, other.min_page_scale_factor))
    return false;
  if (!mojo::Equals(this->max_page_scale_factor, other.max_page_scale_factor))
    return false;
  if (!mojo::Equals(this->root_overflow_x_hidden, other.root_overflow_x_hidden))
    return false;
  if (!mojo::Equals(this->root_overflow_y_hidden, other.root_overflow_y_hidden))
    return false;
  if (!mojo::Equals(this->may_contain_video, other.may_contain_video))
    return false;
  if (!mojo::Equals(this->is_resourceless_software_draw_with_scroll_or_animation, other.is_resourceless_software_draw_with_scroll_or_animation))
    return false;
  if (!mojo::Equals(this->top_controls_height, other.top_controls_height))
    return false;
  if (!mojo::Equals(this->top_controls_shown_ratio, other.top_controls_shown_ratio))
    return false;
  if (!mojo::Equals(this->bottom_controls_height, other.bottom_controls_height))
    return false;
  if (!mojo::Equals(this->bottom_controls_shown_ratio, other.bottom_controls_shown_ratio))
    return false;
  if (!mojo::Equals(this->root_background_color, other.root_background_color))
    return false;
  if (!mojo::Equals(this->selection, other.selection))
    return false;
  if (!mojo::Equals(this->latency_info, other.latency_info))
    return false;
  if (!mojo::Equals(this->referenced_surfaces, other.referenced_surfaces))
    return false;
  if (!mojo::Equals(this->deadline_in_frames, other.deadline_in_frames))
    return false;
  if (!mojo::Equals(this->activation_dependencies, other.activation_dependencies))
    return false;
  if (!mojo::Equals(this->content_source_id, other.content_source_id))
    return false;
  if (!mojo::Equals(this->begin_frame_ack, other.begin_frame_ack))
    return false;
  if (!mojo::Equals(this->frame_token, other.frame_token))
    return false;
  if (!mojo::Equals(this->presentation_token, other.presentation_token))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace viz

namespace mojo {


template <>
struct  StructTraits<::viz::mojom::CompositorFrameMetadata::DataView,
                                         ::viz::mojom::CompositorFrameMetadataPtr> {
  static bool IsNull(const ::viz::mojom::CompositorFrameMetadataPtr& input) { return !input; }
  static void SetToNull(::viz::mojom::CompositorFrameMetadataPtr* output) { output->reset(); }

  static decltype(::viz::mojom::CompositorFrameMetadata::device_scale_factor) device_scale_factor(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->device_scale_factor;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::root_scroll_offset)& root_scroll_offset(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->root_scroll_offset;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::page_scale_factor) page_scale_factor(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->page_scale_factor;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::scrollable_viewport_size)& scrollable_viewport_size(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->scrollable_viewport_size;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::root_layer_size)& root_layer_size(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->root_layer_size;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::min_page_scale_factor) min_page_scale_factor(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->min_page_scale_factor;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::max_page_scale_factor) max_page_scale_factor(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->max_page_scale_factor;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::root_overflow_x_hidden) root_overflow_x_hidden(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->root_overflow_x_hidden;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::root_overflow_y_hidden) root_overflow_y_hidden(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->root_overflow_y_hidden;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::may_contain_video) may_contain_video(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->may_contain_video;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::is_resourceless_software_draw_with_scroll_or_animation) is_resourceless_software_draw_with_scroll_or_animation(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->is_resourceless_software_draw_with_scroll_or_animation;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::top_controls_height) top_controls_height(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->top_controls_height;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::top_controls_shown_ratio) top_controls_shown_ratio(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->top_controls_shown_ratio;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::bottom_controls_height) bottom_controls_height(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->bottom_controls_height;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::bottom_controls_shown_ratio) bottom_controls_shown_ratio(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->bottom_controls_shown_ratio;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::root_background_color) root_background_color(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->root_background_color;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::selection)& selection(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->selection;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::latency_info)& latency_info(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->latency_info;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::referenced_surfaces)& referenced_surfaces(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->referenced_surfaces;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::deadline_in_frames)& deadline_in_frames(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->deadline_in_frames;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::activation_dependencies)& activation_dependencies(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->activation_dependencies;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::content_source_id) content_source_id(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->content_source_id;
  }

  static const decltype(::viz::mojom::CompositorFrameMetadata::begin_frame_ack)& begin_frame_ack(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->begin_frame_ack;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::frame_token) frame_token(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->frame_token;
  }

  static decltype(::viz::mojom::CompositorFrameMetadata::presentation_token) presentation_token(
      const ::viz::mojom::CompositorFrameMetadataPtr& input) {
    return input->presentation_token;
  }

  static bool Read(::viz::mojom::CompositorFrameMetadata::DataView input, ::viz::mojom::CompositorFrameMetadataPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COMPOSITOR_FRAME_METADATA_MOJOM_H_