// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_
#define SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/viz/public/interfaces/compositing/copy_output_request.mojom-shared-internal.h"
#include "gpu/ipc/common/mailbox.mojom-shared.h"
#include "gpu/ipc/common/sync_token.mojom-shared.h"
#include "services/viz/public/interfaces/compositing/copy_output_result.mojom-shared.h"
#include "mojo/common/unguessable_token.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace viz {
namespace mojom {
class CopyOutputRequestDataView;



}  // namespace mojom
}  // namespace viz

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::viz::mojom::CopyOutputRequestDataView> {
  using Data = ::viz::mojom::internal::CopyOutputRequest_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace viz {
namespace mojom {
// Interface base classes. They are used for type safety check.
class CopyOutputResultSenderInterfaceBase {};

using CopyOutputResultSenderPtrDataView =
    mojo::InterfacePtrDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderRequestDataView =
    mojo::InterfaceRequestDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CopyOutputResultSenderInterfaceBase>;
using CopyOutputResultSenderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CopyOutputResultSenderInterfaceBase>;
class CopyOutputRequestDataView {
 public:
  CopyOutputRequestDataView() {}

  CopyOutputRequestDataView(
      internal::CopyOutputRequest_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResultFormat(UserType* output) const {
    auto data_value = data_->result_format;
    return mojo::internal::Deserialize<::viz::mojom::CopyOutputResultFormat>(
        data_value, output);
  }

  ::viz::mojom::CopyOutputResultFormat result_format() const {
    return static_cast<::viz::mojom::CopyOutputResultFormat>(data_->result_format);
  }
  inline void GetScaleFromDataView(
      ::gfx::mojom::Vector2dDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadScaleFrom(UserType* output) {
    auto* pointer = data_->scale_from.Get();
    return mojo::internal::Deserialize<::gfx::mojom::Vector2dDataView>(
        pointer, output, context_);
  }
  inline void GetScaleToDataView(
      ::gfx::mojom::Vector2dDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadScaleTo(UserType* output) {
    auto* pointer = data_->scale_to.Get();
    return mojo::internal::Deserialize<::gfx::mojom::Vector2dDataView>(
        pointer, output, context_);
  }
  inline void GetSourceDataView(
      ::mojo::common::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSource(UserType* output) {
    auto* pointer = data_->source.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
  inline void GetAreaDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadArea(UserType* output) {
    auto* pointer = data_->area.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetMailboxDataView(
      ::gpu::mojom::MailboxDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMailbox(UserType* output) {
    auto* pointer = data_->mailbox.Get();
    return mojo::internal::Deserialize<::gpu::mojom::MailboxDataView>(
        pointer, output, context_);
  }
  inline void GetSyncTokenDataView(
      ::gpu::mojom::SyncTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSyncToken(UserType* output) {
    auto* pointer = data_->sync_token.Get();
    return mojo::internal::Deserialize<::gpu::mojom::SyncTokenDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeResultSender() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::viz::mojom::CopyOutputResultSenderPtrDataView>(
            &data_->result_sender, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::CopyOutputRequest_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CopyOutputResultSender_SendResult_ParamsDataView {
 public:
  CopyOutputResultSender_SendResult_ParamsDataView() {}

  CopyOutputResultSender_SendResult_ParamsDataView(
      internal::CopyOutputResultSender_SendResult_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      ::viz::mojom::CopyOutputResultDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::viz::mojom::CopyOutputResultDataView>(
        pointer, output, context_);
  }
 private:
  internal::CopyOutputResultSender_SendResult_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace viz

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::viz::mojom::CopyOutputRequestDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::viz::mojom::CopyOutputRequestDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::viz::mojom::internal::CopyOutputRequest_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::viz::mojom::CopyOutputResultFormat>(
        CallWithContext(Traits::result_format, input, custom_context), &(*output)->result_format);
    decltype(CallWithContext(Traits::scale_from, input, custom_context)) in_scale_from = CallWithContext(Traits::scale_from, input, custom_context);
    typename decltype((*output)->scale_from)::BaseType::BufferWriter
        scale_from_writer;
    mojo::internal::Serialize<::gfx::mojom::Vector2dDataView>(
        in_scale_from, buffer, &scale_from_writer, context);
    (*output)->scale_from.Set(
        scale_from_writer.is_null() ? nullptr : scale_from_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->scale_from.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null scale_from in CopyOutputRequest struct");
    decltype(CallWithContext(Traits::scale_to, input, custom_context)) in_scale_to = CallWithContext(Traits::scale_to, input, custom_context);
    typename decltype((*output)->scale_to)::BaseType::BufferWriter
        scale_to_writer;
    mojo::internal::Serialize<::gfx::mojom::Vector2dDataView>(
        in_scale_to, buffer, &scale_to_writer, context);
    (*output)->scale_to.Set(
        scale_to_writer.is_null() ? nullptr : scale_to_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->scale_to.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null scale_to in CopyOutputRequest struct");
    decltype(CallWithContext(Traits::source, input, custom_context)) in_source = CallWithContext(Traits::source, input, custom_context);
    typename decltype((*output)->source)::BaseType::BufferWriter
        source_writer;
    mojo::internal::Serialize<::mojo::common::mojom::UnguessableTokenDataView>(
        in_source, buffer, &source_writer, context);
    (*output)->source.Set(
        source_writer.is_null() ? nullptr : source_writer.data());
    decltype(CallWithContext(Traits::area, input, custom_context)) in_area = CallWithContext(Traits::area, input, custom_context);
    typename decltype((*output)->area)::BaseType::BufferWriter
        area_writer;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_area, buffer, &area_writer, context);
    (*output)->area.Set(
        area_writer.is_null() ? nullptr : area_writer.data());
    decltype(CallWithContext(Traits::mailbox, input, custom_context)) in_mailbox = CallWithContext(Traits::mailbox, input, custom_context);
    typename decltype((*output)->mailbox)::BaseType::BufferWriter
        mailbox_writer;
    mojo::internal::Serialize<::gpu::mojom::MailboxDataView>(
        in_mailbox, buffer, &mailbox_writer, context);
    (*output)->mailbox.Set(
        mailbox_writer.is_null() ? nullptr : mailbox_writer.data());
    decltype(CallWithContext(Traits::sync_token, input, custom_context)) in_sync_token = CallWithContext(Traits::sync_token, input, custom_context);
    typename decltype((*output)->sync_token)::BaseType::BufferWriter
        sync_token_writer;
    mojo::internal::Serialize<::gpu::mojom::SyncTokenDataView>(
        in_sync_token, buffer, &sync_token_writer, context);
    (*output)->sync_token.Set(
        sync_token_writer.is_null() ? nullptr : sync_token_writer.data());
    decltype(CallWithContext(Traits::result_sender, input, custom_context)) in_result_sender = CallWithContext(Traits::result_sender, input, custom_context);
    mojo::internal::Serialize<::viz::mojom::CopyOutputResultSenderPtrDataView>(
        in_result_sender, &(*output)->result_sender, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid((*output)->result_sender),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid result_sender in CopyOutputRequest struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::viz::mojom::internal::CopyOutputRequest_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::viz::mojom::CopyOutputRequestDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace viz {
namespace mojom {

inline void CopyOutputRequestDataView::GetScaleFromDataView(
    ::gfx::mojom::Vector2dDataView* output) {
  auto pointer = data_->scale_from.Get();
  *output = ::gfx::mojom::Vector2dDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetScaleToDataView(
    ::gfx::mojom::Vector2dDataView* output) {
  auto pointer = data_->scale_to.Get();
  *output = ::gfx::mojom::Vector2dDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetSourceDataView(
    ::mojo::common::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->source.Get();
  *output = ::mojo::common::mojom::UnguessableTokenDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetAreaDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->area.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetMailboxDataView(
    ::gpu::mojom::MailboxDataView* output) {
  auto pointer = data_->mailbox.Get();
  *output = ::gpu::mojom::MailboxDataView(pointer, context_);
}
inline void CopyOutputRequestDataView::GetSyncTokenDataView(
    ::gpu::mojom::SyncTokenDataView* output) {
  auto pointer = data_->sync_token.Get();
  *output = ::gpu::mojom::SyncTokenDataView(pointer, context_);
}


inline void CopyOutputResultSender_SendResult_ParamsDataView::GetResultDataView(
    ::viz::mojom::CopyOutputResultDataView* output) {
  auto pointer = data_->result.Get();
  *output = ::viz::mojom::CopyOutputResultDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace viz

#endif  // SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_COPY_OUTPUT_REQUEST_MOJOM_SHARED_H_
