// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_SHARED_BITMAP_ALLOCATION_NOTIFIER_MOJOM_BLINK_H_
#define SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_SHARED_BITMAP_ALLOCATION_NOTIFIER_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/viz/public/interfaces/compositing/shared_bitmap_allocation_notifier.mojom-shared.h"
#include "gpu/ipc/common/mailbox.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace viz {
namespace mojom {
namespace blink {
class SharedBitmapAllocationNotifier;
using SharedBitmapAllocationNotifierPtr = mojo::InterfacePtr<SharedBitmapAllocationNotifier>;
using SharedBitmapAllocationNotifierPtrInfo = mojo::InterfacePtrInfo<SharedBitmapAllocationNotifier>;
using ThreadSafeSharedBitmapAllocationNotifierPtr =
    mojo::ThreadSafeInterfacePtr<SharedBitmapAllocationNotifier>;
using SharedBitmapAllocationNotifierRequest = mojo::InterfaceRequest<SharedBitmapAllocationNotifier>;
using SharedBitmapAllocationNotifierAssociatedPtr =
    mojo::AssociatedInterfacePtr<SharedBitmapAllocationNotifier>;
using ThreadSafeSharedBitmapAllocationNotifierAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<SharedBitmapAllocationNotifier>;
using SharedBitmapAllocationNotifierAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<SharedBitmapAllocationNotifier>;
using SharedBitmapAllocationNotifierAssociatedRequest =
    mojo::AssociatedInterfaceRequest<SharedBitmapAllocationNotifier>;


class SharedBitmapAllocationNotifierProxy;

template <typename ImplRefTraits>
class SharedBitmapAllocationNotifierStub;

class SharedBitmapAllocationNotifierRequestValidator;

class  SharedBitmapAllocationNotifier
    : public SharedBitmapAllocationNotifierInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = SharedBitmapAllocationNotifierProxy;

  template <typename ImplRefTraits>
  using Stub_ = SharedBitmapAllocationNotifierStub<ImplRefTraits>;

  using RequestValidator_ = SharedBitmapAllocationNotifierRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kDidAllocateSharedBitmapMinVersion = 0,
    kDidDeleteSharedBitmapMinVersion = 0,
  };
  virtual ~SharedBitmapAllocationNotifier() {}

  virtual void DidAllocateSharedBitmap(mojo::ScopedSharedBufferHandle buffer, ::gpu::mojom::blink::MailboxPtr id) = 0;

  virtual void DidDeleteSharedBitmap(::gpu::mojom::blink::MailboxPtr id) = 0;
};
class  SharedBitmapAllocationNotifierInterceptorForTesting : public SharedBitmapAllocationNotifier {
  virtual SharedBitmapAllocationNotifier* GetForwardingInterface() = 0;
  void DidAllocateSharedBitmap(mojo::ScopedSharedBufferHandle buffer, ::gpu::mojom::blink::MailboxPtr id) override;
  void DidDeleteSharedBitmap(::gpu::mojom::blink::MailboxPtr id) override;
};
class  SharedBitmapAllocationNotifierAsyncWaiter {
 public:
  explicit SharedBitmapAllocationNotifierAsyncWaiter(SharedBitmapAllocationNotifier* proxy);
  ~SharedBitmapAllocationNotifierAsyncWaiter();

 private:
  SharedBitmapAllocationNotifier* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SharedBitmapAllocationNotifierAsyncWaiter);
};

class  SharedBitmapAllocationNotifierProxy
    : public SharedBitmapAllocationNotifier {
 public:
  explicit SharedBitmapAllocationNotifierProxy(mojo::MessageReceiverWithResponder* receiver);
  void DidAllocateSharedBitmap(mojo::ScopedSharedBufferHandle buffer, ::gpu::mojom::blink::MailboxPtr id) final;
  void DidDeleteSharedBitmap(::gpu::mojom::blink::MailboxPtr id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  SharedBitmapAllocationNotifierStubDispatch {
 public:
  static bool Accept(SharedBitmapAllocationNotifier* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      SharedBitmapAllocationNotifier* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<SharedBitmapAllocationNotifier>>
class SharedBitmapAllocationNotifierStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  SharedBitmapAllocationNotifierStub() {}
  ~SharedBitmapAllocationNotifierStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return SharedBitmapAllocationNotifierStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return SharedBitmapAllocationNotifierStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  SharedBitmapAllocationNotifierRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace viz

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_VIZ_PUBLIC_INTERFACES_COMPOSITING_SHARED_BITMAP_ALLOCATION_NOTIFIER_MOJOM_BLINK_H_