// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_REGISTRY_MOJOM_H_
#define THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_REGISTRY_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/common/blob/blob_registry.mojom-shared.h"
#include "mojo/common/file.mojom.h"
#include "mojo/common/file_path.mojom.h"
#include "mojo/common/time.mojom.h"
#include "third_party/WebKit/common/blob/blob.mojom.h"
#include "third_party/WebKit/common/blob/blob_url_store.mojom.h"
#include "url/mojo/origin.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/common/common_export.h"


namespace blink {
namespace mojom {
class BlobRegistry;
using BlobRegistryPtr = mojo::InterfacePtr<BlobRegistry>;
using BlobRegistryPtrInfo = mojo::InterfacePtrInfo<BlobRegistry>;
using ThreadSafeBlobRegistryPtr =
    mojo::ThreadSafeInterfacePtr<BlobRegistry>;
using BlobRegistryRequest = mojo::InterfaceRequest<BlobRegistry>;
using BlobRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<BlobRegistry>;
using ThreadSafeBlobRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BlobRegistry>;
using BlobRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BlobRegistry>;
using BlobRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BlobRegistry>;

class BytesProvider;
using BytesProviderPtr = mojo::InterfacePtr<BytesProvider>;
using BytesProviderPtrInfo = mojo::InterfacePtrInfo<BytesProvider>;
using ThreadSafeBytesProviderPtr =
    mojo::ThreadSafeInterfacePtr<BytesProvider>;
using BytesProviderRequest = mojo::InterfaceRequest<BytesProvider>;
using BytesProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<BytesProvider>;
using ThreadSafeBytesProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BytesProvider>;
using BytesProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BytesProvider>;
using BytesProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BytesProvider>;

class DataElementBytes;
using DataElementBytesPtr = mojo::StructPtr<DataElementBytes>;

class DataElementFile;
using DataElementFilePtr = mojo::StructPtr<DataElementFile>;

class DataElementFilesystemURL;
using DataElementFilesystemURLPtr = mojo::StructPtr<DataElementFilesystemURL>;

class DataElementBlob;
using DataElementBlobPtr = mojo::StructPtr<DataElementBlob>;

class DataElement;

typedef mojo::StructPtr<DataElement> DataElementPtr;


class BlobRegistryProxy;

template <typename ImplRefTraits>
class BlobRegistryStub;

class BlobRegistryRequestValidator;
class BlobRegistryResponseValidator;

class BLINK_COMMON_EXPORT BlobRegistry
    : public BlobRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = BlobRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = BlobRegistryStub<ImplRefTraits>;

  using RequestValidator_ = BlobRegistryRequestValidator;
  using ResponseValidator_ = BlobRegistryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterMinVersion = 0,
    kGetBlobFromUUIDMinVersion = 0,
    kURLStoreForOriginMinVersion = 0,
  };
  virtual ~BlobRegistry() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool Register(::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements);

  using RegisterCallback = base::OnceCallback<void()>;
  virtual void Register(::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements, RegisterCallback callback) = 0;

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool GetBlobFromUUID(::blink::mojom::BlobRequest blob, const std::string& uuid);

  using GetBlobFromUUIDCallback = base::OnceCallback<void()>;
  virtual void GetBlobFromUUID(::blink::mojom::BlobRequest blob, const std::string& uuid, GetBlobFromUUIDCallback callback) = 0;

  virtual void URLStoreForOrigin(const url::Origin& origin, ::blink::mojom::BlobURLStoreAssociatedRequest url_store) = 0;
};
class BLINK_COMMON_EXPORT BlobRegistryInterceptorForTesting : public BlobRegistry {
  virtual BlobRegistry* GetForwardingInterface() = 0;
  void Register(::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements, RegisterCallback callback) override;
  void GetBlobFromUUID(::blink::mojom::BlobRequest blob, const std::string& uuid, GetBlobFromUUIDCallback callback) override;
  void URLStoreForOrigin(const url::Origin& origin, ::blink::mojom::BlobURLStoreAssociatedRequest url_store) override;
};
class BLINK_COMMON_EXPORT BlobRegistryAsyncWaiter {
 public:
  explicit BlobRegistryAsyncWaiter(BlobRegistry* proxy);
  ~BlobRegistryAsyncWaiter();
  void Register(
      ::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements);
  void GetBlobFromUUID(
      ::blink::mojom::BlobRequest blob, const std::string& uuid);

 private:
  BlobRegistry* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(BlobRegistryAsyncWaiter);
};

class BytesProviderProxy;

template <typename ImplRefTraits>
class BytesProviderStub;

class BytesProviderRequestValidator;
class BytesProviderResponseValidator;

class BLINK_COMMON_EXPORT BytesProvider
    : public BytesProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BytesProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = BytesProviderStub<ImplRefTraits>;

  using RequestValidator_ = BytesProviderRequestValidator;
  using ResponseValidator_ = BytesProviderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestAsReplyMinVersion = 0,
    kRequestAsStreamMinVersion = 0,
    kRequestAsFileMinVersion = 0,
  };
  virtual ~BytesProvider() {}


  using RequestAsReplyCallback = base::OnceCallback<void(const std::vector<uint8_t>&)>;
  virtual void RequestAsReply(RequestAsReplyCallback callback) = 0;

  virtual void RequestAsStream(mojo::ScopedDataPipeProducerHandle pipe) = 0;


  using RequestAsFileCallback = base::OnceCallback<void(base::Optional<base::Time>)>;
  virtual void RequestAsFile(uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, RequestAsFileCallback callback) = 0;
};
class BLINK_COMMON_EXPORT BytesProviderInterceptorForTesting : public BytesProvider {
  virtual BytesProvider* GetForwardingInterface() = 0;
  void RequestAsReply(RequestAsReplyCallback callback) override;
  void RequestAsStream(mojo::ScopedDataPipeProducerHandle pipe) override;
  void RequestAsFile(uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, RequestAsFileCallback callback) override;
};
class BLINK_COMMON_EXPORT BytesProviderAsyncWaiter {
 public:
  explicit BytesProviderAsyncWaiter(BytesProvider* proxy);
  ~BytesProviderAsyncWaiter();
  void RequestAsReply(
      std::vector<uint8_t>* out_data);
  void RequestAsFile(
      uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, base::Optional<base::Time>* out_time_file_modified);

 private:
  BytesProvider* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(BytesProviderAsyncWaiter);
};

class BLINK_COMMON_EXPORT BlobRegistryProxy
    : public BlobRegistry {
 public:
  explicit BlobRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  bool Register(::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements) final;
  void Register(::blink::mojom::BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements, RegisterCallback callback) final;
  bool GetBlobFromUUID(::blink::mojom::BlobRequest blob, const std::string& uuid) final;
  void GetBlobFromUUID(::blink::mojom::BlobRequest blob, const std::string& uuid, GetBlobFromUUIDCallback callback) final;
  void URLStoreForOrigin(const url::Origin& origin, ::blink::mojom::BlobURLStoreAssociatedRequest url_store) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_COMMON_EXPORT BytesProviderProxy
    : public BytesProvider {
 public:
  explicit BytesProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestAsReply(RequestAsReplyCallback callback) final;
  void RequestAsStream(mojo::ScopedDataPipeProducerHandle pipe) final;
  void RequestAsFile(uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, RequestAsFileCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_COMMON_EXPORT BlobRegistryStubDispatch {
 public:
  static bool Accept(BlobRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BlobRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BlobRegistry>>
class BlobRegistryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BlobRegistryStub() {}
  ~BlobRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_COMMON_EXPORT BytesProviderStubDispatch {
 public:
  static bool Accept(BytesProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BytesProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BytesProvider>>
class BytesProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BytesProviderStub() {}
  ~BytesProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BytesProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BytesProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_COMMON_EXPORT BlobRegistryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_COMMON_EXPORT BytesProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_COMMON_EXPORT BlobRegistryResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_COMMON_EXPORT BytesProviderResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};






class BLINK_COMMON_EXPORT DataElement {
 public:
  using DataView = DataElementDataView;
  using Data_ = internal::DataElement_Data;
  using Tag = Data_::DataElement_Tag;

  static DataElementPtr New() {
    return DataElementPtr(base::in_place);
  }
  // Construct an instance holding |bytes|.
  static DataElementPtr
  NewBytes(
      DataElementBytesPtr bytes) {
    auto result = DataElementPtr(base::in_place);
    result->set_bytes(std::move(bytes));
    return result;
  }
  // Construct an instance holding |file|.
  static DataElementPtr
  NewFile(
      DataElementFilePtr file) {
    auto result = DataElementPtr(base::in_place);
    result->set_file(std::move(file));
    return result;
  }
  // Construct an instance holding |file_filesystem|.
  static DataElementPtr
  NewFileFilesystem(
      DataElementFilesystemURLPtr file_filesystem) {
    auto result = DataElementPtr(base::in_place);
    result->set_file_filesystem(std::move(file_filesystem));
    return result;
  }
  // Construct an instance holding |blob|.
  static DataElementPtr
  NewBlob(
      DataElementBlobPtr blob) {
    auto result = DataElementPtr(base::in_place);
    result->set_blob(std::move(blob));
    return result;
  }

  template <typename U>
  static DataElementPtr From(const U& u) {
    return mojo::TypeConverter<DataElementPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElement>::Convert(*this);
  }

  DataElement();
  ~DataElement();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename UnionPtrType = DataElementPtr>
  DataElementPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElement>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  Tag which() const {
    return tag_;
  }


  bool is_bytes() const { return tag_ == Tag::BYTES; }

  DataElementBytesPtr& get_bytes() const {
    DCHECK(tag_ == Tag::BYTES);
    return *(data_.bytes);
  }

  void set_bytes(
      DataElementBytesPtr bytes);
  bool is_file() const { return tag_ == Tag::FILE; }

  DataElementFilePtr& get_file() const {
    DCHECK(tag_ == Tag::FILE);
    return *(data_.file);
  }

  void set_file(
      DataElementFilePtr file);
  bool is_file_filesystem() const { return tag_ == Tag::FILE_FILESYSTEM; }

  DataElementFilesystemURLPtr& get_file_filesystem() const {
    DCHECK(tag_ == Tag::FILE_FILESYSTEM);
    return *(data_.file_filesystem);
  }

  void set_file_filesystem(
      DataElementFilesystemURLPtr file_filesystem);
  bool is_blob() const { return tag_ == Tag::BLOB; }

  DataElementBlobPtr& get_blob() const {
    DCHECK(tag_ == Tag::BLOB);
    return *(data_.blob);
  }

  void set_blob(
      DataElementBlobPtr blob);

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DataElement::DataView>(input);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    return mojo::internal::DeserializeImpl<DataElement::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

 private:
  union Union_ {
    Union_() {}
    ~Union_() {}
    DataElementBytesPtr* bytes;
    DataElementFilePtr* file;
    DataElementFilesystemURLPtr* file_filesystem;
    DataElementBlobPtr* blob;
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  void DestroyActive();
  Tag tag_;
  Union_ data_;
};


class BLINK_COMMON_EXPORT DataElementBytes {
 public:
  using DataView = DataElementBytesDataView;
  using Data_ = internal::DataElementBytes_Data;
  static constexpr uint64_t kMaximumEmbeddedDataSize = 256000ULL;

  template <typename... Args>
  static DataElementBytesPtr New(Args&&... args) {
    return DataElementBytesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementBytesPtr From(const U& u) {
    return mojo::TypeConverter<DataElementBytesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementBytes>::Convert(*this);
  }


  DataElementBytes();

  DataElementBytes(
      uint64_t length,
      const base::Optional<std::vector<uint8_t>>& embedded_data,
      BytesProviderPtrInfo data);

  ~DataElementBytes();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementBytesPtr>
  DataElementBytesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementBytes>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        DataElementBytes::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DataElementBytes::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::DataElementBytes_UnserializedMessageContext<
            UserType, DataElementBytes::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<DataElementBytes::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return DataElementBytes::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::DataElementBytes_UnserializedMessageContext<
            UserType, DataElementBytes::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<DataElementBytes::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  uint64_t length;
  base::Optional<std::vector<uint8_t>> embedded_data;
  BytesProviderPtrInfo data;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(DataElementBytes);
};


class BLINK_COMMON_EXPORT DataElementFile {
 public:
  using DataView = DataElementFileDataView;
  using Data_ = internal::DataElementFile_Data;

  template <typename... Args>
  static DataElementFilePtr New(Args&&... args) {
    return DataElementFilePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementFilePtr From(const U& u) {
    return mojo::TypeConverter<DataElementFilePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementFile>::Convert(*this);
  }


  DataElementFile();

  DataElementFile(
      const base::FilePath& path,
      uint64_t offset,
      uint64_t length,
      base::Optional<base::Time> expected_modification_time);

  ~DataElementFile();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementFilePtr>
  DataElementFilePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementFile>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        DataElementFile::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DataElementFile::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::DataElementFile_UnserializedMessageContext<
            UserType, DataElementFile::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<DataElementFile::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return DataElementFile::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::DataElementFile_UnserializedMessageContext<
            UserType, DataElementFile::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<DataElementFile::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  base::FilePath path;
  uint64_t offset;
  uint64_t length;
  base::Optional<base::Time> expected_modification_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class BLINK_COMMON_EXPORT DataElementFilesystemURL {
 public:
  using DataView = DataElementFilesystemURLDataView;
  using Data_ = internal::DataElementFilesystemURL_Data;

  template <typename... Args>
  static DataElementFilesystemURLPtr New(Args&&... args) {
    return DataElementFilesystemURLPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementFilesystemURLPtr From(const U& u) {
    return mojo::TypeConverter<DataElementFilesystemURLPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementFilesystemURL>::Convert(*this);
  }


  DataElementFilesystemURL();

  DataElementFilesystemURL(
      const GURL& url,
      uint64_t offset,
      uint64_t length,
      base::Optional<base::Time> expected_modification_time);

  ~DataElementFilesystemURL();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementFilesystemURLPtr>
  DataElementFilesystemURLPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementFilesystemURL>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        DataElementFilesystemURL::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DataElementFilesystemURL::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::DataElementFilesystemURL_UnserializedMessageContext<
            UserType, DataElementFilesystemURL::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<DataElementFilesystemURL::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return DataElementFilesystemURL::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::DataElementFilesystemURL_UnserializedMessageContext<
            UserType, DataElementFilesystemURL::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<DataElementFilesystemURL::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  GURL url;
  uint64_t offset;
  uint64_t length;
  base::Optional<base::Time> expected_modification_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class BLINK_COMMON_EXPORT DataElementBlob {
 public:
  using DataView = DataElementBlobDataView;
  using Data_ = internal::DataElementBlob_Data;

  template <typename... Args>
  static DataElementBlobPtr New(Args&&... args) {
    return DataElementBlobPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementBlobPtr From(const U& u) {
    return mojo::TypeConverter<DataElementBlobPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementBlob>::Convert(*this);
  }


  DataElementBlob();

  DataElementBlob(
      ::blink::mojom::BlobPtrInfo blob,
      uint64_t offset,
      uint64_t length);

  ~DataElementBlob();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementBlobPtr>
  DataElementBlobPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementBlob>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        DataElementBlob::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DataElementBlob::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::DataElementBlob_UnserializedMessageContext<
            UserType, DataElementBlob::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<DataElementBlob::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return DataElementBlob::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::DataElementBlob_UnserializedMessageContext<
            UserType, DataElementBlob::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<DataElementBlob::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::blink::mojom::BlobPtrInfo blob;
  uint64_t offset;
  uint64_t length;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(DataElementBlob);
};

template <typename UnionPtrType>
DataElementPtr DataElement::Clone() const {
  // Use UnionPtrType to prevent the compiler from trying to compile this
  // without being asked.
  UnionPtrType rv(New());
  switch (tag_) {
    case Tag::BYTES:
      rv->set_bytes(mojo::Clone(*data_.bytes));
      break;
    case Tag::FILE:
      rv->set_file(mojo::Clone(*data_.file));
      break;
    case Tag::FILE_FILESYSTEM:
      rv->set_file_filesystem(mojo::Clone(*data_.file_filesystem));
      break;
    case Tag::BLOB:
      rv->set_blob(mojo::Clone(*data_.blob));
      break;
  };
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElement>::value>::type*>
bool DataElement::Equals(const T& other) const {
  if (tag_ != other.which())
    return false;

  switch (tag_) {
    case Tag::BYTES:
      return mojo::Equals(*(data_.bytes), *(other.data_.bytes));
    case Tag::FILE:
      return mojo::Equals(*(data_.file), *(other.data_.file));
    case Tag::FILE_FILESYSTEM:
      return mojo::Equals(*(data_.file_filesystem), *(other.data_.file_filesystem));
    case Tag::BLOB:
      return mojo::Equals(*(data_.blob), *(other.data_.blob));
  };

  return false;
}
template <typename StructPtrType>
DataElementBytesPtr DataElementBytes::Clone() const {
  return New(
      mojo::Clone(length),
      mojo::Clone(embedded_data),
      mojo::Clone(data)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementBytes>::value>::type*>
bool DataElementBytes::Equals(const T& other) const {
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->embedded_data, other.embedded_data))
    return false;
  if (!mojo::Equals(this->data, other.data))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementFilePtr DataElementFile::Clone() const {
  return New(
      mojo::Clone(path),
      mojo::Clone(offset),
      mojo::Clone(length),
      mojo::Clone(expected_modification_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementFile>::value>::type*>
bool DataElementFile::Equals(const T& other) const {
  if (!mojo::Equals(this->path, other.path))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->expected_modification_time, other.expected_modification_time))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementFilesystemURLPtr DataElementFilesystemURL::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(offset),
      mojo::Clone(length),
      mojo::Clone(expected_modification_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementFilesystemURL>::value>::type*>
bool DataElementFilesystemURL::Equals(const T& other) const {
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->expected_modification_time, other.expected_modification_time))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementBlobPtr DataElementBlob::Clone() const {
  return New(
      mojo::Clone(blob),
      mojo::Clone(offset),
      mojo::Clone(length)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementBlob>::value>::type*>
bool DataElementBlob::Equals(const T& other) const {
  if (!mojo::Equals(this->blob, other.blob))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::DataElementBytes::DataView,
                                         ::blink::mojom::DataElementBytesPtr> {
  static bool IsNull(const ::blink::mojom::DataElementBytesPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::DataElementBytesPtr* output) { output->reset(); }

  static decltype(::blink::mojom::DataElementBytes::length) length(
      const ::blink::mojom::DataElementBytesPtr& input) {
    return input->length;
  }

  static const decltype(::blink::mojom::DataElementBytes::embedded_data)& embedded_data(
      const ::blink::mojom::DataElementBytesPtr& input) {
    return input->embedded_data;
  }

  static  decltype(::blink::mojom::DataElementBytes::data)& data(
       ::blink::mojom::DataElementBytesPtr& input) {
    return input->data;
  }

  static bool Read(::blink::mojom::DataElementBytes::DataView input, ::blink::mojom::DataElementBytesPtr* output);
};


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::DataElementFile::DataView,
                                         ::blink::mojom::DataElementFilePtr> {
  static bool IsNull(const ::blink::mojom::DataElementFilePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::DataElementFilePtr* output) { output->reset(); }

  static const decltype(::blink::mojom::DataElementFile::path)& path(
      const ::blink::mojom::DataElementFilePtr& input) {
    return input->path;
  }

  static decltype(::blink::mojom::DataElementFile::offset) offset(
      const ::blink::mojom::DataElementFilePtr& input) {
    return input->offset;
  }

  static decltype(::blink::mojom::DataElementFile::length) length(
      const ::blink::mojom::DataElementFilePtr& input) {
    return input->length;
  }

  static const decltype(::blink::mojom::DataElementFile::expected_modification_time)& expected_modification_time(
      const ::blink::mojom::DataElementFilePtr& input) {
    return input->expected_modification_time;
  }

  static bool Read(::blink::mojom::DataElementFile::DataView input, ::blink::mojom::DataElementFilePtr* output);
};


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::DataElementFilesystemURL::DataView,
                                         ::blink::mojom::DataElementFilesystemURLPtr> {
  static bool IsNull(const ::blink::mojom::DataElementFilesystemURLPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::DataElementFilesystemURLPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::DataElementFilesystemURL::url)& url(
      const ::blink::mojom::DataElementFilesystemURLPtr& input) {
    return input->url;
  }

  static decltype(::blink::mojom::DataElementFilesystemURL::offset) offset(
      const ::blink::mojom::DataElementFilesystemURLPtr& input) {
    return input->offset;
  }

  static decltype(::blink::mojom::DataElementFilesystemURL::length) length(
      const ::blink::mojom::DataElementFilesystemURLPtr& input) {
    return input->length;
  }

  static const decltype(::blink::mojom::DataElementFilesystemURL::expected_modification_time)& expected_modification_time(
      const ::blink::mojom::DataElementFilesystemURLPtr& input) {
    return input->expected_modification_time;
  }

  static bool Read(::blink::mojom::DataElementFilesystemURL::DataView input, ::blink::mojom::DataElementFilesystemURLPtr* output);
};


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::DataElementBlob::DataView,
                                         ::blink::mojom::DataElementBlobPtr> {
  static bool IsNull(const ::blink::mojom::DataElementBlobPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::DataElementBlobPtr* output) { output->reset(); }

  static  decltype(::blink::mojom::DataElementBlob::blob)& blob(
       ::blink::mojom::DataElementBlobPtr& input) {
    return input->blob;
  }

  static decltype(::blink::mojom::DataElementBlob::offset) offset(
      const ::blink::mojom::DataElementBlobPtr& input) {
    return input->offset;
  }

  static decltype(::blink::mojom::DataElementBlob::length) length(
      const ::blink::mojom::DataElementBlobPtr& input) {
    return input->length;
  }

  static bool Read(::blink::mojom::DataElementBlob::DataView input, ::blink::mojom::DataElementBlobPtr* output);
};


template <>
struct BLINK_COMMON_EXPORT UnionTraits<::blink::mojom::DataElement::DataView,
                                        ::blink::mojom::DataElementPtr> {
  static bool IsNull(const ::blink::mojom::DataElementPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::DataElementPtr* output) { output->reset(); }

  static ::blink::mojom::DataElement::Tag GetTag(const ::blink::mojom::DataElementPtr& input) {
    return input->which();
  }

  static  ::blink::mojom::DataElementBytesPtr& bytes( ::blink::mojom::DataElementPtr& input) {
    return input->get_bytes();
  }

  static const ::blink::mojom::DataElementFilePtr& file(const ::blink::mojom::DataElementPtr& input) {
    return input->get_file();
  }

  static const ::blink::mojom::DataElementFilesystemURLPtr& file_filesystem(const ::blink::mojom::DataElementPtr& input) {
    return input->get_file_filesystem();
  }

  static  ::blink::mojom::DataElementBlobPtr& blob( ::blink::mojom::DataElementPtr& input) {
    return input->get_blob();
  }

  static bool Read(::blink::mojom::DataElement::DataView input, ::blink::mojom::DataElementPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_REGISTRY_MOJOM_H_