// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_URL_STORE_MOJOM_BLINK_H_
#define THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_URL_STORE_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/common/blob/blob_url_store.mojom-shared.h"
#include "third_party/WebKit/common/blob/blob.mojom-blink.h"
#include "url/mojo/url.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/Source/platform/PlatformExport.h"


namespace blink {
namespace mojom {
namespace blink {
class BlobURLStore;
using BlobURLStorePtr = mojo::InterfacePtr<BlobURLStore>;
using BlobURLStorePtrInfo = mojo::InterfacePtrInfo<BlobURLStore>;
using ThreadSafeBlobURLStorePtr =
    mojo::ThreadSafeInterfacePtr<BlobURLStore>;
using BlobURLStoreRequest = mojo::InterfaceRequest<BlobURLStore>;
using BlobURLStoreAssociatedPtr =
    mojo::AssociatedInterfacePtr<BlobURLStore>;
using ThreadSafeBlobURLStoreAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BlobURLStore>;
using BlobURLStoreAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BlobURLStore>;
using BlobURLStoreAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BlobURLStore>;


class BlobURLStoreProxy;

template <typename ImplRefTraits>
class BlobURLStoreStub;

class BlobURLStoreRequestValidator;
class BlobURLStoreResponseValidator;

class PLATFORM_EXPORT BlobURLStore
    : public BlobURLStoreInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = BlobURLStoreProxy;

  template <typename ImplRefTraits>
  using Stub_ = BlobURLStoreStub<ImplRefTraits>;

  using RequestValidator_ = BlobURLStoreRequestValidator;
  using ResponseValidator_ = BlobURLStoreResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterMinVersion = 0,
    kRevokeMinVersion = 0,
    kResolveMinVersion = 0,
  };
  virtual ~BlobURLStore() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool Register(::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url);

  using RegisterCallback = base::OnceCallback<void()>;
  virtual void Register(::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url, RegisterCallback callback) = 0;

  virtual void Revoke(const ::blink::KURL& url) = 0;


  using ResolveCallback = base::OnceCallback<void(::blink::mojom::blink::BlobPtr)>;
  virtual void Resolve(const ::blink::KURL& url, ResolveCallback callback) = 0;
};
class PLATFORM_EXPORT BlobURLStoreInterceptorForTesting : public BlobURLStore {
  virtual BlobURLStore* GetForwardingInterface() = 0;
  void Register(::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url, RegisterCallback callback) override;
  void Revoke(const ::blink::KURL& url) override;
  void Resolve(const ::blink::KURL& url, ResolveCallback callback) override;
};
class PLATFORM_EXPORT BlobURLStoreAsyncWaiter {
 public:
  explicit BlobURLStoreAsyncWaiter(BlobURLStore* proxy);
  ~BlobURLStoreAsyncWaiter();
  void Register(
      ::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url);
  void Resolve(
      const ::blink::KURL& url, ::blink::mojom::blink::BlobPtr* out_blob);

 private:
  BlobURLStore* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(BlobURLStoreAsyncWaiter);
};

class PLATFORM_EXPORT BlobURLStoreProxy
    : public BlobURLStore {
 public:
  explicit BlobURLStoreProxy(mojo::MessageReceiverWithResponder* receiver);
  bool Register(::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url) final;
  void Register(::blink::mojom::blink::BlobPtr blob, const ::blink::KURL& url, RegisterCallback callback) final;
  void Revoke(const ::blink::KURL& url) final;
  void Resolve(const ::blink::KURL& url, ResolveCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class PLATFORM_EXPORT BlobURLStoreStubDispatch {
 public:
  static bool Accept(BlobURLStore* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BlobURLStore* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BlobURLStore>>
class BlobURLStoreStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BlobURLStoreStub() {}
  ~BlobURLStoreStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobURLStoreStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobURLStoreStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class PLATFORM_EXPORT BlobURLStoreRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class PLATFORM_EXPORT BlobURLStoreResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_COMMON_BLOB_BLOB_URL_STORE_MOJOM_BLINK_H_