// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_COMMON_MESSAGE_PORT_MESSAGE_PORT_MOJOM_H_
#define THIRD_PARTY_WEBKIT_COMMON_MESSAGE_PORT_MESSAGE_PORT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/common/message_port/message_port.mojom-shared.h"
#include "third_party/WebKit/common/blob/blob.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/common/message_port/cloneable_message.h"
#include "third_party/WebKit/common/message_port/message_port_channel.h"
#include "third_party/WebKit/common/message_port/transferable_message.h"
#include "third_party/WebKit/common/common_export.h"


namespace blink {
namespace mojom {
class CloneableMessage;
using CloneableMessagePtr = mojo::StructPtr<CloneableMessage>;

class TransferableMessage;
using TransferableMessagePtr = mojo::StructPtr<TransferableMessage>;

class SerializedBlob;
using SerializedBlobPtr = mojo::StructPtr<SerializedBlob>;








class BLINK_COMMON_EXPORT CloneableMessage {
 public:
  using DataView = CloneableMessageDataView;
  using Data_ = internal::CloneableMessage_Data;

  template <typename... Args>
  static CloneableMessagePtr New(Args&&... args) {
    return CloneableMessagePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CloneableMessagePtr From(const U& u) {
    return mojo::TypeConverter<CloneableMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CloneableMessage>::Convert(*this);
  }


  CloneableMessage();

  CloneableMessage(
      const std::vector<uint8_t>& encoded_message,
      std::vector<SerializedBlobPtr> blobs,
      uint64_t stack_trace_id,
      int64_t stack_trace_debugger_id_first,
      int64_t stack_trace_debugger_id_second);

  ~CloneableMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CloneableMessagePtr>
  CloneableMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CloneableMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CloneableMessage::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CloneableMessage::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CloneableMessage_UnserializedMessageContext<
            UserType, CloneableMessage::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CloneableMessage::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CloneableMessage::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CloneableMessage_UnserializedMessageContext<
            UserType, CloneableMessage::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CloneableMessage::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<uint8_t> encoded_message;
  std::vector<SerializedBlobPtr> blobs;
  uint64_t stack_trace_id;
  int64_t stack_trace_debugger_id_first;
  int64_t stack_trace_debugger_id_second;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CloneableMessage);
};


class BLINK_COMMON_EXPORT TransferableMessage {
 public:
  using DataView = TransferableMessageDataView;
  using Data_ = internal::TransferableMessage_Data;

  template <typename... Args>
  static TransferableMessagePtr New(Args&&... args) {
    return TransferableMessagePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static TransferableMessagePtr From(const U& u) {
    return mojo::TypeConverter<TransferableMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, TransferableMessage>::Convert(*this);
  }


  TransferableMessage();

  TransferableMessage(
      ::blink::CloneableMessage message,
      std::vector<mojo::ScopedMessagePipeHandle> ports);

  ~TransferableMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = TransferableMessagePtr>
  TransferableMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, TransferableMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        TransferableMessage::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        TransferableMessage::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::TransferableMessage_UnserializedMessageContext<
            UserType, TransferableMessage::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<TransferableMessage::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return TransferableMessage::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::TransferableMessage_UnserializedMessageContext<
            UserType, TransferableMessage::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<TransferableMessage::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::blink::CloneableMessage message;
  std::vector<mojo::ScopedMessagePipeHandle> ports;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(TransferableMessage);
};


class BLINK_COMMON_EXPORT SerializedBlob {
 public:
  using DataView = SerializedBlobDataView;
  using Data_ = internal::SerializedBlob_Data;

  template <typename... Args>
  static SerializedBlobPtr New(Args&&... args) {
    return SerializedBlobPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SerializedBlobPtr From(const U& u) {
    return mojo::TypeConverter<SerializedBlobPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SerializedBlob>::Convert(*this);
  }


  SerializedBlob();

  SerializedBlob(
      const std::string& uuid,
      const std::string& content_type,
      uint64_t size,
      ::blink::mojom::BlobPtrInfo blob);

  ~SerializedBlob();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SerializedBlobPtr>
  SerializedBlobPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SerializedBlob>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SerializedBlob::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SerializedBlob::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SerializedBlob_UnserializedMessageContext<
            UserType, SerializedBlob::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SerializedBlob::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SerializedBlob::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SerializedBlob_UnserializedMessageContext<
            UserType, SerializedBlob::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SerializedBlob::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::string uuid;
  std::string content_type;
  uint64_t size;
  ::blink::mojom::BlobPtrInfo blob;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(SerializedBlob);
};

template <typename StructPtrType>
CloneableMessagePtr CloneableMessage::Clone() const {
  return New(
      mojo::Clone(encoded_message),
      mojo::Clone(blobs),
      mojo::Clone(stack_trace_id),
      mojo::Clone(stack_trace_debugger_id_first),
      mojo::Clone(stack_trace_debugger_id_second)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CloneableMessage>::value>::type*>
bool CloneableMessage::Equals(const T& other) const {
  if (!mojo::Equals(this->encoded_message, other.encoded_message))
    return false;
  if (!mojo::Equals(this->blobs, other.blobs))
    return false;
  if (!mojo::Equals(this->stack_trace_id, other.stack_trace_id))
    return false;
  if (!mojo::Equals(this->stack_trace_debugger_id_first, other.stack_trace_debugger_id_first))
    return false;
  if (!mojo::Equals(this->stack_trace_debugger_id_second, other.stack_trace_debugger_id_second))
    return false;
  return true;
}
template <typename StructPtrType>
TransferableMessagePtr TransferableMessage::Clone() const {
  return New(
      mojo::Clone(message),
      mojo::Clone(ports)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, TransferableMessage>::value>::type*>
bool TransferableMessage::Equals(const T& other) const {
  if (!mojo::Equals(this->message, other.message))
    return false;
  if (!mojo::Equals(this->ports, other.ports))
    return false;
  return true;
}
template <typename StructPtrType>
SerializedBlobPtr SerializedBlob::Clone() const {
  return New(
      mojo::Clone(uuid),
      mojo::Clone(content_type),
      mojo::Clone(size),
      mojo::Clone(blob)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SerializedBlob>::value>::type*>
bool SerializedBlob::Equals(const T& other) const {
  if (!mojo::Equals(this->uuid, other.uuid))
    return false;
  if (!mojo::Equals(this->content_type, other.content_type))
    return false;
  if (!mojo::Equals(this->size, other.size))
    return false;
  if (!mojo::Equals(this->blob, other.blob))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::CloneableMessage::DataView,
                                         ::blink::mojom::CloneableMessagePtr> {
  static bool IsNull(const ::blink::mojom::CloneableMessagePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::CloneableMessagePtr* output) { output->reset(); }

  static const decltype(::blink::mojom::CloneableMessage::encoded_message)& encoded_message(
      const ::blink::mojom::CloneableMessagePtr& input) {
    return input->encoded_message;
  }

  static  decltype(::blink::mojom::CloneableMessage::blobs)& blobs(
       ::blink::mojom::CloneableMessagePtr& input) {
    return input->blobs;
  }

  static decltype(::blink::mojom::CloneableMessage::stack_trace_id) stack_trace_id(
      const ::blink::mojom::CloneableMessagePtr& input) {
    return input->stack_trace_id;
  }

  static decltype(::blink::mojom::CloneableMessage::stack_trace_debugger_id_first) stack_trace_debugger_id_first(
      const ::blink::mojom::CloneableMessagePtr& input) {
    return input->stack_trace_debugger_id_first;
  }

  static decltype(::blink::mojom::CloneableMessage::stack_trace_debugger_id_second) stack_trace_debugger_id_second(
      const ::blink::mojom::CloneableMessagePtr& input) {
    return input->stack_trace_debugger_id_second;
  }

  static bool Read(::blink::mojom::CloneableMessage::DataView input, ::blink::mojom::CloneableMessagePtr* output);
};


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::TransferableMessage::DataView,
                                         ::blink::mojom::TransferableMessagePtr> {
  static bool IsNull(const ::blink::mojom::TransferableMessagePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::TransferableMessagePtr* output) { output->reset(); }

  static  decltype(::blink::mojom::TransferableMessage::message)& message(
       ::blink::mojom::TransferableMessagePtr& input) {
    return input->message;
  }

  static  decltype(::blink::mojom::TransferableMessage::ports)& ports(
       ::blink::mojom::TransferableMessagePtr& input) {
    return input->ports;
  }

  static bool Read(::blink::mojom::TransferableMessage::DataView input, ::blink::mojom::TransferableMessagePtr* output);
};


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::SerializedBlob::DataView,
                                         ::blink::mojom::SerializedBlobPtr> {
  static bool IsNull(const ::blink::mojom::SerializedBlobPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::SerializedBlobPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::SerializedBlob::uuid)& uuid(
      const ::blink::mojom::SerializedBlobPtr& input) {
    return input->uuid;
  }

  static const decltype(::blink::mojom::SerializedBlob::content_type)& content_type(
      const ::blink::mojom::SerializedBlobPtr& input) {
    return input->content_type;
  }

  static decltype(::blink::mojom::SerializedBlob::size) size(
      const ::blink::mojom::SerializedBlobPtr& input) {
    return input->size;
  }

  static  decltype(::blink::mojom::SerializedBlob::blob)& blob(
       ::blink::mojom::SerializedBlobPtr& input) {
    return input->blob;
  }

  static bool Read(::blink::mojom::SerializedBlob::DataView input, ::blink::mojom::SerializedBlobPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_COMMON_MESSAGE_PORT_MESSAGE_PORT_MOJOM_H_