// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_COMMON_SERVICE_WORKER_SERVICE_WORKER_OBJECT_MOJOM_H_
#define THIRD_PARTY_WEBKIT_COMMON_SERVICE_WORKER_SERVICE_WORKER_OBJECT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/common/service_worker/service_worker_object.mojom-shared.h"
#include "third_party/WebKit/common/service_worker/service_worker_state.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/common/common_export.h"


namespace blink {
namespace mojom {
constexpr int32_t kInvalidServiceWorkerHandleId = -1;
constexpr int64_t kInvalidServiceWorkerVersionId = -1;
class ServiceWorkerObjectInfo;
using ServiceWorkerObjectInfoPtr = mojo::StructPtr<ServiceWorkerObjectInfo>;






class BLINK_COMMON_EXPORT ServiceWorkerObjectInfo {
 public:
  using DataView = ServiceWorkerObjectInfoDataView;
  using Data_ = internal::ServiceWorkerObjectInfo_Data;

  template <typename... Args>
  static ServiceWorkerObjectInfoPtr New(Args&&... args) {
    return ServiceWorkerObjectInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ServiceWorkerObjectInfoPtr From(const U& u) {
    return mojo::TypeConverter<ServiceWorkerObjectInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ServiceWorkerObjectInfo>::Convert(*this);
  }


  ServiceWorkerObjectInfo();

  ServiceWorkerObjectInfo(
      int32_t handle_id,
      int64_t version_id,
      ::blink::mojom::ServiceWorkerState state,
      const GURL& url);

  ~ServiceWorkerObjectInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ServiceWorkerObjectInfoPtr>
  ServiceWorkerObjectInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ServiceWorkerObjectInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ServiceWorkerObjectInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ServiceWorkerObjectInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ServiceWorkerObjectInfo_UnserializedMessageContext<
            UserType, ServiceWorkerObjectInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ServiceWorkerObjectInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ServiceWorkerObjectInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ServiceWorkerObjectInfo_UnserializedMessageContext<
            UserType, ServiceWorkerObjectInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ServiceWorkerObjectInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  int32_t handle_id;
  int64_t version_id;
  ::blink::mojom::ServiceWorkerState state;
  GURL url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
ServiceWorkerObjectInfoPtr ServiceWorkerObjectInfo::Clone() const {
  return New(
      mojo::Clone(handle_id),
      mojo::Clone(version_id),
      mojo::Clone(state),
      mojo::Clone(url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ServiceWorkerObjectInfo>::value>::type*>
bool ServiceWorkerObjectInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->handle_id, other.handle_id))
    return false;
  if (!mojo::Equals(this->version_id, other.version_id))
    return false;
  if (!mojo::Equals(this->state, other.state))
    return false;
  if (!mojo::Equals(this->url, other.url))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct BLINK_COMMON_EXPORT StructTraits<::blink::mojom::ServiceWorkerObjectInfo::DataView,
                                         ::blink::mojom::ServiceWorkerObjectInfoPtr> {
  static bool IsNull(const ::blink::mojom::ServiceWorkerObjectInfoPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::ServiceWorkerObjectInfoPtr* output) { output->reset(); }

  static decltype(::blink::mojom::ServiceWorkerObjectInfo::handle_id) handle_id(
      const ::blink::mojom::ServiceWorkerObjectInfoPtr& input) {
    return input->handle_id;
  }

  static decltype(::blink::mojom::ServiceWorkerObjectInfo::version_id) version_id(
      const ::blink::mojom::ServiceWorkerObjectInfoPtr& input) {
    return input->version_id;
  }

  static decltype(::blink::mojom::ServiceWorkerObjectInfo::state) state(
      const ::blink::mojom::ServiceWorkerObjectInfoPtr& input) {
    return input->state;
  }

  static const decltype(::blink::mojom::ServiceWorkerObjectInfo::url)& url(
      const ::blink::mojom::ServiceWorkerObjectInfoPtr& input) {
    return input->url;
  }

  static bool Read(::blink::mojom::ServiceWorkerObjectInfo::DataView input, ::blink::mojom::ServiceWorkerObjectInfoPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_COMMON_SERVICE_WORKER_SERVICE_WORKER_OBJECT_MOJOM_H_