// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MEDIA_ENGAGEMENT_MOJOM_BLINK_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MEDIA_ENGAGEMENT_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/media_engagement.mojom-shared.h"
#include "url/mojo/origin.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace blink {
namespace mojom {
namespace blink {
class MediaEngagementClient;
using MediaEngagementClientPtr = mojo::InterfacePtr<MediaEngagementClient>;
using MediaEngagementClientPtrInfo = mojo::InterfacePtrInfo<MediaEngagementClient>;
using ThreadSafeMediaEngagementClientPtr =
    mojo::ThreadSafeInterfacePtr<MediaEngagementClient>;
using MediaEngagementClientRequest = mojo::InterfaceRequest<MediaEngagementClient>;
using MediaEngagementClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<MediaEngagementClient>;
using ThreadSafeMediaEngagementClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MediaEngagementClient>;
using MediaEngagementClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MediaEngagementClient>;
using MediaEngagementClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MediaEngagementClient>;


class MediaEngagementClientProxy;

template <typename ImplRefTraits>
class MediaEngagementClientStub;

class MediaEngagementClientRequestValidator;

class BLINK_PLATFORM_EXPORT MediaEngagementClient
    : public MediaEngagementClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MediaEngagementClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaEngagementClientStub<ImplRefTraits>;

  using RequestValidator_ = MediaEngagementClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetHasHighMediaEngagementMinVersion = 0,
  };
  virtual ~MediaEngagementClient() {}

  virtual void SetHasHighMediaEngagement(const scoped_refptr<const ::blink::SecurityOrigin>& origin) = 0;
};
class BLINK_PLATFORM_EXPORT MediaEngagementClientInterceptorForTesting : public MediaEngagementClient {
  virtual MediaEngagementClient* GetForwardingInterface() = 0;
  void SetHasHighMediaEngagement(const scoped_refptr<const ::blink::SecurityOrigin>& origin) override;
};
class BLINK_PLATFORM_EXPORT MediaEngagementClientAsyncWaiter {
 public:
  explicit MediaEngagementClientAsyncWaiter(MediaEngagementClient* proxy);
  ~MediaEngagementClientAsyncWaiter();

 private:
  MediaEngagementClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(MediaEngagementClientAsyncWaiter);
};

class BLINK_PLATFORM_EXPORT MediaEngagementClientProxy
    : public MediaEngagementClient {
 public:
  explicit MediaEngagementClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetHasHighMediaEngagement(const scoped_refptr<const ::blink::SecurityOrigin>& origin) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT MediaEngagementClientStubDispatch {
 public:
  static bool Accept(MediaEngagementClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaEngagementClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaEngagementClient>>
class MediaEngagementClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaEngagementClientStub() {}
  ~MediaEngagementClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaEngagementClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaEngagementClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT MediaEngagementClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MEDIA_ENGAGEMENT_MOJOM_BLINK_H_