// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_BLINK_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/keyboard_lock/keyboard_lock.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace WTF {
struct blink_mojom_internal_KeyboardLockRequestResult_DataHashFn {
  static unsigned GetHash(const ::blink::mojom::KeyboardLockRequestResult& value) {
    using utype = std::underlying_type<::blink::mojom::KeyboardLockRequestResult>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::blink::mojom::KeyboardLockRequestResult& left, const ::blink::mojom::KeyboardLockRequestResult& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::blink::mojom::KeyboardLockRequestResult> {
  using Hash = blink_mojom_internal_KeyboardLockRequestResult_DataHashFn;
};

template <>
struct HashTraits<::blink::mojom::KeyboardLockRequestResult>
    : public GenericHashTraits<::blink::mojom::KeyboardLockRequestResult> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::blink::mojom::KeyboardLockRequestResult& value) {
    return value == static_cast<::blink::mojom::KeyboardLockRequestResult>(-1000000);
  }
  static void ConstructDeletedValue(::blink::mojom::KeyboardLockRequestResult& slot, bool) {
    slot = static_cast<::blink::mojom::KeyboardLockRequestResult>(-1000001);
  }
  static bool IsDeletedValue(const ::blink::mojom::KeyboardLockRequestResult& value) {
    return value == static_cast<::blink::mojom::KeyboardLockRequestResult>(-1000001);
  }
};
}  // namespace WTF


namespace blink {
namespace mojom {
namespace blink {
using KeyboardLockRequestResult = KeyboardLockRequestResult;  // Alias for definition in the parent namespace.
class KeyboardLockService;
using KeyboardLockServicePtr = mojo::InterfacePtr<KeyboardLockService>;
using KeyboardLockServicePtrInfo = mojo::InterfacePtrInfo<KeyboardLockService>;
using ThreadSafeKeyboardLockServicePtr =
    mojo::ThreadSafeInterfacePtr<KeyboardLockService>;
using KeyboardLockServiceRequest = mojo::InterfaceRequest<KeyboardLockService>;
using KeyboardLockServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<KeyboardLockService>;
using ThreadSafeKeyboardLockServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<KeyboardLockService>;
using KeyboardLockServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<KeyboardLockService>;
using KeyboardLockServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<KeyboardLockService>;


class KeyboardLockServiceProxy;

template <typename ImplRefTraits>
class KeyboardLockServiceStub;

class KeyboardLockServiceRequestValidator;
class KeyboardLockServiceResponseValidator;

class BLINK_PLATFORM_EXPORT KeyboardLockService
    : public KeyboardLockServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = KeyboardLockServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = KeyboardLockServiceStub<ImplRefTraits>;

  using RequestValidator_ = KeyboardLockServiceRequestValidator;
  using ResponseValidator_ = KeyboardLockServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestKeyboardLockMinVersion = 0,
    kCancelKeyboardLockMinVersion = 0,
  };
  virtual ~KeyboardLockService() {}


  using RequestKeyboardLockCallback = base::OnceCallback<void(KeyboardLockRequestResult)>;
  virtual void RequestKeyboardLock(const WTF::Vector<WTF::String>& key_codes, RequestKeyboardLockCallback callback) = 0;

  virtual void CancelKeyboardLock() = 0;
};
class BLINK_PLATFORM_EXPORT KeyboardLockServiceInterceptorForTesting : public KeyboardLockService {
  virtual KeyboardLockService* GetForwardingInterface() = 0;
  void RequestKeyboardLock(const WTF::Vector<WTF::String>& key_codes, RequestKeyboardLockCallback callback) override;
  void CancelKeyboardLock() override;
};
class BLINK_PLATFORM_EXPORT KeyboardLockServiceAsyncWaiter {
 public:
  explicit KeyboardLockServiceAsyncWaiter(KeyboardLockService* proxy);
  ~KeyboardLockServiceAsyncWaiter();
  void RequestKeyboardLock(
      const WTF::Vector<WTF::String>& key_codes, KeyboardLockRequestResult* out_result);

 private:
  KeyboardLockService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(KeyboardLockServiceAsyncWaiter);
};

class BLINK_PLATFORM_EXPORT KeyboardLockServiceProxy
    : public KeyboardLockService {
 public:
  explicit KeyboardLockServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestKeyboardLock(const WTF::Vector<WTF::String>& key_codes, RequestKeyboardLockCallback callback) final;
  void CancelKeyboardLock() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT KeyboardLockServiceStubDispatch {
 public:
  static bool Accept(KeyboardLockService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      KeyboardLockService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<KeyboardLockService>>
class KeyboardLockServiceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  KeyboardLockServiceStub() {}
  ~KeyboardLockServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return KeyboardLockServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return KeyboardLockServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT KeyboardLockServiceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT KeyboardLockServiceResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_BLINK_H_