// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBSOCKETS_WEBSOCKET_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBSOCKETS_WEBSOCKET_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/websockets/websocket.mojom-shared.h"
#include "url/mojo/origin.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class WebSocketClient;
using WebSocketClientPtr = mojo::InterfacePtr<WebSocketClient>;
using WebSocketClientPtrInfo = mojo::InterfacePtrInfo<WebSocketClient>;
using ThreadSafeWebSocketClientPtr =
    mojo::ThreadSafeInterfacePtr<WebSocketClient>;
using WebSocketClientRequest = mojo::InterfaceRequest<WebSocketClient>;
using WebSocketClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<WebSocketClient>;
using ThreadSafeWebSocketClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<WebSocketClient>;
using WebSocketClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<WebSocketClient>;
using WebSocketClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<WebSocketClient>;

class WebSocket;
using WebSocketPtr = mojo::InterfacePtr<WebSocket>;
using WebSocketPtrInfo = mojo::InterfacePtrInfo<WebSocket>;
using ThreadSafeWebSocketPtr =
    mojo::ThreadSafeInterfacePtr<WebSocket>;
using WebSocketRequest = mojo::InterfaceRequest<WebSocket>;
using WebSocketAssociatedPtr =
    mojo::AssociatedInterfacePtr<WebSocket>;
using ThreadSafeWebSocketAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<WebSocket>;
using WebSocketAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<WebSocket>;
using WebSocketAssociatedRequest =
    mojo::AssociatedInterfaceRequest<WebSocket>;

class HttpHeader;
using HttpHeaderPtr = mojo::InlinedStructPtr<HttpHeader>;

class WebSocketHandshakeRequest;
using WebSocketHandshakeRequestPtr = mojo::StructPtr<WebSocketHandshakeRequest>;

class WebSocketHandshakeResponse;
using WebSocketHandshakeResponsePtr = mojo::StructPtr<WebSocketHandshakeResponse>;


class WebSocketClientProxy;

template <typename ImplRefTraits>
class WebSocketClientStub;

class WebSocketClientRequestValidator;

class CONTENT_EXPORT WebSocketClient
    : public WebSocketClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = WebSocketClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = WebSocketClientStub<ImplRefTraits>;

  using RequestValidator_ = WebSocketClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnFailChannelMinVersion = 0,
    kOnStartOpeningHandshakeMinVersion = 0,
    kOnFinishOpeningHandshakeMinVersion = 0,
    kOnAddChannelResponseMinVersion = 0,
    kOnDataFrameMinVersion = 0,
    kOnFlowControlMinVersion = 0,
    kOnDropChannelMinVersion = 0,
    kOnClosingHandshakeMinVersion = 0,
  };
  virtual ~WebSocketClient() {}

  virtual void OnFailChannel(const std::string& reason) = 0;

  virtual void OnStartOpeningHandshake(WebSocketHandshakeRequestPtr request) = 0;

  virtual void OnFinishOpeningHandshake(WebSocketHandshakeResponsePtr response) = 0;

  virtual void OnAddChannelResponse(const std::string& selected_protocol, const std::string& extensions) = 0;

  virtual void OnDataFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) = 0;

  virtual void OnFlowControl(int64_t quota) = 0;

  virtual void OnDropChannel(bool was_clean, uint16_t code, const std::string& reason) = 0;

  virtual void OnClosingHandshake() = 0;
};
class CONTENT_EXPORT WebSocketClientInterceptorForTesting : public WebSocketClient {
  virtual WebSocketClient* GetForwardingInterface() = 0;
  void OnFailChannel(const std::string& reason) override;
  void OnStartOpeningHandshake(WebSocketHandshakeRequestPtr request) override;
  void OnFinishOpeningHandshake(WebSocketHandshakeResponsePtr response) override;
  void OnAddChannelResponse(const std::string& selected_protocol, const std::string& extensions) override;
  void OnDataFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) override;
  void OnFlowControl(int64_t quota) override;
  void OnDropChannel(bool was_clean, uint16_t code, const std::string& reason) override;
  void OnClosingHandshake() override;
};
class CONTENT_EXPORT WebSocketClientAsyncWaiter {
 public:
  explicit WebSocketClientAsyncWaiter(WebSocketClient* proxy);
  ~WebSocketClientAsyncWaiter();

 private:
  WebSocketClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(WebSocketClientAsyncWaiter);
};

class WebSocketProxy;

template <typename ImplRefTraits>
class WebSocketStub;

class WebSocketRequestValidator;

class CONTENT_EXPORT WebSocket
    : public WebSocketInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = WebSocketProxy;

  template <typename ImplRefTraits>
  using Stub_ = WebSocketStub<ImplRefTraits>;

  using RequestValidator_ = WebSocketRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddChannelRequestMinVersion = 0,
    kSendFrameMinVersion = 0,
    kSendFlowControlMinVersion = 0,
    kStartClosingHandshakeMinVersion = 0,
  };
  static constexpr uint32_t kInsufficientResources = 1U;
  virtual ~WebSocket() {}

  virtual void AddChannelRequest(const GURL& url, const std::vector<std::string>& requested_protocols, const url::Origin& origin, const GURL& first_party_for_cookies, const std::string& user_agent_override, WebSocketClientPtr client) = 0;

  virtual void SendFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) = 0;

  virtual void SendFlowControl(int64_t quota) = 0;

  virtual void StartClosingHandshake(uint16_t code, const std::string& reason) = 0;
};
class CONTENT_EXPORT WebSocketInterceptorForTesting : public WebSocket {
  virtual WebSocket* GetForwardingInterface() = 0;
  void AddChannelRequest(const GURL& url, const std::vector<std::string>& requested_protocols, const url::Origin& origin, const GURL& first_party_for_cookies, const std::string& user_agent_override, WebSocketClientPtr client) override;
  void SendFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) override;
  void SendFlowControl(int64_t quota) override;
  void StartClosingHandshake(uint16_t code, const std::string& reason) override;
};
class CONTENT_EXPORT WebSocketAsyncWaiter {
 public:
  explicit WebSocketAsyncWaiter(WebSocket* proxy);
  ~WebSocketAsyncWaiter();

 private:
  WebSocket* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(WebSocketAsyncWaiter);
};

class CONTENT_EXPORT WebSocketClientProxy
    : public WebSocketClient {
 public:
  explicit WebSocketClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnFailChannel(const std::string& reason) final;
  void OnStartOpeningHandshake(WebSocketHandshakeRequestPtr request) final;
  void OnFinishOpeningHandshake(WebSocketHandshakeResponsePtr response) final;
  void OnAddChannelResponse(const std::string& selected_protocol, const std::string& extensions) final;
  void OnDataFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) final;
  void OnFlowControl(int64_t quota) final;
  void OnDropChannel(bool was_clean, uint16_t code, const std::string& reason) final;
  void OnClosingHandshake() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT WebSocketProxy
    : public WebSocket {
 public:
  explicit WebSocketProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddChannelRequest(const GURL& url, const std::vector<std::string>& requested_protocols, const url::Origin& origin, const GURL& first_party_for_cookies, const std::string& user_agent_override, WebSocketClientPtr client) final;
  void SendFrame(bool fin, WebSocketMessageType type, const std::vector<uint8_t>& data) final;
  void SendFlowControl(int64_t quota) final;
  void StartClosingHandshake(uint16_t code, const std::string& reason) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT WebSocketClientStubDispatch {
 public:
  static bool Accept(WebSocketClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WebSocketClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WebSocketClient>>
class WebSocketClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WebSocketClientStub() {}
  ~WebSocketClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebSocketClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebSocketClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT WebSocketStubDispatch {
 public:
  static bool Accept(WebSocket* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WebSocket* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WebSocket>>
class WebSocketStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WebSocketStub() {}
  ~WebSocketStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebSocketStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebSocketStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT WebSocketClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT WebSocketRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};


class CONTENT_EXPORT HttpHeader {
 public:
  using DataView = HttpHeaderDataView;
  using Data_ = internal::HttpHeader_Data;

  template <typename... Args>
  static HttpHeaderPtr New(Args&&... args) {
    return HttpHeaderPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static HttpHeaderPtr From(const U& u) {
    return mojo::TypeConverter<HttpHeaderPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HttpHeader>::Convert(*this);
  }


  HttpHeader();

  HttpHeader(
      const std::string& name,
      const std::string& value);

  ~HttpHeader();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = HttpHeaderPtr>
  HttpHeaderPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, HttpHeader>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        HttpHeader::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        HttpHeader::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::HttpHeader_UnserializedMessageContext<
            UserType, HttpHeader::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<HttpHeader::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return HttpHeader::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::HttpHeader_UnserializedMessageContext<
            UserType, HttpHeader::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<HttpHeader::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::string name;
  std::string value;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class CONTENT_EXPORT WebSocketHandshakeRequest {
 public:
  using DataView = WebSocketHandshakeRequestDataView;
  using Data_ = internal::WebSocketHandshakeRequest_Data;

  template <typename... Args>
  static WebSocketHandshakeRequestPtr New(Args&&... args) {
    return WebSocketHandshakeRequestPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static WebSocketHandshakeRequestPtr From(const U& u) {
    return mojo::TypeConverter<WebSocketHandshakeRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebSocketHandshakeRequest>::Convert(*this);
  }


  WebSocketHandshakeRequest();

  WebSocketHandshakeRequest(
      const GURL& url,
      std::vector<HttpHeaderPtr> headers,
      const std::string& headers_text);

  ~WebSocketHandshakeRequest();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebSocketHandshakeRequestPtr>
  WebSocketHandshakeRequestPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebSocketHandshakeRequest>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebSocketHandshakeRequest::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebSocketHandshakeRequest::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebSocketHandshakeRequest_UnserializedMessageContext<
            UserType, WebSocketHandshakeRequest::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebSocketHandshakeRequest::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebSocketHandshakeRequest::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebSocketHandshakeRequest_UnserializedMessageContext<
            UserType, WebSocketHandshakeRequest::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebSocketHandshakeRequest::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  GURL url;
  std::vector<HttpHeaderPtr> headers;
  std::string headers_text;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(WebSocketHandshakeRequest);
};


class CONTENT_EXPORT WebSocketHandshakeResponse {
 public:
  using DataView = WebSocketHandshakeResponseDataView;
  using Data_ = internal::WebSocketHandshakeResponse_Data;

  template <typename... Args>
  static WebSocketHandshakeResponsePtr New(Args&&... args) {
    return WebSocketHandshakeResponsePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static WebSocketHandshakeResponsePtr From(const U& u) {
    return mojo::TypeConverter<WebSocketHandshakeResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebSocketHandshakeResponse>::Convert(*this);
  }


  WebSocketHandshakeResponse();

  WebSocketHandshakeResponse(
      const GURL& url,
      int32_t status_code,
      const std::string& status_text,
      std::vector<HttpHeaderPtr> headers,
      const std::string& headers_text);

  ~WebSocketHandshakeResponse();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebSocketHandshakeResponsePtr>
  WebSocketHandshakeResponsePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebSocketHandshakeResponse>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebSocketHandshakeResponse::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebSocketHandshakeResponse::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebSocketHandshakeResponse_UnserializedMessageContext<
            UserType, WebSocketHandshakeResponse::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebSocketHandshakeResponse::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebSocketHandshakeResponse::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebSocketHandshakeResponse_UnserializedMessageContext<
            UserType, WebSocketHandshakeResponse::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebSocketHandshakeResponse::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  GURL url;
  int32_t status_code;
  std::string status_text;
  std::vector<HttpHeaderPtr> headers;
  std::string headers_text;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(WebSocketHandshakeResponse);
};

template <typename StructPtrType>
HttpHeaderPtr HttpHeader::Clone() const {
  return New(
      mojo::Clone(name),
      mojo::Clone(value)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, HttpHeader>::value>::type*>
bool HttpHeader::Equals(const T& other) const {
  if (!mojo::Equals(this->name, other.name))
    return false;
  if (!mojo::Equals(this->value, other.value))
    return false;
  return true;
}
template <typename StructPtrType>
WebSocketHandshakeRequestPtr WebSocketHandshakeRequest::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(headers),
      mojo::Clone(headers_text)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebSocketHandshakeRequest>::value>::type*>
bool WebSocketHandshakeRequest::Equals(const T& other) const {
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->headers, other.headers))
    return false;
  if (!mojo::Equals(this->headers_text, other.headers_text))
    return false;
  return true;
}
template <typename StructPtrType>
WebSocketHandshakeResponsePtr WebSocketHandshakeResponse::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(status_code),
      mojo::Clone(status_text),
      mojo::Clone(headers),
      mojo::Clone(headers_text)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebSocketHandshakeResponse>::value>::type*>
bool WebSocketHandshakeResponse::Equals(const T& other) const {
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->status_code, other.status_code))
    return false;
  if (!mojo::Equals(this->status_text, other.status_text))
    return false;
  if (!mojo::Equals(this->headers, other.headers))
    return false;
  if (!mojo::Equals(this->headers_text, other.headers_text))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::HttpHeader::DataView,
                                         ::blink::mojom::HttpHeaderPtr> {
  static bool IsNull(const ::blink::mojom::HttpHeaderPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::HttpHeaderPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::HttpHeader::name)& name(
      const ::blink::mojom::HttpHeaderPtr& input) {
    return input->name;
  }

  static const decltype(::blink::mojom::HttpHeader::value)& value(
      const ::blink::mojom::HttpHeaderPtr& input) {
    return input->value;
  }

  static bool Read(::blink::mojom::HttpHeader::DataView input, ::blink::mojom::HttpHeaderPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::WebSocketHandshakeRequest::DataView,
                                         ::blink::mojom::WebSocketHandshakeRequestPtr> {
  static bool IsNull(const ::blink::mojom::WebSocketHandshakeRequestPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::WebSocketHandshakeRequestPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::WebSocketHandshakeRequest::url)& url(
      const ::blink::mojom::WebSocketHandshakeRequestPtr& input) {
    return input->url;
  }

  static const decltype(::blink::mojom::WebSocketHandshakeRequest::headers)& headers(
      const ::blink::mojom::WebSocketHandshakeRequestPtr& input) {
    return input->headers;
  }

  static const decltype(::blink::mojom::WebSocketHandshakeRequest::headers_text)& headers_text(
      const ::blink::mojom::WebSocketHandshakeRequestPtr& input) {
    return input->headers_text;
  }

  static bool Read(::blink::mojom::WebSocketHandshakeRequest::DataView input, ::blink::mojom::WebSocketHandshakeRequestPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::WebSocketHandshakeResponse::DataView,
                                         ::blink::mojom::WebSocketHandshakeResponsePtr> {
  static bool IsNull(const ::blink::mojom::WebSocketHandshakeResponsePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::WebSocketHandshakeResponsePtr* output) { output->reset(); }

  static const decltype(::blink::mojom::WebSocketHandshakeResponse::url)& url(
      const ::blink::mojom::WebSocketHandshakeResponsePtr& input) {
    return input->url;
  }

  static decltype(::blink::mojom::WebSocketHandshakeResponse::status_code) status_code(
      const ::blink::mojom::WebSocketHandshakeResponsePtr& input) {
    return input->status_code;
  }

  static const decltype(::blink::mojom::WebSocketHandshakeResponse::status_text)& status_text(
      const ::blink::mojom::WebSocketHandshakeResponsePtr& input) {
    return input->status_text;
  }

  static const decltype(::blink::mojom::WebSocketHandshakeResponse::headers)& headers(
      const ::blink::mojom::WebSocketHandshakeResponsePtr& input) {
    return input->headers;
  }

  static const decltype(::blink::mojom::WebSocketHandshakeResponse::headers_text)& headers_text(
      const ::blink::mojom::WebSocketHandshakeResponsePtr& input) {
    return input->headers_text;
  }

  static bool Read(::blink::mojom::WebSocketHandshakeResponse::DataView input, ::blink::mojom::WebSocketHandshakeResponsePtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBSOCKETS_WEBSOCKET_MOJOM_H_