// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/web/devtools_agent.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "third_party/WebKit/Source/platform/mojo/GeometryStructTraits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace blink {
namespace mojom {
namespace blink {
DevToolsMessageChunk::DevToolsMessageChunk()
    : is_first(),
      is_last(),
      data(),
      call_id(),
      post_state() {}

DevToolsMessageChunk::DevToolsMessageChunk(
    bool is_first_in,
    bool is_last_in,
    const WTF::String& data_in,
    int32_t call_id_in,
    const WTF::String& post_state_in)
    : is_first(std::move(is_first_in)),
      is_last(std::move(is_last_in)),
      data(std::move(data_in)),
      call_id(std::move(call_id_in)),
      post_state(std::move(post_state_in)) {}

DevToolsMessageChunk::~DevToolsMessageChunk() = default;
size_t DevToolsMessageChunk::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->is_first);
  seed = mojo::internal::WTFHash(seed, this->is_last);
  seed = mojo::internal::WTFHash(seed, this->data);
  seed = mojo::internal::WTFHash(seed, this->call_id);
  seed = mojo::internal::WTFHash(seed, this->post_state);
  return seed;
}

bool DevToolsMessageChunk::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char DevToolsAgent::Name_[] = "blink::mojom::DevToolsAgent";

DevToolsAgentProxy::DevToolsAgentProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void DevToolsAgentProxy::AttachDevToolsSession(
    DevToolsSessionHostAssociatedPtrInfo in_host, DevToolsSessionAssociatedRequest in_session, DevToolsSessionRequest in_io_session, const WTF::String& in_reattach_state) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "blink::mojom::DevToolsAgent::AttachDevToolsSession");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kDevToolsAgent_AttachDevToolsSession_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::blink::mojom::internal::DevToolsAgent_AttachDevToolsSession_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::blink::mojom::DevToolsSessionHostAssociatedPtrInfoDataView>(
      in_host, &params->host, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->host),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
      "invalid host in DevToolsAgent.AttachDevToolsSession request");
  mojo::internal::Serialize<::blink::mojom::DevToolsSessionAssociatedRequestDataView>(
      in_session, &params->session, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->session),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
      "invalid session in DevToolsAgent.AttachDevToolsSession request");
  mojo::internal::Serialize<::blink::mojom::DevToolsSessionRequestDataView>(
      in_io_session, &params->io_session, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->io_session),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid io_session in DevToolsAgent.AttachDevToolsSession request");
  typename decltype(params->reattach_state)::BaseType::BufferWriter
      reattach_state_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_reattach_state, buffer, &reattach_state_writer, &serialization_context);
  params->reattach_state.Set(
      reattach_state_writer.is_null() ? nullptr : reattach_state_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool DevToolsAgentStubDispatch::Accept(
    DevToolsAgent* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDevToolsAgent_AttachDevToolsSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsAgent::AttachDevToolsSession",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::DevToolsAgent_AttachDevToolsSession_Params_Data* params =
          reinterpret_cast<internal::DevToolsAgent_AttachDevToolsSession_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DevToolsSessionHostAssociatedPtrInfo p_host{};
      DevToolsSessionAssociatedRequest p_session{};
      DevToolsSessionRequest p_io_session{};
      WTF::String p_reattach_state{};
      DevToolsAgent_AttachDevToolsSession_ParamsDataView input_data_view(params, &serialization_context);
      
      p_host =
          input_data_view.TakeHost<decltype(p_host)>();
      p_session =
          input_data_view.TakeSession<decltype(p_session)>();
      p_io_session =
          input_data_view.TakeIoSession<decltype(p_io_session)>();
      if (!input_data_view.ReadReattachState(&p_reattach_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DevToolsAgent::AttachDevToolsSession deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AttachDevToolsSession(
std::move(p_host), 
std::move(p_session), 
std::move(p_io_session), 
std::move(p_reattach_state));
      return true;
    }
  }
  return false;
}

// static
bool DevToolsAgentStubDispatch::AcceptWithResponder(
    DevToolsAgent* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDevToolsAgent_AttachDevToolsSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsAgent::AttachDevToolsSession",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool DevToolsAgentRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DevToolsAgent RequestValidator");

  switch (message->header()->name) {
    case internal::kDevToolsAgent_AttachDevToolsSession_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DevToolsAgent_AttachDevToolsSession_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void DevToolsAgentInterceptorForTesting::AttachDevToolsSession(DevToolsSessionHostAssociatedPtrInfo host, DevToolsSessionAssociatedRequest session, DevToolsSessionRequest io_session, const WTF::String& reattach_state) {
  GetForwardingInterface()->AttachDevToolsSession(std::move(host), std::move(session), std::move(io_session), std::move(reattach_state));
}
DevToolsAgentAsyncWaiter::DevToolsAgentAsyncWaiter(
    DevToolsAgent* proxy) : proxy_(proxy) {}

DevToolsAgentAsyncWaiter::~DevToolsAgentAsyncWaiter() = default;


const char DevToolsSession::Name_[] = "blink::mojom::DevToolsSession";

DevToolsSessionProxy::DevToolsSessionProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void DevToolsSessionProxy::DispatchProtocolMessage(
    int32_t in_call_id, const WTF::String& in_method, const WTF::String& in_message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "blink::mojom::DevToolsSession::DispatchProtocolMessage");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kDevToolsSession_DispatchProtocolMessage_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::blink::mojom::internal::DevToolsSession_DispatchProtocolMessage_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->call_id = in_call_id;
  typename decltype(params->method)::BaseType::BufferWriter
      method_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_method, buffer, &method_writer, &serialization_context);
  params->method.Set(
      method_writer.is_null() ? nullptr : method_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->method.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null method in DevToolsSession.DispatchProtocolMessage request");
  typename decltype(params->message)::BaseType::BufferWriter
      message_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_message, buffer, &message_writer, &serialization_context);
  params->message.Set(
      message_writer.is_null() ? nullptr : message_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in DevToolsSession.DispatchProtocolMessage request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void DevToolsSessionProxy::InspectElement(
    const ::blink::WebPoint& in_point) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "blink::mojom::DevToolsSession::InspectElement");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kDevToolsSession_InspectElement_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::blink::mojom::internal::DevToolsSession_InspectElement_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->point)::BaseType::BufferWriter
      point_writer;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_point, buffer, &point_writer, &serialization_context);
  params->point.Set(
      point_writer.is_null() ? nullptr : point_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->point.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null point in DevToolsSession.InspectElement request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool DevToolsSessionStubDispatch::Accept(
    DevToolsSession* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDevToolsSession_DispatchProtocolMessage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSession::DispatchProtocolMessage",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::DevToolsSession_DispatchProtocolMessage_Params_Data* params =
          reinterpret_cast<internal::DevToolsSession_DispatchProtocolMessage_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_call_id{};
      WTF::String p_method{};
      WTF::String p_message{};
      DevToolsSession_DispatchProtocolMessage_ParamsDataView input_data_view(params, &serialization_context);
      
      p_call_id = input_data_view.call_id();
      if (!input_data_view.ReadMethod(&p_method))
        success = false;
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DevToolsSession::DispatchProtocolMessage deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DispatchProtocolMessage(
std::move(p_call_id), 
std::move(p_method), 
std::move(p_message));
      return true;
    }
    case internal::kDevToolsSession_InspectElement_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSession::InspectElement",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::DevToolsSession_InspectElement_Params_Data* params =
          reinterpret_cast<internal::DevToolsSession_InspectElement_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::blink::WebPoint p_point{};
      DevToolsSession_InspectElement_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPoint(&p_point))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DevToolsSession::InspectElement deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->InspectElement(
std::move(p_point));
      return true;
    }
  }
  return false;
}

// static
bool DevToolsSessionStubDispatch::AcceptWithResponder(
    DevToolsSession* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDevToolsSession_DispatchProtocolMessage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSession::DispatchProtocolMessage",
               "message", message->name());
#endif
      break;
    }
    case internal::kDevToolsSession_InspectElement_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSession::InspectElement",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool DevToolsSessionRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DevToolsSession RequestValidator");

  switch (message->header()->name) {
    case internal::kDevToolsSession_DispatchProtocolMessage_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DevToolsSession_DispatchProtocolMessage_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kDevToolsSession_InspectElement_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DevToolsSession_InspectElement_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void DevToolsSessionInterceptorForTesting::DispatchProtocolMessage(int32_t call_id, const WTF::String& method, const WTF::String& message) {
  GetForwardingInterface()->DispatchProtocolMessage(std::move(call_id), std::move(method), std::move(message));
}
void DevToolsSessionInterceptorForTesting::InspectElement(const ::blink::WebPoint& point) {
  GetForwardingInterface()->InspectElement(std::move(point));
}
DevToolsSessionAsyncWaiter::DevToolsSessionAsyncWaiter(
    DevToolsSession* proxy) : proxy_(proxy) {}

DevToolsSessionAsyncWaiter::~DevToolsSessionAsyncWaiter() = default;


const char DevToolsSessionHost::Name_[] = "blink::mojom::DevToolsSessionHost";

DevToolsSessionHostProxy::DevToolsSessionHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void DevToolsSessionHostProxy::DispatchProtocolMessage(
    DevToolsMessageChunkPtr in_chunk) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "blink::mojom::DevToolsSessionHost::DispatchProtocolMessage");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kDevToolsSessionHost_DispatchProtocolMessage_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::blink::mojom::internal::DevToolsSessionHost_DispatchProtocolMessage_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->chunk)::BaseType::BufferWriter
      chunk_writer;
  mojo::internal::Serialize<::blink::mojom::DevToolsMessageChunkDataView>(
      in_chunk, buffer, &chunk_writer, &serialization_context);
  params->chunk.Set(
      chunk_writer.is_null() ? nullptr : chunk_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->chunk.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null chunk in DevToolsSessionHost.DispatchProtocolMessage request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool DevToolsSessionHostStubDispatch::Accept(
    DevToolsSessionHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDevToolsSessionHost_DispatchProtocolMessage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSessionHost::DispatchProtocolMessage",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::DevToolsSessionHost_DispatchProtocolMessage_Params_Data* params =
          reinterpret_cast<internal::DevToolsSessionHost_DispatchProtocolMessage_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DevToolsMessageChunkPtr p_chunk{};
      DevToolsSessionHost_DispatchProtocolMessage_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadChunk(&p_chunk))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DevToolsSessionHost::DispatchProtocolMessage deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DispatchProtocolMessage(
std::move(p_chunk));
      return true;
    }
  }
  return false;
}

// static
bool DevToolsSessionHostStubDispatch::AcceptWithResponder(
    DevToolsSessionHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDevToolsSessionHost_DispatchProtocolMessage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)blink::mojom::DevToolsSessionHost::DispatchProtocolMessage",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool DevToolsSessionHostRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DevToolsSessionHost RequestValidator");

  switch (message->header()->name) {
    case internal::kDevToolsSessionHost_DispatchProtocolMessage_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DevToolsSessionHost_DispatchProtocolMessage_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void DevToolsSessionHostInterceptorForTesting::DispatchProtocolMessage(DevToolsMessageChunkPtr chunk) {
  GetForwardingInterface()->DispatchProtocolMessage(std::move(chunk));
}
DevToolsSessionHostAsyncWaiter::DevToolsSessionHostAsyncWaiter(
    DevToolsSessionHost* proxy) : proxy_(proxy) {}

DevToolsSessionHostAsyncWaiter::~DevToolsSessionHostAsyncWaiter() = default;


}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {


// static
bool StructTraits<::blink::mojom::blink::DevToolsMessageChunk::DataView, ::blink::mojom::blink::DevToolsMessageChunkPtr>::Read(
    ::blink::mojom::blink::DevToolsMessageChunk::DataView input,
    ::blink::mojom::blink::DevToolsMessageChunkPtr* output) {
  bool success = true;
  ::blink::mojom::blink::DevToolsMessageChunkPtr result(::blink::mojom::blink::DevToolsMessageChunk::New());
  
      result->is_first = input.is_first();
      result->is_last = input.is_last();
      if (!input.ReadData(&result->data))
        success = false;
      result->call_id = input.call_id();
      if (!input.ReadPostState(&result->post_state))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif