// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_WEB_DEVTOOLS_AGENT_MOJOM_BLINK_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_WEB_DEVTOOLS_AGENT_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_features.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/web/devtools_agent.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace blink {
namespace mojom {
namespace blink {
class DevToolsAgent;
using DevToolsAgentPtr = mojo::InterfacePtr<DevToolsAgent>;
using DevToolsAgentPtrInfo = mojo::InterfacePtrInfo<DevToolsAgent>;
using ThreadSafeDevToolsAgentPtr =
    mojo::ThreadSafeInterfacePtr<DevToolsAgent>;
using DevToolsAgentRequest = mojo::InterfaceRequest<DevToolsAgent>;
using DevToolsAgentAssociatedPtr =
    mojo::AssociatedInterfacePtr<DevToolsAgent>;
using ThreadSafeDevToolsAgentAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DevToolsAgent>;
using DevToolsAgentAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DevToolsAgent>;
using DevToolsAgentAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DevToolsAgent>;

class DevToolsSession;
using DevToolsSessionPtr = mojo::InterfacePtr<DevToolsSession>;
using DevToolsSessionPtrInfo = mojo::InterfacePtrInfo<DevToolsSession>;
using ThreadSafeDevToolsSessionPtr =
    mojo::ThreadSafeInterfacePtr<DevToolsSession>;
using DevToolsSessionRequest = mojo::InterfaceRequest<DevToolsSession>;
using DevToolsSessionAssociatedPtr =
    mojo::AssociatedInterfacePtr<DevToolsSession>;
using ThreadSafeDevToolsSessionAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DevToolsSession>;
using DevToolsSessionAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DevToolsSession>;
using DevToolsSessionAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DevToolsSession>;

class DevToolsSessionHost;
using DevToolsSessionHostPtr = mojo::InterfacePtr<DevToolsSessionHost>;
using DevToolsSessionHostPtrInfo = mojo::InterfacePtrInfo<DevToolsSessionHost>;
using ThreadSafeDevToolsSessionHostPtr =
    mojo::ThreadSafeInterfacePtr<DevToolsSessionHost>;
using DevToolsSessionHostRequest = mojo::InterfaceRequest<DevToolsSessionHost>;
using DevToolsSessionHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<DevToolsSessionHost>;
using ThreadSafeDevToolsSessionHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DevToolsSessionHost>;
using DevToolsSessionHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DevToolsSessionHost>;
using DevToolsSessionHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DevToolsSessionHost>;

class DevToolsMessageChunk;
using DevToolsMessageChunkPtr = mojo::StructPtr<DevToolsMessageChunk>;


class DevToolsAgentProxy;

template <typename ImplRefTraits>
class DevToolsAgentStub;

class DevToolsAgentRequestValidator;

class BLINK_PLATFORM_EXPORT DevToolsAgent
    : public DevToolsAgentInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DevToolsAgentProxy;

  template <typename ImplRefTraits>
  using Stub_ = DevToolsAgentStub<ImplRefTraits>;

  using RequestValidator_ = DevToolsAgentRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAttachDevToolsSessionMinVersion = 0,
  };
  virtual ~DevToolsAgent() {}

  virtual void AttachDevToolsSession(DevToolsSessionHostAssociatedPtrInfo host, DevToolsSessionAssociatedRequest session, DevToolsSessionRequest io_session, const WTF::String& reattach_state) = 0;
};
class BLINK_PLATFORM_EXPORT DevToolsAgentInterceptorForTesting : public DevToolsAgent {
  virtual DevToolsAgent* GetForwardingInterface() = 0;
  void AttachDevToolsSession(DevToolsSessionHostAssociatedPtrInfo host, DevToolsSessionAssociatedRequest session, DevToolsSessionRequest io_session, const WTF::String& reattach_state) override;
};
class BLINK_PLATFORM_EXPORT DevToolsAgentAsyncWaiter {
 public:
  explicit DevToolsAgentAsyncWaiter(DevToolsAgent* proxy);
  ~DevToolsAgentAsyncWaiter();

 private:
  DevToolsAgent* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DevToolsAgentAsyncWaiter);
};

class DevToolsSessionProxy;

template <typename ImplRefTraits>
class DevToolsSessionStub;

class DevToolsSessionRequestValidator;

class BLINK_PLATFORM_EXPORT DevToolsSession
    : public DevToolsSessionInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DevToolsSessionProxy;

  template <typename ImplRefTraits>
  using Stub_ = DevToolsSessionStub<ImplRefTraits>;

  using RequestValidator_ = DevToolsSessionRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kDispatchProtocolMessageMinVersion = 0,
    kInspectElementMinVersion = 0,
  };
  virtual ~DevToolsSession() {}

  virtual void DispatchProtocolMessage(int32_t call_id, const WTF::String& method, const WTF::String& message) = 0;

  virtual void InspectElement(const ::blink::WebPoint& point) = 0;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionInterceptorForTesting : public DevToolsSession {
  virtual DevToolsSession* GetForwardingInterface() = 0;
  void DispatchProtocolMessage(int32_t call_id, const WTF::String& method, const WTF::String& message) override;
  void InspectElement(const ::blink::WebPoint& point) override;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionAsyncWaiter {
 public:
  explicit DevToolsSessionAsyncWaiter(DevToolsSession* proxy);
  ~DevToolsSessionAsyncWaiter();

 private:
  DevToolsSession* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DevToolsSessionAsyncWaiter);
};

class DevToolsSessionHostProxy;

template <typename ImplRefTraits>
class DevToolsSessionHostStub;

class DevToolsSessionHostRequestValidator;

class BLINK_PLATFORM_EXPORT DevToolsSessionHost
    : public DevToolsSessionHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DevToolsSessionHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = DevToolsSessionHostStub<ImplRefTraits>;

  using RequestValidator_ = DevToolsSessionHostRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kDispatchProtocolMessageMinVersion = 0,
  };
  virtual ~DevToolsSessionHost() {}

  virtual void DispatchProtocolMessage(DevToolsMessageChunkPtr chunk) = 0;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionHostInterceptorForTesting : public DevToolsSessionHost {
  virtual DevToolsSessionHost* GetForwardingInterface() = 0;
  void DispatchProtocolMessage(DevToolsMessageChunkPtr chunk) override;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionHostAsyncWaiter {
 public:
  explicit DevToolsSessionHostAsyncWaiter(DevToolsSessionHost* proxy);
  ~DevToolsSessionHostAsyncWaiter();

 private:
  DevToolsSessionHost* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DevToolsSessionHostAsyncWaiter);
};

class BLINK_PLATFORM_EXPORT DevToolsAgentProxy
    : public DevToolsAgent {
 public:
  explicit DevToolsAgentProxy(mojo::MessageReceiverWithResponder* receiver);
  void AttachDevToolsSession(DevToolsSessionHostAssociatedPtrInfo host, DevToolsSessionAssociatedRequest session, DevToolsSessionRequest io_session, const WTF::String& reattach_state) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT DevToolsSessionProxy
    : public DevToolsSession {
 public:
  explicit DevToolsSessionProxy(mojo::MessageReceiverWithResponder* receiver);
  void DispatchProtocolMessage(int32_t call_id, const WTF::String& method, const WTF::String& message) final;
  void InspectElement(const ::blink::WebPoint& point) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT DevToolsSessionHostProxy
    : public DevToolsSessionHost {
 public:
  explicit DevToolsSessionHostProxy(mojo::MessageReceiverWithResponder* receiver);
  void DispatchProtocolMessage(DevToolsMessageChunkPtr chunk) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT DevToolsAgentStubDispatch {
 public:
  static bool Accept(DevToolsAgent* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DevToolsAgent* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DevToolsAgent>>
class DevToolsAgentStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DevToolsAgentStub() {}
  ~DevToolsAgentStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsAgentStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsAgentStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionStubDispatch {
 public:
  static bool Accept(DevToolsSession* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DevToolsSession* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DevToolsSession>>
class DevToolsSessionStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DevToolsSessionStub() {}
  ~DevToolsSessionStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsSessionStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsSessionStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionHostStubDispatch {
 public:
  static bool Accept(DevToolsSessionHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DevToolsSessionHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DevToolsSessionHost>>
class DevToolsSessionHostStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DevToolsSessionHostStub() {}
  ~DevToolsSessionHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsSessionHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DevToolsSessionHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT DevToolsAgentRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT DevToolsSessionHostRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class BLINK_PLATFORM_EXPORT DevToolsMessageChunk {
 public:
  using DataView = DevToolsMessageChunkDataView;
  using Data_ = internal::DevToolsMessageChunk_Data;

  template <typename... Args>
  static DevToolsMessageChunkPtr New(Args&&... args) {
    return DevToolsMessageChunkPtr(
        WTF::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DevToolsMessageChunkPtr From(const U& u) {
    return mojo::TypeConverter<DevToolsMessageChunkPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DevToolsMessageChunk>::Convert(*this);
  }


  DevToolsMessageChunk();

  DevToolsMessageChunk(
      bool is_first,
      bool is_last,
      const WTF::String& data,
      int32_t call_id,
      const WTF::String& post_state);

  ~DevToolsMessageChunk();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DevToolsMessageChunkPtr>
  DevToolsMessageChunkPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DevToolsMessageChunk>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        DevToolsMessageChunk::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        DevToolsMessageChunk::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::DevToolsMessageChunk_UnserializedMessageContext<
            UserType, DevToolsMessageChunk::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<DevToolsMessageChunk::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return DevToolsMessageChunk::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::DevToolsMessageChunk_UnserializedMessageContext<
            UserType, DevToolsMessageChunk::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<DevToolsMessageChunk::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  bool is_first;
  bool is_last;
  WTF::String data;
  int32_t call_id;
  WTF::String post_state;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
DevToolsMessageChunkPtr DevToolsMessageChunk::Clone() const {
  return New(
      mojo::Clone(is_first),
      mojo::Clone(is_last),
      mojo::Clone(data),
      mojo::Clone(call_id),
      mojo::Clone(post_state)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DevToolsMessageChunk>::value>::type*>
bool DevToolsMessageChunk::Equals(const T& other) const {
  if (!mojo::Equals(this->is_first, other.is_first))
    return false;
  if (!mojo::Equals(this->is_last, other.is_last))
    return false;
  if (!mojo::Equals(this->data, other.data))
    return false;
  if (!mojo::Equals(this->call_id, other.call_id))
    return false;
  if (!mojo::Equals(this->post_state, other.post_state))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::blink::mojom::blink::DevToolsMessageChunk::DataView,
                                         ::blink::mojom::blink::DevToolsMessageChunkPtr> {
  static bool IsNull(const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::blink::DevToolsMessageChunkPtr* output) { output->reset(); }

  static decltype(::blink::mojom::blink::DevToolsMessageChunk::is_first) is_first(
      const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) {
    return input->is_first;
  }

  static decltype(::blink::mojom::blink::DevToolsMessageChunk::is_last) is_last(
      const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) {
    return input->is_last;
  }

  static const decltype(::blink::mojom::blink::DevToolsMessageChunk::data)& data(
      const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) {
    return input->data;
  }

  static decltype(::blink::mojom::blink::DevToolsMessageChunk::call_id) call_id(
      const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) {
    return input->call_id;
  }

  static const decltype(::blink::mojom::blink::DevToolsMessageChunk::post_state)& post_state(
      const ::blink::mojom::blink::DevToolsMessageChunkPtr& input) {
    return input->post_state;
  }

  static bool Read(::blink::mojom::blink::DevToolsMessageChunk::DataView input, ::blink::mojom::blink::DevToolsMessageChunkPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_WEB_DEVTOOLS_AGENT_MOJOM_BLINK_H_